#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName=CommonUsage


Function load_p3b(fileName)
    String fileName
    print fileName

    SVAR folderPath=root:fileIO:folderPath
    Variable refNum
    Open/R/Z=2 refNum as folderPath+fileName
    Variable tagNum, tagLen
    FBinRead/F=3/B=3/U refNum, tagNum
    printf "number of tag: %d\r", tagNum

    Variable i,j,k
    String tagName
    Variable dataType, dataLen, data
    Variable pixType
    NVAR dimX = root:IP_consts:dimX
    NVAR dimY = root:IP_consts:dimY
    for (i=0;i<tagNum;i+=1)
        FBinRead/F=3/B=3/U refNum, tagLen
        printf "\r length of tag: %d characters\r", tagLen
        tagName = PadString("", tagLen,0)
        FBinRead refNum, tagName
        print "the name of the tag: " + tagName
        FBinRead/F=3/B=3/U refNum, dataType
        printf "data type is %d, ", dataType
        FBinRead/F=3/B=3/U refNum, dataLen
        
        printf "data length is %d\r", dataLen
        strswitch (tagName)
            case "DIMX":
                FBinRead/F=3/B=3/U refNum, dimX
                printf "dimension in x: %d\r", dimX
                break
            case "DIMY":
                FBinRead/F=3/B=3/U refNum, dimY
                printf "dimension in y: %d\r", dimY
                break
            case "PIXTYPE":
                FBinRead/F=2/B=3/U refNum, pixType 
                printf "pixel type for the image: %d\r", pixType
                break
            case "DATA":
                if (dimX*dimY != dataLen)
		    doAlert 0,"dimX*dimY != dataLen"
                endif
                Make/W/U/O/N=(dimX,dimY) image_p3b
                FBinRead/F=2/B=3/U refNum, image_p3b
                Rename image_p3b $fileName
                Close refNum
                return 0 // when data is load, return
                break
            default:
                break
        endswitch
    endfor

    return 0
End

//==================================================================

Function selectAll()

    NVAR dimX = root:IP_consts:dimX
    NVAR dimY = root:IP_consts:dimY
    NVAR imageDType = root:IP_consts:imageDType

    SetDataFolder root:images:

    KillWaves/Z M_ROIMask
    switch (imageDType)
        case 72: // 8bit unsigned int
            Make/B/U/O/N=(dimX,dimY) Mm_ROIMask
            Rename Mm_ROIMask, M_ROIMask
        break
        case 80: // 16bit unsigned int
            Make/W/U/O/N=(dimX,dimY) Mn_ROIMask
            Rename Mn_ROIMask, M_ROIMask
        break
        default:
            doAlert 0, "the image datatype is too interesting, i.e. "+num2str(imageDType)+ ". Stopped in CommonUsage#selectAll()."
            return 1
        break
    endswitch
    
    setROI#clearROI()

    // draw ROI edge as feedback
    SetDrawLayer/K ProgFront
    Duplicate/O M_ROIMask, M_ImageEdges
    M_ImageEdges = 64
    M_ImageEdges[dimX-1][] = 0
    M_ImageEdges[0][] = 0
    M_ImageEdges[][0] = 0
    M_ImageEdges[][dimY-1] = 0
    AppendImage M_ImageEdges
    ModifyImage M_ImageEdges explicit=0,ctab= {*,2,Rainbow,0};DelayUpdate
    ModifyImage M_ImageEdges minRGB=0,maxRGB=NaN

    file_IO#updateImageList()            

    return 0
End

//==================================================================

Function checkImageDatatype(imageName)
    String imageName

    if (Exists(imageName)==0)
        print imageName +" does not exist"
        return 0
    endif

    Wave img_check = $imageName
    String info = WaveInfo(img_check,0)

    return NumberByKey("NUMTYPE", info)
    
End

//==================================================================

Function updateImageDatatype(imageName)
    String imageName

    NVAR imageDType = root:IP_consts:imageDType
    imageDType = checkImageDatatype(imageName)
    print "image data type: ", imageDType

    NVAR imageDType_disp = root:IP_consts:imageDType_disp
    switch (imageDType)
        case 72: // 8bit unsigned int
            imageDType_disp = 1
        break
        case 80: // 16 bit unsigned int
            imageDType_disp = 3
        break
        default: 
            imageDType_disp = 2
        break
    endswitch 

    return 0
End

//==================================================================

Function load_xim(fileName)
    String fileName
    print fileName

    SVAR folderPath=root:fileIO:folderPath
    LoadWave/J/M/N=wave/O/K=0 folderPath+fileName
    Redimension/U/W wave0
    Rename wave0 $fileName

    return 0
End

//==================================================================
Function mouseReadHook (s)
    Struct WMWinHookStruct &s

    //String infoStr
    //String event= StringByKey("EVENT",infoStr)
    NVAR imageX = root:IP_consts:imageX 
    NVAR imageY = root:IP_consts:imageY
    NVAR imageZ = root:IP_consts:imageZ

    String topGraphName

    topGraphName = WinName(0, 1)
    if (strlen(topGraphName) == 0)
        DoAlert 0, "There are no graphs."
        return -1
    endif

    string ImageNameString=stringfromlist (0, ImageNameList(topGraphName, ";" ))
    wave wImage=ImageNameToWaveRef(topGraphName,ImageNameString)
//    print ImageNameString + " found in window " + topGraphName
    switch (s.eventCode)
        case 4: // mouse moved
            imageX = round(AxisValFromPixel(topGraphName, "bottom", s.mouseLoc.h))
            imageY = round(AxisValFromPixel(topGraphName, "left", s.mouseLoc.v ))
            if (cmpstr(topGraphName,"stack_window")==0)
                NVAR gThePlane = root:IP_consts:gThePlane
                imageZ = wImage[imageX][imageY][gThePlane]
            else
                imageZ = wImage[imageX][imageY]
            endif
        break
        case 5: // mouse up
            NVAR MWand_on = root:IP_consts:MWand_on
            if (MWand_on == 1)
                MagicWand#MWand_select()
            endif
        break
        case 11: // key stroke
            print s.keycode
            switch (s.keycode)
                case 28: // the 4 arrow keys
                case 29: // left right up down
                case 30: // respectively
                case 31: // 
                    if (Exists("root:images:M_ImageEdges"))
                        CommonUsage#moveROI_keystroke(s.keycode)
                    endif
                case 44: // ,
                case 46: // .
                    DoWindow stack_window
                    if (V_flag==1) // stack_window does exist
                        CommonUsage#movieFlip(s.keycode)
                    endif
                break
                case 61: // =
                    CommonUsage#play_stop_movie()
                break
                case 65: // A
                case 97: // a
                    if (cmpstr(topGraphName,"stack_window")==0)
                        CommonUsage#selectAll()
                    endif
                break
            endswitch
        break
    endswitch
    // 1 means the the events are handled, but the marquee is gone
    // with 0, one needs to close the command window
    return 0 
End

//==================================================================

Function moveROI_keystroke(keycode)
    Variable keycode

    SetDataFolder root:images:

    setROI#clearROI()
    WAVE M_ROIMask = root:images:M_ROIMask
    NVAR dimX = root:IP_consts:dimX
    NVAR dimY = root:IP_consts:dimY
    Duplicate/O M_ROIMask, M_tmp

    Variable i,j
    switch (keycode)
        case 28: // left
            M_tmp[][] = M_ROIMask[p+1][q]
            M_tmp[dimX-1][] = 1
        break
        case 29: // right
            M_tmp[][] = M_ROIMask[p-1][q]
            M_tmp[0][] = 1
        break
        case 30: // up
            M_tmp[][] = M_ROIMask[p][q+1]
            M_tmp[][dimY-1] = 1
        break
        case 31: // down
            M_tmp[][] = M_ROIMask[p][q-1]
            M_tmp[][0] = 1
        break
    endswitch
    Duplicate/O M_tmp,M_ROIMask
    KillWaves M_tmp

    // draw ROI edge as feedback
    SetDrawLayer/K ProgFront
    ImageEdgeDetection/N/M=2 Sobel, M_ROIMask
    AppendImage M_ImageEdges
    ModifyImage M_ImageEdges explicit=0,ctab= {*,2,Rainbow,0};DelayUpdate
    ModifyImage M_ImageEdges minRGB=0,maxRGB=NaN

    return 0
End

//==================================================================

Function movieFlip(keycode)
    Variable keycode 

    NVAR gThePlane = root:IP_consts:gThePlane
    NVAR numImages = root:IP_consts:numImages
    WAVE M_Stack = root:images:M_Stack
    
    switch (keycode)
        case 44: // ,
            if (gThePlane > 1)
                gThePlane -= 1
            endif
        break
        case 46: // .
            if (gThePlane < numImages-1)
                gThePlane += 1
            endif
        break
    endswitch

    ModifyImage/W=stack_window M_Stack,plane=gThePlane

    return 0
End

//==================================================================

Function load_dat()
    NVAR energy_steps = root:IP_consts:energy_steps
    NVAR numImages = root:IP_consts:numImages
    //SVAR folderPath=root:fileIO:folderPath
    // it may be a bug to use diskFolderPath

    String dat_file_name=indexedfile(diskFolderPath,0,".dat")
    if (strlen(dat_file_name)<=0)
        doAlert 0,"Could not find .dat file!"
	return 1
    endif

    SetDataFolder root:IP_consts:
    //LoadWave/G/M/D/P=diskFolderPath/B="N=energy;"/O/L={0,0,0,0,1} dat_file_name
    LoadWave/G/M/D/P=diskFolderPath/B="N=energy;"/O/L={0,0,0,0,1} 
    if (V_flag == 0) // nothing is loaded
        return -2
    endif

    Sort energy, energy // make sure energy is increasing
    Duplicate/O energy, energy_r
    reverse energy_r
    energy_steps = DimSize(energy,0)
    SetDataFolder root:images

    if (numImages != energy_steps)
	doAlert 0,".dat file is not consistent with the stack file!"
	return -1
    else
        print "the .dat is consistent with stack."
        return 0
    endif
End

//==================================================================

Function displayImage(imgName, winName)
    String imgName
    String winName

    NVAR imageColorReverse = root:IP_consts:imageColorReverse
    ControlInfo/W=fileIO ReverseColor
    imageColorReverse = V_Value

    Variable imageBitInfo
    switch (CommonUsage#checkImageDatatype(imgName))
        case 72: // 8 bit unsigned int
            imageBitInfo = 255
        break
        case 80: // 16 bit unsigned int
            imageBitInfo = 65535
        break
        default:
            doAlert 0, "caution! the displayed stack is neither 8 bit nor 16 bit!"
            WAVE M_tmp = $imgName
            ImageStats/M=1 M_tmp
            imageBitInfo = V_max
        break
    endswitch

    SetDataFolder root:images:
    if (cmpstr(winName, "view") != 0) // allow multiple view
        DoWindow/K $winName
    endif
    NewImage/K=1/N=$winName $imgName
    ModifyImage/W=$winName $imgName ctab={0,imageBitInfo,Grays,imageColorReverse}
    
    setwindow $winName hook(keypress)=CommonUsage#mouseReadHook
    return 0
End

//==================================================================

Function saveImageAs(imgName, fileName)
    String imgName
    String fileName

    print "save image"
    SVAR folderPath = root:fileIO:folderPath
    NewPath/O path, folderPath

    SetDataFolder root:images:

    switch (CommonUsage#checkImageDatatype(imgName))
        case 72: // 8 bit unsigned int
            if (cmpstr(imgName,"M_Stack") ==0)
                ImageSave/D=40/P=path/S/I/T="TIFF" M_Stack as fileName+".tif"
            else
                ImageSave/D=40/P=path/I/T="TIFF" $imgName as fileName
            endif
        break
        case 80: // 16 bit unsigned int
            if (cmpstr(imgName,"M_Stack") ==0)
                ImageSave/D=16/P=path/S/I/T="TIFF" M_Stack as fileName+".tif"
            else
                ImageSave/D=16/P=path/I/T="TIFF" $imgName as fileName
            endif
        break
        default:
            doAlert 0, "caution! the stack to be saved is neither 8 bit nor 16 bit!"
            if (cmpstr(imgName,"M_Stack") ==0)
                ImageSave/P=path/S/I/T="TIFF" M_Stack as fileName+".tif"
            else
                ImageSave/P=path/I/T="TIFF" $imgName as fileName
            endif
        break
    endswitch
    
    return 0
End

//==================================================================

Function play_stop_movie()

    NVAR isPlaying = root:IP_consts:isPlaying

    if (Exists("root:images:M_Stack")==0)
        return 1;
    endif
    doWindow stack_window
    if (V_flag==0) // stack_window does not exist
        return 2
    endif 

    if (isPlaying)
        CtrlBackground stop
        isPlaying = 0
    else
        CtrlBackground period=1,start
        isPlaying = 1
    endif
	
    return 0
End
//==================================================================

Function/S getImageSelection()

    WAVE/T imageWaveList = root:IP_consts:imageWaveList
    ControlInfo/W=fileIO imageListBox
    String imageName = imageWaveList[V_value];
    print "row selection: ", V_value, imageName
    print "image exists?",Exists(imageName)
    if (GetRTError(1))
        imageName = ""
    endif

    return imageName
End


