#pragma rtGlobals=1		// Use modern global access method.

Function fitspectra(w,xx)
	wave w
	variable xx
	
	variable s=0
	variable nPeaks=DimSize(w,0)/3

	Variable i=0
	Variable pk=0
	do
		pk = i*3
		s += GaussPeak(w[pk],w[pk+1],w[pk+2],xx)
		i += 1
	while (i < nPeaks)

   	return s
end

//          ---------- Peak model -------------------
Function GaussPeak(ampl,pos,width,r)
	variable ampl,pos,width,r

	variable q
	q=ampl*exp(-4*ln(2)*((pos-r)/width)^2)
	return q
end
//------------------------------------------

// The following function is used to update the display of individual peaks.  It has been made to
//  update the peaks automatically through a dependency formula.
Function AppendPeaks(cwave)
	Wave cwave		//Peak coefficient wave
	
	Variable nPeaks = DimSize(cwave,0)/3
	Variable i=0
	Variable pk=0
	Wave cW=Colors
	
	String PkName=""
	
	do
		pk = 3*i
		PkName = "Peak_"+num2istr(i+1)
		Make/O/N=100 $PkName
		Wave w=$PkName
		GetAxis /Q bottom
		SetScale/I x,V_min, V_max, w
		w = GaussPeak(cwave[pk],cwave[pk+1],cwave[pk+2],x)
		CheckDisplayed w
		if (V_flag == 0)
			AppendToGraph w
		endif
		ModifyGraph rgb($PkName)=(cW[i][0],cW[i][1],cW[i][2])
		i += 1
	while (i<nPeaks)
end	

Window PeakFitWithConstraint() : Graph
	PauseUpdate; Silent 1		// building window...
	Display /W=(8,45,511,390) lipox13 vs x_as as "Peak fit with Constraints"
	AppendToGraph Peak_1,Peak_2,Peak_3,Peak_4,Peak_5,Peak_6
	ModifyGraph gbRGB=(39321,39321,39321)
	ModifyGraph mode(lipox13)=3
	ModifyGraph marker(lipox13)=8
	ModifyGraph rgb(lipox13)=(0,0,0),rgb(Peak_2)=(0,0,65535),rgb(Peak_3)=(65535,65535,0)
	ModifyGraph rgb(Peak_4)=(65535,43690,0),rgb(Peak_5)=(65535,0,65535),rgb(Peak_6)=(0,65535,65535)
	ModifyGraph msize(lipox13)=2
	ModifyGraph nticks(bottom)=30
	ModifyGraph minor(bottom)=1
	ModifyGraph standoff=0
	ModifyGraph axOffset(bottom)=0.2
	ModifyGraph lblPos(left)=64
	ModifyGraph manTick(bottom)={1610,20,0,0},manMinor(bottom)={9,5}
	ShowTools
	ControlBar 53
	Button TableButton,pos={387,19},size={59,20},proc=TableButtonProc,title="Table..."
	Button FitButton,pos={253,19},size={63,20},proc=FitButtonProc,title="Do Fit"
	CheckBox ConstraintCheck,pos={70,9},size={125,20},title="Use Constraints",value=0
	CheckBox BadGuessCheck,pos={70,25},size={163,20},proc=BadGuessCheckProc,title="Lousy Initial Guesses",value=1
EndMacro

Function TableButtonProc(ctrlName) : ButtonControl
	String ctrlName

	if (WinType("FitTable") != 2)
		Execute "FitTable()"
	else
		DoWindow/F FitTable
	endif
End

Proc FitTable()

	Edit/W=(5,42,405,399) Kn,Description,PeakParams,'Bad Guesses',Constraints as "Fit Parameters"
	ModifyTable size=9,width(Point)=0,style(Kn)=1,alignment(Kn)=1,width(Kn)=38,width('Bad Guesses')=84
EndMacro

Function BadGuessCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	Wave BG='Bad Guesses'
	Wave PP=PeakParams
	Wave GG=GoodGuesses
	Wave fit_lipox13=fit_lipox13

	if (checked)
		PP = BG
	else
		PP = GG
	endif
	fit_lipox13= fitspectra(PP,x)
End

Function FitButtonProc(ctrlName) : ButtonControl
	String ctrlName
	
	ControlInfo ConstraintCheck
	Variable UseConstraints=V_value
	Variable V_FitMaxIters
	Variable V_FitNumIters
	
	ControlInfo BadGuessCheck
	BadGuessCheckProc("", V_value)
	
	Wave/T Constraints=Constraints
	
	if (UseConstraints)
		V_FitMaxIters=500
		print " FuncFit fitspectra lipox13 PeakParams /X=x_as /D /R /C=Constraints"
		FuncFit fitspectra PeakParams lipox13 /X=x_as /D /R /C=Constraints
	else
		V_FitMaxIters=500
		print " FuncFit fitspectra lipox13 PeakParams /X=x_as /D /R"
		FuncFit fitspectra PeakParams lipox13 /X=x_as /D /R
	endif
	print "  Took",V_FitNumIters, " iterations"
End
