#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName=file_IO

//==================================================================

Static Function loadImagesInFolder()

    ControlInfo/W=fileIO fileTypeMenu
    String fileType=S_Value
    String suffix="????"
    strswitch(fileType)
	case "TIFF":
	    suffix=".tif"
	    break
	case "JPEG":
	    suffix=".jpg"
	    break
	case "PNG":
	    suffix=".png"
	    break
	case "BMP":
	    suffix=".bmp"
            break
        case "P3B":
            suffix=".p3b"
            break
        case "XIM":
            suffix=".xim"
	    break
    endswitch
	
    String listAllFiles=indexedfile(diskFolderPath,-1,suffix)
    if(ItemsInList(listAllFiles)<=0)
	doAlert 0,"Could not find files of the specified type."
	return 0
    endif
	
    String fileName
    SVAR folderPath=root:fileIO:folderPath

    String waveStr,myStr
    Variable renameFlag
	
    ControlInfo/W=fileIO RenameCheck
    renameFlag=V_Value
	
    ControlInfo/W=fileIO StackCheck
    NVAR isStacking = root:IP_consts:isStacking
    isStacking = V_Value
	
    Variable loadAllFlag
    ControlInfo/W=fileIO loadAllCheck
    loadAllFlag=V_Value

    Variable tmp_refNum
    if (loadAllFlag == 0) // load single image
        strswitch(fileType)
            case "P3B":
                Open/D/R/T=".P3B"/P=diskFolderPath tmp_refNum 
                if (V_flag!=-1)
                    CommonUsage#load_p3b(ParseFilePath(0,S_fileName,":",1,0))
                else
                    return V_flag
                endif
            break
            case "XIM":
                Open/D/R/T=".XIM"/P=diskFolderPath tmp_refNum
                if (V_flag!=-1)
                    CommonUsage#load_xim(ParseFilePath(0,S_fileName,":",1,0))
                else
                    return V_flag
                endif
            break
            case "TIFF":
                ImageLoad/O/Q/P=diskFolderPath/T=$fileType 
                WAVE M_tmp = $S_fileName
                switch (CommonUsage#checkImageDatatype(S_fileName))
                    case 72: // 8 bit unsigned int
                        M_tmp = 255 - M_tmp
                    break
                    case 80: // 16 bit unsigned int
                        M_tmp = 65535 - M_tmp
                    break
                    default:
                        doAlert 0, "caution! the loaded is neither 8 bit nor 16 bit!"
                    break
                endswitch
            break
            default:
                ImageLoad/O/Q/P=diskFolderPath/T=$fileType 
            break
        endswitch
        file_IO#updateImageList()            

        return 0
    endif

    // load multiple images
    if(isStacking && renameFlag==0)
	doAlert 0,"Stacking requires a renaming loaded waves sequentially."
	return 0
    endif

    // remove existing stack to save memory
    if (Exists("root:images:M_Stack")==1)
        doWindow/K stack_window
        SVAR oldImg = root:IP_consts:oldImg
        if (cmpstr(oldImg, "M_Stack")==0)
            oldImg = "none"
            KillWindow fileIO#preview
        endif

        killWaves/Z M_Stack
    endif

    String firstWave, imageLoadName
    NVAR numImages=root:IP_consts:numImages
    numImages = 0
    Variable i=0
    Variable stack_max=0,stack_min=65536
    do
	fileName=StringFromList(i,listAllFiles)
	if(strlen(fileName)<=0)
            break
    	endif
        strswitch(fileType)
            case "P3B":
                CommonUsage#load_p3b(fileName)
                imageLoadName = fileName
                break;
            case "XIM":
                CommonUsage#load_xim(fileName)
                imageLoadName = fileName
                break
            default: 
        	ImageLoad/O/Q/P=diskFolderPath/T=$fileType fileName
                imageLoadName = S_waveNames
                break
        endswitch

	if(renameFlag)
            SVAR baseName = root:fileIO:baseName
            waveStr=StringFromList(0,imageLoadName)
    	    sprintf myStr,"%s%04d",baseName,i
	    Rename $waveStr,$myStr
	    if(i==0)
                firstWave=myStr
            endif
	endif
		
        numImages+=1
		i+=1
    while(1)

    NVAR manual_hist = root:IP_consts:manual_hist
    if (isStacking)
        CommonUsage#updateImageDatatype(firstWave)
        NVAR imageDType = root:IP_consts:imageDType
        if(imageDType != 72)
            doAlert 1, "Convert to 8bit stack?"
            if (V_flag == 1)
                manual_hist = 1
            endif
        endif
    endif

    // make a histogram for the future stack
    if (isStacking && manual_hist)
        print "creat a histogram for the unstacked images"
        for (i=0;i<numImages;i+=1)
    	    sprintf myStr,"%s%04d",baseName,i
            if (i==0) // the 1st wave
                Make/D/O/N=256 manual_hist_wave
                SetScale/I x 1,65536,"", manual_hist_wave;DelayUpdate
                SetScale d 1,65536,"", manual_hist_wave
                Histogram/B=2 $myStr, manual_hist_wave
            endif
            Histogram/A $myStr, manual_hist_wave
        endfor
        // get upperbound and lowerbound from histogram
        file_IO#convert_16_8bit("none")
        NVAR lowerbound = root:makeHistogram:lowerbound
        NVAR upperbound = root:makeHistogram:upperbound
        Variable norm = 255/(upperbound-lowerbound)
        NVAR dimX= root:IP_consts:dimX
        NVAR dimY= root:IP_consts:dimY
        WAVE M_tmp = $firstWave
        dimX = DimSize(M_tmp, 0)
        dimY = DimSize(M_tmp, 1)
        for (i=0;i<numImages;i+=1)
    	    sprintf myStr,"%s%04d",baseName,i
            WAVE M_ImagePlane = $myStr
            MatrixOP/O outWave=upperbound*greater(M_ImagePlane, upperbound) +M_ImagePlane*(equal(greater(M_ImagePlane, lowerbound), greater(upperbound,M_ImagePlane)))+ lowerbound*greater(lowerbound,M_ImagePlane)
            MatrixOP/O outWave=(outWave-lowerbound)*norm
            Make/O/B/U/N=(dimX,dimY) tmp_slice
            tmp_slice = outWave
            KillWaves M_ImagePlane
            Rename tmp_slice, $myStr
        endfor
        manual_hist = 0
        KillWaves manual_hist_wave, outWave
    endif

    if (isStacking)
        print "stacking the images"
        ControlInfo/W=fileIO killWavesCheck
        if(V_value==0)
            ImageTransform stackImages $firstWave
        else
            ImageTransform/K stackImages $firstWave
        endif
        
        NVAR dimX= root:IP_consts:dimX
        NVAR dimY= root:IP_consts:dimY
        dimX = DimSize(M_Stack, 0)
        dimY = DimSize(M_Stack, 1)
        CommonUsage#updateImageDatatype("M_Stack")
    endif

    file_IO#updateImageList()            

    return 0
End
//==================================================================

Function convert_16_8bit(imageName)
    String imageName

    NewDataFolder/O/S root:makeHistogram
    String/G imgName = imageName

    if (cmpstr(imageName, "none")==0) // manual histogram is done
        Duplicate/O root:images:manual_hist_wave, W_ImageHist 
    else 
        WAVE img = $("root:images:"+imageName)
        print "calculating histogram for " + imgName
        if (cmpstr(imgName, "M_Stack")==0)
            ImageHistogram/S img
        else
            ImageHistogram img
        endif
    endif

    NVAR dimX= root:IP_consts:dimX
    NVAR dimY= root:IP_consts:dimY
    NVAR numImages=root:IP_consts:numImages

    DoWindow/K histogram_window
    Display/K=1/N=histogram_window/W=(400,100,1200,500) W_ImageHist

    NewDataFolder/O root:WinGlobals
    NewDataFolder/O root:WinGlobals:histogram_window
    String/G root:WinGlobals:histogram_window:S_TraceOffsetInfo= ""
    Variable/G lowerbound, upperbound

    Make/O lowerDrag={-Inf,Inf},upperDrag={-Inf,Inf},xDrag={0, 0}
    AppendToGraph/W=histogram_window lowerDrag,:upperDrag vs xDrag
    ModifyGraph quickdrag(lowerDrag)=1,live(lowerDrag)=1
    ModifyGraph quickdrag(upperDrag)=1,live(upperDrag)=1
    ModifyGraph lSize(upperDrag)=2,lSize(lowerDrag)=2
    ModifyGraph rgb(W_ImageHist)=(0,0,0),rgb(upperDrag)=(1,4,52428)

    ControlBar 26

    ValDisplay upperVar,pos={20,5}, size={150,25},title="upper bound (blue line):"
    ValDisplay upperVar, limits={-Inf,Inf,0},value=#"root:makeHistogram:upperbound"
    ValDisplay lowerVar,pos={200,5}, size={150,25},title="lower bound (red line):"
    ValDisplay lowerVar, limits={-Inf,Inf,0},value=#"root:makeHistogram:lowerbound"
    Button rescaleButton,pos={500,5},size={95,18},proc=file_IO#rescaleProc,title="Rescale"
    Button cancelButton,pos={600,5},size={95,18},proc=file_IO#cancelRescaleProc,title="Cancel"
    
    SetWindow kwTopWin, hook(fun)=histUpdateVarWindowProc

    PauseForUser histogram_window

    return 0
End

//==================================================================

Function histUpdateVarWindowProc(s)
    STRUCT WMWinHookStruct &s

    switch(s.eventCode) 
        case 4: // mouse moved
            file_IO#histUpdateVarFromLines()
        break
    endswitch
    return 0
End

//==================================================================

Function histUpdateVarFromLines()
    String topGraphName
    topGraphName = WinName(0,1)
    SVAR s=root:WinGlobals:$(topGraphName):S_TraceOffsetInfo
    Variable isLower = StrSearch(s,"TNAME:lowerDrag;",0)>0
    Variable isUpper = StrSearch(s,"TNAME:upperDrag;",0)>0
    if ((isLower==0) %& (isUpper==0))
        return -1;  // not valid yet
    endif
    
    String targ = ";XOFFSET:"
    Variable v1 = StrSearch(s,targ,0)
    if (v1 < 0)
        return 0
    endif

    v1 += strlen(targ)
    Variable v2 = StrSearch (s,";",v1)
    Variable xoff = str2num(s[v1,v2-1])
    if (NumType(xoff) != 0)
        return 0;
    endif
    
    NVAR lowerbound = root:makeHistogram:lowerbound
    NVAR upperbound = root:makeHistogram:upperbound
    if (isLower)
        lowerbound = xoff
    else
        upperbound = xoff
    endif

    return 0
End

//==================================================================

Function cancelRescaleProc (ctrlName) : ButtonControl
    String ctrlName

    DoWindow/K histogram_window
    KillWaves/Z root:makeHistogram:W_ImageHist
    SetDataFolder root:images:

    return 0
End

//==================================================================

Function rescaleProc (ctrlName) : ButtonControl
    String ctrlName

    NVAR manual_hist = root:IP_consts:manual_hist
    
    if (manual_hist)
        DoWindow/K histogram_window
        SetDataFolder root:images:
        return 0
    endif 

    NVAR lowerbound = root:makeHistogram:lowerbound
    NVAR upperbound = root:makeHistogram:upperbound
    WAVE M_Stack = root:images:M_Stack
    SVAR imgName = root:makeHistogram:imgName
    NVAR imageDType = root:IP_consts:imageDType
    print "rescaling "+imgName 
    print "lower cut",lowerbound, " upper cut ", upperbound

    SetDataFolder root:images:

    Variable imageBitInfo
    switch (CommonUsage#checkImageDatatype(imgName))
        case 72: // 8 bit unsigned int
            imageBitInfo = 255
        break
        case 80: // 16 bit unsigned int
            imageBitInfo = 65535
        break
        default:
            doAlert 1, "caution! the image to be rescaled is neither 8 bit nor 16 bit! rescale it to 8 bit? no means rescale it to 16 bit."
            if (V_flag == 1)
                imageBitInfo = 255
            elseif (V_flag == 2)
                imageBitInfo = 65535
            endif
        break
    endswitch

    Variable norm 
    // rescale to 8bit, i.e. 0~255, if it's M_Stack
    if (cmpstr(imgName,"M_Stack")==0)
        norm = 255/(upperbound-lowerbound)
    else
        norm = imageBitInfo/(upperbound-lowerbound)
    endif
    print "normalization factor: " , norm

    SetDataFolder root:images

    if (cmpstr(imgName,"M_Stack")==0)
        NVAR dimX = root:IP_consts:dimX
        NVAR dimY = root:IP_consts:dimY
        NVAR numImages = root:IP_consts:numImages
        // this swap is solely to avoid memory overflow
        // it's by no means efficient
        print "disassemble M_Stack"
        Make/O/B/U/N=(dimX,dimY) tmp_slice
        doWindow/K stack_window
        SVAR oldImg = root:IP_consts:oldImg
        if (cmpstr(oldImg, "M_Stack")==0)
            oldImg = "none"
            KillWindow fileIO#preview
        endif

        // removed M_Stack slice by slice, rescale each piece
        // then make a new stack
        String myStr, firstWave
        SVAR baseName = root:fileIO:baseName
        Variable i
        for (i=0;i<numImages;i+=1)
            ImageTransform/P=0 getPlane M_Stack
            if (i==numImages-1)
                KillWaves M_Stack
            else
                ImageTransform/P=0/O removeZplane M_Stack
            endif
            WAVE M_ImagePlane = root:images:M_ImagePlane
            MatrixOP/O outWave = upperbound*greater(M_ImagePlane, upperbound) +M_ImagePlane*(equal(greater(M_ImagePlane, lowerbound), greater(upperbound,M_ImagePlane)))+ lowerbound*greater(lowerbound,M_ImagePlane)
            MatrixOP/O outWave=(outWave-lowerbound)*norm
            tmp_slice = outWave
    	    sprintf myStr,"%s%04d",baseName,i
	    Duplicate tmp_slice,$myStr
	    if(i==0)
                firstWave=myStr
	    endif
print "dealing with ", i, "'th image."
        endfor
        print "make new M_Stack"
        ImageTransform/K stackImages $firstWave
        KillWaves M_ImagePlane

        KillWaves tmp_slice
    else
        WAVE M_tmp = $("root:images:"+imgName)
        MatrixOP/O outWave = upperbound*greater(M_tmp, upperbound) +M_tmp*(equal(greater(M_tmp, lowerbound), greater(upperbound,M_tmp)))+ lowerbound*greater(lowerbound,M_tmp)
        MatrixOP/O outWave=(outWave-lowerbound)*norm
	if (imageDType == 72)
            Redimension/U/B outWave
        elseif (imageDType == 80)
            Redimension/U/W outWave
        endif
        Duplicate/O outWave, $(imgName+"_level")
    endif

    DoWindow/K histogram_window
    KillWaves/Z outWave M_ImagePlane root:makeHistogram:W_ImageHist

    file_IO#updateImageList()

    return 0
End

//==================================================================
Function load_image_panel()

    init()
        DoWindow/K fileIO       // remove existing window
	NewPanel/K=1/W=(0,44,210,800) as "File I/O"
	DoWindow/C fileIO
	SetDrawLayer UserBack
    Button SetFolderButton,pos={5,15},size={95,22},proc=file_IO#setFolderButtonProc,title="Set Folder"
    PopupMenu fileTypeMenu,pos={120,15},size={102,20},title="Type:"
    PopupMenu fileTypeMenu,mode=1,popvalue="TIFF",value= #"\"TIFF;JPEG;PNG;BMP;P3B;XIM\""
    CheckBox ReverseColor,pos={10,40},size={84,14},title="Reverse Graylevel",value= 0
    CheckBox RenameCheck,pos={10,60},size={84,14},title="Rename Image",value= 1
    ValDisplay imgDatatype pos={180, 50}, size={14,14}, mode=2
    ValDisplay imgDatatype barmisc={0,0}, bodyWidth=15, value=#"root:IP_consts:imageDType_disp",limits={1,3,0}
    ValDisplay imgDatatype frame=2,lowcolor=(65535,65535,0),zeroColor=(65535,65535,65535), highcolor=(0,65535,65535),title="DataType"
	SetVariable baseNameSetVar,pos={10,120},size={166,15},title="New Name:"
    SetVariable baseNameSetVar,format="%g"
    SetVariable baseNameSetVar,value= root:fileIO:baseName
    CheckBox killWavesCheck,pos={10,80},size={111,14},title="Kill Individual Waves"
    CheckBox killWavesCheck,value= 1
    CheckBox E_increaseCheck,pos={140,80},size={69,14},title="E ->",value= 1
    CheckBox StackCheck,pos={10,100},size={69,14},title="Make Stack",value= 1
    CheckBox loadAllCheck,pos={140,100},size={69,14},title="Load All",value= 1

    Button loadImagesButton,pos={5,140},size={95,21},proc=file_IO#loadImageButtonProc,title="Load Image"
    Button loadDatButton,pos={5,170},size={95,21},proc=file_IO#loadDatButtonProc,title="Load dat"
    Button makeMovieButton,pos={5,200},size={95,21},proc=file_IO#stack2movieButtonProc,title="Stack2Movie"
    Button loadStackButton,pos={105,140},size={95,21},proc=file_IO#loadStackButtonProc,title="Load Stack"
    Button MovieCtrlButton,pos={105,170},size={95,21},proc=file_IO#MovieCtrlButtonProc,title="play/stop"

    TitleBox currentMem frame=0,fsize=12,pos={48,224},size={60,20},title="Current Memory"
    ListBox imageListBox mode=1, pos={10,241}, size={190,130}, userColumnResize=1
    ListBox imageListBox listWave=root:IP_consts:imageWaveList
    ListBox imageListBox selWave=root:IP_consts:imageWaveSel, proc=file_IO#imageListBoxProc

    Button viewImageButton,pos={15,373},size={50,20},proc=file_IO#processSingleImageButtonProc,title="View"  
    Button clearButton,pos={80,373},size={50,20},proc=file_IO#clearImageButtonProc,title="Clear"  
    Button saveImageButton,pos={142,373},size={50,20},proc=file_IO#processSingleImageButtonProc,title="Save"  
    Button deleteImageButton,pos={10,395},size={60,20},proc=file_IO#processSingleImageButtonProc,title="Delete"  
    Button renameImageButton,pos={75,395},size={60,20},proc=file_IO#processSingleImageButtonProc,title="Rename"  
    Button extractImageButton,pos={140,395},size={60,20},proc=file_IO#processSingleImageButtonProc,title="Extract"  

    Slider high_thresholdSlider pos={10,418},limits={0,1,0},size={150,20}, disable=0,variable=root:IP_consts:threshold_high,vert=0,proc=imgThresh#thresholdProc,thumbColor=(0,1000,0),side=0,fsize=8
    TitleBox highThresh frame=0,fsize=12,pos={175,418},size={60,20},title="high"
    Button thresholdImageButton,pos={10,435},size={60,20},proc=file_IO#processSingleImageButtonProc,title="Thresh"  
    Button MWandImageButton,pos={105,435},size={70,20},proc=file_IO#processSingleImageButtonProc,title="M-Wand"  
    Slider low_thresholdSlider pos={10,458},limits={0,1,0},size={150,20}, disable=0,variable=root:IP_consts:threshold_low,vert=0,proc=imgThresh#thresholdProc,thumbColor=(1000,1000,0),side=0,fsize=8
    TitleBox lowThresh frame=0,fsize=12,pos={175,458},size={60,20},title="low"
    NVAR MWand_on = root:IP_consts:MWand_on
    ValDisplay MWandDisp pos={180,438},size={15,15}, mode=2,barmisc={0,0},bodyWidth=15,value=#"root:IP_consts:MWand_on",limits={0,1,0},frame=3,highColor=(65535,0,0),zeroColor=(0,0,65535)
    NVAR thresh_on = root:IP_consts:thresh_on
    ValDisplay thresholdDisp pos={78,438},size={15,15}, mode=2,barmisc={0,0},bodyWidth=15,value=#"root:IP_consts:thresh_on",limits={0,1,0},frame=3,highColor=(65535,0,0),zeroColor=(0,0,65535)

    Button histImageButton,pos={10,475},size={45,20},proc=file_IO#processSingleImageButtonProc,title="Hist"  
    Button depthConvertButton,pos={60,475},size={65,20},proc=file_IO#processSingleImageButtonProc,title="Convert"  
//    Button PCAButton, pos={132,475}, size={40,20}, title="PCA"

    ValDisplay Xdisp pos={10,680},size={63,10},value=#"root:IP_consts:imageX", disable=2,title="x:"
    ValDisplay Ydisp pos={10,692},size={63,10},value=#"root:IP_consts:imageY", disable=2,title="y:"
    ValDisplay Zdisp pos={10,704},size={63,10},value=#"root:IP_consts:imageZ", disable=2,title="z:"
    TitleBox versionTitle frame=3,fsize=12,pos={80,685},size={60,20},title="Version 2-18-2008"

    ModifyPanel fixedSize=1

    file_IO#updateImageList()            

    return 0
End

//==================================================================
Static Function clearImageButtonProc(ctrlName) : ButtonControl
    String ctrlName

    NVAR numImages = root:IP_consts:numImages
    numImages = 0;

    DoWindow/K stack_window
    DoWindow/K histogram_window
    DoWindow/K view
    // kill all view related windows
    Variable i=0
    do
        DoWindow $("view"+num2str(i))
        if (V_flag==1)
            DoWindow/K $("view"+num2str(i))
            i += 1
        else
            break
        endif
    while(1)

    DoWindow/K divmap_window 
    DoWindow/K current_spectrum
    DoWindow/K ImageCalPanel 
    DoWindow/K threshold_window
    DoWindow/K magicWand_window
    KillWindow fileIO#preview
    if( GetRTError(1) )
        print "preview window did not exist"
    endif

    SetDataFolder root:images 
    KillWaves/A/F/Z

    SVAR oldImg = root:IP_consts:oldImg
    oldImg = "none"
    file_IO#updateImageList()

    NVAR imageDType_disp = root:IP_consts:imageDType_disp
    imageDType_disp = 0
    return 0;

End

//==================================================================

Function MovieCtrlButtonProc(ctrlName) : ButtonControl
    String ctrlName;

    CommonUsage#play_stop_movie()

    return 0
End

//==================================================================

Static Function processSingleImageButtonProc(ctrlName) : ButtonControl
    String ctrlName

    String curDF=GetDataFolder(1)
    SetDataFolder root:images:    

    String imageName=CommonUsage#getImageSelection()

    if (Exists(imageName)==0)
        return 1;
    endif

    strswitch (ctrlName)
        case "viewImageButton":
            print "now view " + imageName
            if (cmpstr(imageName, "M_Stack")==0)
                file_IO#displayStackImage()
            else
                CommonUsage#displayImage(imageName,"view")
            endif
        break
        case "depthConvertButton":
            NVAR imageDType = root:IP_consts:imageDType
            if (imageDType == 72) // 8bit
                Redimension/W/U $imageName
            else
                Redimension/B/U $imageName
            endif
            CommonUsage#updateImageDatatype(imageName)
        break
        case "saveImageButton":
            if (cmpstr(imageName,"M_Stack")==0)
                SVAR baseName=root:fileIO:baseName
                CommonUsage#saveImageAs(imageName, baseName)
            else // single image
                // remove the old filetype
                Variable i
                String pureName = imageName
                for (i=0;i<4;i+=1)
                        pureName = RemoveEnding(pureName)
                endfor
                CommonUsage#saveImageAs(imageName,pureName+".tif")
            endif
        break
        case "deleteImageButton":
            Print "delete " + imageName
            KillWindow fileIO#preview
            KillWaves/Z $imageName 
            if( GetRTError(1) )
                print "preview window did not exist"
            endif
            SVAR oldImg= root:IP_consts:oldImg
            oldImg = "none"
        break
        case "renameImageButton":
            String newName
            Prompt newName, "New name"
            DoPrompt "Enter new image name", newName
            if (V_Flag)
                return -3
            endif
            Print "new image name is " + newName
            Rename $imageName, $newName
        break
        case "MWandImageButton":
            NVAR MWand_on = root:IP_consts:MWand_on
            if (MWand_on)
                MWand_on = 0
                NVAR old_pixel = root:MagicWand:old_pixel
                old_pixel = -1
                // invert the 0 and 1 in M_MWand_Sel
                // since for M_ROIMask, 0 is ROI
                WAVE M_MWand_Sel=root:images:M_MWand_Sel
                MatrixOP/O M_MWand_Sel= equal(0, M_MWand_Sel)
                NVAR imageDType=root:IP_consts:imageDType
                if (imageDType == 72)
                    Redimension/B/U M_MWand_Sel 
                elseif (imageDType == 80)
                    Redimension/W/U M_MWand_Sel
                endif
// the following annotated code sets magic wand result as ROI 
// automatically, which i don't prefer
// jan. 02. 2008
//                Duplicate/O M_MWand_Sel,M_ROIMask
//                KillWaves/Z M_ImageEdges
                // draw ROI edge as feedback
//                ImageEdgeDetection/N/M=2 Sobel, M_ROIMask
//                DoWindow stack_window
//                if (V_flag==0) // stack_window does not exist
//                    file_IO#displayStackImage()
//                endif
//                AppendImage/W=stack_window M_ImageEdges
//                ModifyImage M_ImageEdges explicit=0,ctab= {0,2,Rainbow,0};DelayUpdate
//                ModifyImage M_ImageEdges minRGB=0,maxRGB=NaN
                KillWaves/Z M_MWand
                DoWindow/K magicWand_window
            else
                MWand_on = 1;
                MagicWand#useMagicWand(imageName)
            endif
        break
        case "histImageButton":
            file_IO#convert_16_8bit(imageName)
        break
        case "extractImageButton":
            if (Exists("M_Stack")==0)
                return 1
            endif
            Duplicate/O $imageName, M_ROIMask
            extract_spectrum()
        break
        case "thresholdImageButton":
            NVAR thresh_on = root:IP_consts:thresh_on
            if (thresh_on)
                thresh_on = 0;
                KillWaves/Z root:images:origin_img
                DoWindow/K threshold_window
            else
                thresh_on = 1;
                imgThresh#doThresh(imageName)
            endif
        break
    endswitch

    file_IO#updateImageList()
    SetDataFolder curDF

    return 0
End

Static Function setFolderButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			NewPath/O/Q/M="Locate Disk Folder"  diskFolderPath
			SVAR folderPath=root:fileIO:folderPath
			if(V_flag==0)
				PathInfo diskFolderPath
				folderPath=S_path
			else
				folderPath="_none_"
			endif
		break
	endswitch

	return 0
End

//==================================================================

Static Function loadStackButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			file_IO#load_stack()
		break
	endswitch

	return 0
End

//==================================================================

Function stack2movieButtonProc(ctrlName) : ButtonControl
    String ctrlName;

    NVAR numImages = root:IP_consts:numImages
    NVAR imageDType = root:IP_consts:imageDType 
    NVAR imageColorReverse = root:IP_consts:imageColorReverse
    WAVE M_Stack = root:images:M_Stack
    NVAR dimX = root:IP_consts:dimX
    NVAR dimY = root:IP_consts:dimY
    
    if (exists("M_Stack")==0)
        return -1;
    endif
    Make/O/D/N=(dimX,dimY,0,0) Frame;

    Variable imageBitInfo
    switch (CommonUsage#checkImageDatatype("M_Stack"))
        case 72: // 8 bit unsigned int
            imageBitInfo = 255
        break
        case 80: // 16 bit unsigned int
            imageBitInfo = 65535
        break
        default:
            doAlert 0, "caution! the displayed stack is neither 8 bit nor 16 bit!"
            ImageStats/M=1 M_Stack
            imageBitInfo = V_max
        break
    endswitch

    Frame[][] = M_Stack[p][q][0]
    DoWindow/K stack_window
    Display/K=1/N=stack_window/W=(270,44,dimX+270,dimY+44);AppendImage Frame
    SetAxis/A/R/W=stack_window left
    ModifyGraph width={Plan,1,bottom,left} 
    ModifyImage/W=stack_window Frame ctab={0,imageBitInfo,Grays,imageColorReverse}
    DoUpdate
    
    NewMovie/I/O/F=20 as "Movie"
    AddMovieFrame 
    
    Variable i
    for (i=1;i<numImages;i+=1)
        Frame[][] = M_Stack[p][q][i]
        DoUpdate
        AddMovieFrame
    endfor

    closeMovie
    RemoveImage Frame
    AppendImage M_Stack
    SetAxis/A/R/W=stack_window left
    ModifyGraph width={Plan,1,bottom,left} 
    ModifyImage/W=stack_window M_Stack ctab={0,imageBitInfo,Grays,imageColorReverse}
    DoUpdate
    SetBackground file_IO#movieEffect()

    SetWindow stack_window hook(keypress)=CommonUsage#mouseReadHook

    KillWaves Frame
    return 0
End

//==================================================================

Static Function loadImageButtonProc(ctrlName) : ButtonControl
    String ctrlName
    SVAR folderPath=root:fileIO:folderPath
    if (cmpstr(folderPath, "_none_") == 0)
        doAlert 0,"Set folder path first!"
        return -1
    endif

    file_IO#loadImagesInFolder()

    return 0
End

//==================================================================

Static Function displayStackImage()

    NVAR dimX = root:IP_consts:dimX
    NVAR dimY = root:IP_consts:dimY

    print "dimX", dimX
    print "dimY", dimY

    NVAR imageColorReverse = root:IP_consts:imageColorReverse
    ControlInfo/W=fileIO ReverseColor
    imageColorReverse = V_Value
    
    Variable imageBitInfo
    switch (CommonUsage#checkImageDatatype("M_Stack"))
        case 72: // 8 bit unsigned int
            imageBitInfo = 255
        break
        case 80: // 16 bit unsigned int
            imageBitInfo = 65535
        break
        default:
            doAlert 0, "caution! the displayed stack is neither 8 bit nor 16 bit!"
            ImageStats/M=1 M_Stack
            imageBitInfo = V_max
        break
    endswitch

    SetDataFolder root:images:
    DoWindow/K stack_window
    Display/K=1/N=stack_window/W=(450,44,450+dimX,dimY+44);AppendImage M_Stack
    SetAxis/A/R/W=stack_window left
    ModifyGraph width={Plan,1,bottom,left} 
    ModifyImage/W=stack_window M_Stack ctab={0,imageBitInfo,Grays,imageColorReverse}
    SetBackground file_IO#movieEffect()
    
    SetWindow stack_window hook(keypress)=CommonUsage#mouseReadHook
    return 0
End

//==================================================================

Function movieEffect()

    NVAR numImages = root:IP_consts:numImages
    NVAR gThePlane = root:IP_consts:gThePlane
    WAVE M_Stack = root:images:M_Stack

    ModifyImage/W=stack_window M_Stack,plane=gThePlane
    gThePlane+=1
    if( gThePlane>=numImages)
        gThePlane= 0
    endif

    return 0
End
//==================================================================

Function load_stack()
    NVAR numImages = root:IP_consts:numImages
    NVAR energy_steps = root:IP_consts:energy_steps
    NVAR dimX= root:IP_consts:dimX
    NVAR dimY= root:IP_consts:dimY

    SVAR folderPath=root:fileIO:folderPath
    if(cmpstr(folderPath, "_none_")==0)
        doAlert 0, "set folder path first."
        return 0
    endif

    // delete existing stack, this is to save memory
    SetDataFolder root:images:
    if (Exists("M_Stack")==1)
        doWindow/K stack_window
        killWaves/Z M_Stack
    endif

    NewPath/O path, folderPath
    ImageLoad/O/C=-1/N=M_Stack/T=TIFF/P=path

    if (Exists("M_Stack")==1)
        dimX = DimSize(M_Stack, 0)
        dimY = DimSize(M_Stack, 1)
        numImages= V_numImages;
        print numImages, " images in the stack."
// energy_steps should come from .dat file
//        energy_steps = -1; 
        displayStackImage();
        file_IO#updateImageList()
    endif
End


//==================================================================

Static Function loadDatButtonProc(ba) : ButtonControl
    STRUCT WMButtonAction &ba

    switch( ba.eventCode )
	case 2: // mouse up
            CommonUsage#load_dat()
	    break
    endswitch

    return 0
End

//==================================================================

Static Function imageListBoxProc(lb) : ListBoxControl
    STRUCT WMListboxAction &lb

    switch (lb.eventCode)
        case 3: // double click
            break;
        case 4: // cell selection
            WAVE/T imageWaveList = root:IP_consts:imageWaveList
            print "cell selection: row ", lb.row
            SVAR oldImg= root:IP_consts:oldImg
            // the old image is selected again
            if (cmpstr(oldImg,imageWaveList[lb.row]) == 0)
                return 0;
            endif
            if (cmpstr(oldImg, "none") != 0) // not the very first selection
                RemoveImage/W=fileIO#preview/Z $oldImg
                print "remove " + oldImg
            else
                Display/K=1/N=preview/HOST=fileIO/W=(0,500,210,675)
            endif
            oldImg = imageWaveList[lb.row]
            AppendImage/W=fileIO#preview root:images:$oldImg
            SetAxis/A/R/W=fileIO#preview left

            // update image data type information
            CommonUsage#updateImageDatatype(oldImg)
            break;
        case 5:
            break;
        default:
            break;
    endswitch
    
    return 0

End

//==================================================================

Function updateImageList()
    WAVE/T imageWaveList = root:IP_consts:imageWaveList
    WAVE imageWaveSel = root:IP_consts:imageWaveSel
    WAVE myColors = root:IP_consts:myColors
   
    SetDataFolder root:images 
    String listAllImages = WaveList("*",";","")
    String imgName;
    
    Variable imgNumber = ItemsInList(listAllImages)
    if(imgNumber<0)
        return -1
    endif
    
    Redimension/N=(imgNumber) imageWaveList 
    Redimension/N=(imgNumber) imageWaveSel
//    imageWaveSel = 0
//    imageWaveSel[0] = 1

    Variable i;
    for (i=0;i<imgNumber;i+=1)
        imgName = StringFromList(i,listAllimages)
        imageWaveList[i] = imgName
    endfor
    
    doUpdate;
    return 0
End

//==================================================================

Static Function init()
    String curDF=GetDataFolder(1)
    SetDataFolder curDF

    SetDataFolder root:
    NewDataFolder/O/S root:IP_consts
    // number of images from load images button
    Variable/G numImages=0
    // energy_steps from .dat file
    Variable/G energy_steps = -1
    Variable/G imageColorReverse = 0
    Variable/G isPlaying = 0
    Variable/G isStacking = 1
    Variable/G dimX = 0
    Variable/G dimY = 0
    Variable/G gThePlane = 0
    Variable/G threshold_high= 1
    Variable/G threshold_low = 0.5
    Variable/G thresh_on = 0
    Variable/G MWand_on = 0
    Variable/G imageDType = 1
    Variable/G imageDType_disp = 0
    Variable/G imageX= -1
    Variable/G imageY= -1
    Variable/G imageZ= -1
    Variable/G manual_hist = 0
    String/G oldImg= "none"
    Make /O/N=0/T imageWaveList
    Make /O/N=0 imageWaveSel
    Make/O/W/U myColors={{0,0,0},{65535,0,0},{0,65535,0},{0,0,65535},{0,65535,65535},{65535,65535,0},{65535,0,65535}}
    MatrixTranspose myColors

    // energy range of divmap calculation
    NewDataFolder/O/S root:extractSpec
    Variable/G off_peak_r = 3
    Variable/G on_peak_r  = 2
    Variable/G max_spacing  = 0
    Variable/G min_spacing  = 100
    Variable/G energy_shift = 0
    Variable/G energy_copy = 0
    Variable/G onpeak_E = -1
    Variable/G offpeak_E = -1

    NewDataFolder/O/S root:MagicWand
    Variable/G old_pixel = -1

    NewDataFolder/O root:makeHistogram
    NewDataFolder/O root:fitPeaks

    NewDataFolder/O/S root:fileIO
	String/G folderPath="_none_"
	String/G baseName="Stack_"

    NewDataFolder/O root:spectra
    NewDataFolder/O/S root:images

End
