#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName=makeDivmap

Function make_divmap()

    if (WinType("current_spectrum") == 0)
        DoAlert 0, "There is no spectrum."
        return -1
    endif
    DoWindow/F current_spectrum

    NVAR off_peak_r = root:extractSpec:off_peak_r
    NVAR on_peak_r = root:extractSpec:on_peak_r
    NVAR max_spacing = root:extractSpec:max_spacing
    NVAR min_spacing = root:extractSpec:min_spacing
    Wave spectrum = root:IP_consts:spectrum
    Wave energy = root:IP_consts:energy
    NVAR energy_steps = root:IP_consts:energy_steps
    
    SetDataFolder root:extractSpec

    NewDataFolder/O root:WinGlobals
    NewDataFolder/O root:WinGlobals:current_spectrum
    String/G root:WinGlobals:current_spectrum:S_TraceOffsetInfo= ""
    NVAR offpeak_E = root:extractSpec:offpeak_E
    NVAR onpeak_E = root:extractSpec:onpeak_E
    if (offpeak_E == -1)
        offpeak_E = energy[1]
    endif
    if (onpeak_E == -1)
        onpeak_E = energy[1]
    endif

    Make/O offpeakDrag={-Inf,Inf},onpeakDrag={-Inf,Inf}
    Make/O offDrag={offpeak_E, offpeak_E}, onDrag={onpeak_E, onpeak_E}
    AppendToGraph/W=current_spectrum offpeakDrag vs offDrag
    AppendToGraph/W=current_spectrum onpeakDrag vs onDrag
    ModifyGraph quickdrag(offpeakDrag)=1,live(offpeakDrag)=1
    ModifyGraph quickdrag(onpeakDrag)=1,live(onpeakDrag)=1
    ModifyGraph lSize(offpeakDrag)=2,lSize(onpeakDrag)=2
    ModifyGraph rgb(spectrum)=(0,0,0),rgb(onpeakDrag)=(1,4,52428)

    ControlBar 56
    
    SetVariable OnPeakVar,pos={20,10},size={170,25},proc=NIH_OnOffSetVarProc,title="on peak energy (blue line):"
    SetVariable OnPeakVar,limits={-Inf,Inf,0},value= root:extractSpec:onpeak_E
    SetVariable OffPeakVar,pos={20,35},size={170,25},proc=NIH_OnOffSetVarProc,title="off peak energy (red line):"
    SetVariable OffPeakVar,limits={-Inf,Inf,0},value= root:extractSpec:offpeak_E
    
    SetVariable onpeak_radius,pos={200,10},size={110,15},title="On Peak Radius:"
    SetVariable onpeak_radius,format="%u"
    SetVariable onpeak_radius,value=root:extractSpec:on_peak_r

    SetVariable offpeak_radius,pos={200,35},size={110,15},title="Off Peak Radius:"
    SetVariable offpeak_radius,format="%u"
    SetVariable offpeak_radius,value=root:extractSpec:off_peak_r;

   SetVariable energy_shift, pos={320,10}, size={100,15}, title="E Shift (eV):"
    SetVariable energy_shift, value=root:extractSpec:energy_shift;

   SetVariable energy_copy, pos={320,35}, size={100,15}, title="E copy (eV):"
    SetVariable energy_copy, value=root:extractSpec:energy_copy;

    Button button0,pos={600,5},size={50,20},title="Try it"
    Button button0,proc=makeDivmap#UserTryButtonProc
    Button button1,pos={710,30},size={70,20},title="Close"
    Button button1,proc=makeDivmap#User_CancelButtonProc
    Button button3,pos={660,5},size={120,20},title="Save Spectrum"
    Button button3,proc=makeDivmap#User_saveSpecProc
    Button button4,pos={600,30},size={100,20},title="Save Div Map"
    Button button4,proc=makeDivmap#User_saveDivMapProc
    Button button5,pos={490,5},size={100,20},title="Flip Vertical"
    Button button5,proc=makeDivmap#UserFlipProc
    Button button6,pos={490,30},size={100,20},title="Save Graph"
    Button button6,proc=makeDivmap#UserSaveGraphProc
    Button button7,pos={430,5},size={50,20},title="Shift"
    Button button7,proc=makeDivmap#UserShiftEnergyProc
    Button button8,pos={430,30},size={50,20},title="Copy"
    Button button8,proc=makeDivmap#UserSaveSliceProc

    SetWindow kwTopWin,hook(fun)=NIH_updateVarWindowProc

    // find the minimal spacing of the sampled energies 
    Variable i, tmp

    for (i=1;i<energy_steps;i+=1)
        tmp = energy[i] - energy[i-1]
        if (tmp > max_spacing)
            max_spacing = tmp;
        endif    
        if (tmp < min_spacing)
            min_spacing = tmp;
        endif
    endfor

    SetDataFolder root:images:

    return 0;
End

//==============================================================

Function UserSaveSliceProc(ctrlName): ButtonControl
    String ctrlName;
    
    NVAR min_spacing = root:extractSpec:min_spacing
    NVAR max_spacing = root:extractSpec:max_spacing
    NVAR energy_copy= root:extractSpec:energy_copy
    WAVE energy = root:IP_consts:energy
    WAVE energy_r = root:IP_consts:energy_r
    WAVE M_Stack = root:images:M_Stack
    SVAR folderPath = root:fileIO:folderPath
    NewPath/O path, folderPath

    // find the closest energy value available to the input
    FindValue/T=(min_spacing)/V=(energy_copy) energy
    if (V_value == -1)  // not found with small tolerance
        FindValue/T=(max_spacing)/V=(energy_copy) energy
    endif
    energy_copy = energy[V_value]

    // save that image
    print "save image"

    ControlInfo/W=fileIO E_increaseCheck
    if (V_Value == 0) 
        FindValue/V=(energy_copy) energy_r
    else
        FindValue/V=(energy_copy) energy
    endif
    
    ImageTransform/P=(V_Value) getPlane M_Stack
    switch (CommonUsage#checkImageDatatype("M_ImagePlane"))
        case 72: // 8 bit unsigned int
            ImageSave/D=40/P=path/I/T="TIFF" M_ImagePlane
        break
        case 80: // 16 bit unsigned int
            ImageSave/D=16/P=path/I/T="TIFF" M_ImagePlane
        break
        default:
            doAlert 0, "caution! the stack is neither 8 bit nor 16 bit!"
            ImageSave/P=path/I/T="TIFF" M_ImagePlane
        break
    endswitch

    return 0
End

//==============================================================

Static Function UserShiftEnergyProc(ctrlName): ButtonControl
    String ctrlName;

    WAVE spectrum = root:IP_consts:spectrum
    WAVE energy = root:IP_consts:energy
    WAVE energy_r = root:IP_consts:energy_r
    NVAR energy_shift = root:extractSpec:energy_shift
    NVAR onpeak_E = root:extractSpec:onpeak_E 
    NVAR offpeak_E = root:extractSpec:offpeak_E 
    
    spectrum[][0] = spectrum[p][0] + energy_shift
    energy   = energy + energy_shift
    energy_r = energy_r + energy_shift
    onpeak_E  += energy_shift
    offpeak_E += energy_shift
    
    DoWindow/K current_spectrum
    Display/K=1/N=current_spectrum/W=(150,100,950,550) spectrum[][1] vs spectrum[][0]
    makeDivmap#make_divmap()

    return 0;
End

//==============================================================

Static Function UserSaveGraphProc(ctrlName): ButtonControl
    String ctrlName;

    String filename
    SVAR folderPath = root:fileIO:folderPath
    NewPath/O path, folderPath

    SavePICT/E=-5/P=path/B=288/WIN=current_spectrum as "Extracted Spectrum.png"

    return 0
End

//==============================================================

Static Function User_saveDivMapProc(ctrlName): ButtonControl
    String ctrlName;
    
    NVAR onpeak_E = root:extractSpec:onpeak_E
    NVAR offpeak_E = root:extractSpec:offpeak_E
    String fileName
    sprintf fileName, "Div_Map_%4.1f_%4.1feV.tif", onpeak_E, offpeak_E
    CommonUsage#saveImageAs("divmap", fileName)

    return 0;
End

//==============================================================

Static Function UserFlipProc(ctrlName): ButtonControl
    String ctrlName;
    WAVE spectrum = root:IP_consts:spectrum
    NVAR energy_steps = root:IP_consts:energy_steps

    Variable imageBitInfo
    switch (CommonUsage#checkImageDatatype("M_Stack"))
        case 72: // 8 bit unsigned int
            imageBitInfo = 255
        break
        case 80: // 16 bit unsigned int
            imageBitInfo = 65535
        break
        default:
            doAlert 0, "caution! the stack is neither 8 bit nor 16 bit!"
            ImageStats/M=1 M_Stack
            imageBitInfo = V_max
        break
    endswitch
	
    spectrum[][1] = imageBitInfo - spectrum[p][1] 

    return 0;
End

//==============================================================

Static Function User_saveSpecProc(ctrlName): ButtonControl
    String ctrlName;
    
    WAVE spectrum = root:IP_consts:spectrum
    SVAR folderPath = root:fileIO:folderPath
    NewPath/O path, folderPath
    Save/I/G/P=path spectrum as "spectrum.txt"
    return 0;
End

//==============================================================

Static Function User_CancelButtonProc(ctrlName): ButtonControl
    String ctrlName;

    DoWindow/K $WinName(0, 1)

    return 0;
End

//==============================================================

Function NIH_OnOFFSetVarProc(ctrlName, varNum, varStr, varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

        Wave energy = root:IP_consts:energy
        varNum = varNum - energy[1]
	// NOTE: setting the offsets here causes a dependency to fire that ends up updating the image
	if( CmpStr("OnPeakVar",ctrlName) == 0 )
		ModifyGraph/W=current_spectrum offset(onpeakDrag)={varNum,0}
	else
		ModifyGraph/W=current_spectrum offset(offpeakDrag)={varNum,0}
	endif

End

//==============================================================
Function NIH_updateVarWindowProc(s)
    STRUCT WMWinHookStruct &s 

    switch(s.eventCode) 
        case 4: // mouse moved
            NIH_updateVarFromLines()
        break
    endswitch

    return 0
End

//==============================================================
Function NIH_updateVarFromLines()
    String topGraphName
    topGraphName = WinName(0, 1)
    SVAR s=root:WinGlobals:$(topGraphName):S_TraceOffsetInfo
    Variable isOffPeak= StrSearch(s,"TNAME:offpeakDrag;",0)>0
    Variable isOnPeak= StrSearch(s,"TNAME:onpeakDrag;",0)>0
    if( (isOffPeak==0) %& (isOnPeak==0) )
        return -1		// not valid yet
    endif

    String targ= ";XOFFSET:"
    Variable v1= StrSearch(s,targ,0)
    if( v1<0 )
        return -2
    endif

    v1 += strlen(targ)
    Variable v2= StrSearch(s,";",v1)
    Variable xoff= str2num(s[v1,v2-1])
    if( NumType(xoff) != 0 )
        return -3;
    endif

    WAVE energy = root:IP_consts:energy
    NVAR onpeak_E = root:extractSpec:onpeak_E
    NVAR offpeak_E = root:extractSpec:offpeak_E
    NVAR min_spacing = root:extractSpec:min_spacing
    NVAR max_spacing = root:extractSpec:max_spacing
    WAVE onDrag = root:extractSpec:onDrag
    WAVE offDrag = root:extractSpec:offDrag

    if( isOnPeak )
        onpeak_E  = xoff + onDrag[1]
        FindValue/T=(min_spacing)/V=(onpeak_E) energy
        if (V_value == -1)  // not found with small tolerance
            FindValue/T=(max_spacing)/V=(onpeak_E) energy
        endif
        onpeak_E = energy[V_value]
    else
        offpeak_E = xoff + offDrag[1]
        FindValue/T=(min_spacing)/V=(offpeak_E) energy
        if (V_value == -1)  // not found with small tolerance
            FindValue/T=(max_spacing)/V=(offpeak_E) energy
        endif
        offpeak_E = energy[V_value]
    endif
    
    return 0
End

//==============================================================

Static Function UserTryButtonProc(ctrlName): ButtonControl
    String ctrlName;

    makeDivmap#NIH_updateVarFromLines()

    NVAR off_p_r = root:extractSpec:off_peak_r
    NVAR on_p_r = root:extractSpec:on_peak_r

    WAVE spectrum = root:IP_consts:spectrum
    NVAR energy_steps = root:IP_consts:energy_steps
    NVAR onpeak_E = root:extractSpec:onpeak_E
    NVAR offpeak_E = root:extractSpec:offpeak_E

    // note in igor, 2D wave is organized column-wise
    // FindValue for 2D wave
    // col = floor (V_value/rowsInWave)
    // row = V_value - col*rowsInWave
    FindValue/V=(onpeak_E) spectrum
    Variable low_on = V_value - on_p_r
    Variable high_on = V_value + on_p_r
    print "on peak sample range:"
    printf "%4.1f - %4.1f - %4.1f\r", spectrum[high_on], spectrum[V_value], spectrum[low_on]

    FindValue/V=(offpeak_E) spectrum
    Variable low_off = V_value - off_p_r
    Variable high_off = V_value + off_p_r
    print "off peak sample range:"
    printf "%4.1f - %4.1f - %4.1f\r", spectrum[high_off], spectrum[V_value], spectrum[low_off]

    if (low_on<0 || high_on>=energy_steps)
	doAlert 0,"Not enough data points for the current ON PEAK radius!"
        return -1;
    endif
    if (low_off<0 || high_off>=energy_steps)
	doAlert 0,"Not enough data points for the current OFF PEAK radius!"
        return -1;
    endif

    SetDataFolder root:images:

    WAVE M_Stack = root:images:M_Stack
    print "make divmap" 
    Variable imageBitInfo
    switch (CommonUsage#checkImageDatatype("M_Stack"))
        case 72: // 8 bit unsigned int
            imageBitInfo = 255
        break
        case 80: // 16 bit unsigned int
            imageBitInfo = 65535
        break
        default:
            doAlert 0, "caution! the stack is neither 8 bit nor 16 bit!"
            ImageStats/M=1 M_Stack
            imageBitInfo = V_max
        break
    endswitch

    NVAR imageColorReverse = root:IP_consts:imageColorReverse
    ControlInfo/W=fileIO ReverseColor
    imageColorReverse = V_Value

    print "calculating on/off peak quotients"
    Variable k;
    NVAR dimX = root:IP_consts:dimX
    NVAR dimY = root:IP_consts:dimY
    Make/O/D/N=(dimX,dimY) on_p_avg;
    Make/O/D/N=(dimX,dimY) off_p_avg;
    on_p_avg = 0
    off_p_avg = 0
    for (k=low_on;k<=high_on;k+=1)
        ImageTransform/P=(k) getPlane M_Stack
        WAVE M_ImagePlane = root:images:M_ImagePlane
        MatrixOP/O on_p_avg = on_p_avg + M_ImagePlane
        ImageTransform/P=(k)/D=M_ImagePlane setPlane M_Stack
    endfor
    MatrixOp/O on_p_avg = on_p_avg/(high_on-low_on+1)

    for (k=low_off;k<=high_off;k+=1)
        ImageTransform/P=(k) getPlane M_Stack
        WAVE M_ImagePlane = root:images:M_ImagePlane
        MatrixOP/O off_p_avg = off_p_avg + M_ImagePlane
        ImageTransform/P=(k)/D=M_ImagePlane setPlane M_Stack
    endfor
    MatrixOP/O off_p_avg = off_p_avg/(high_off-low_off+1)

    if (imageColorReverse)
        // identify the boundary pts on Sphinx
        MatrixOP/O divmap = greater(imageBitInfo*0.92,on_p_avg)*((-1)*on_p_avg+imageBitInfo)/((-1)*off_p_avg+imageBitInfo)
    else
        // identify the boundary pts on Sphinx
        MatrixOP/O divmap = on_p_avg / (greater(imageBitInfo*0.08,off_p_avg)*imageBitInfo+off_p_avg)  
    endif

    print "rescaling divmap"
    ImageStats/M=1 divmap
    k = imageBitInfo / (V_max-V_min)
    MatrixOP/O divmap = (divmap-V_min)*k

    // transfrom divmap to be 8bit/16bit image
    // instead of single floating point image
    if (imageBitInfo == 255)
        Redimension/B/U divmap
    else
        Redimension/W/U divmap
    endif
    

    DoWindow/K divmap_window
    Display/K=1/N=divmap_window/W=(100, 100, 100+dimX, 100+dimY);AppendImage divmap
    SetAxis/A/R/W=divmap_window left
    ModifyGraph width={Plan,1,bottom,left}

    setwindow divmap_window hook(keypress)=CommonUsage#mouseReadHook

    KillWaves/Z root:images:M_ImagePlane

    file_IO#updateImageList()            

    return 0;
End
