#pragma rtGlobals=2		// Use modern global access method.
#pragma ModuleName=setROI

#include <Image Common>

// ************ Image ROI **********************


Function create_ROI_panel()

	DoWindow/F WMImageROIPanel
	if( V_Flag==1 )
		return 0
	endif

	NewPanel/K=1/W=(211,44,350,280) as "ROI"
	DoWindow/C WMImageROIPanel
//	AutoPositionWindow/E/M=0/R=$igName
	ModifyPanel fixedSize=1
	Button StartROI,pos={10,9},size={120,20},proc=WMRoiDrawButtonProc,title="Start ROI Draw"
	Button StartROI,help={"Adds drawing tools to top image graph. Use rectangle, circle or polygon."}
	Button clearROI,pos={10,63},size={120,20},proc=WMRoiDrawButtonProc,title="Erase ROI"
	Button clearROI,help={"Erases previous ROI. Not undoable."}
	Button FinishROI,pos={10,35},size={120,20},proc=WMRoiDrawButtonProc,title="Finish ROI"
	Button FinishROI,help={"Click after you are finished editing the ROI"}
    Button saveROICopy,pos={10,92},size={120,20},proc=saveRoiProc,title="Save ROI Copy"

    SVAR folderPath = root:fileIO:folderPath
    NewPath/O path, folderPath

    PopupMenu fileSelectMenu,pos={10,120},size={150,20},title="Image File:"
    PopupMenu fileSelectMenu,mode=2,popvalue="_none_",value= "_none_;" + IndexedFile(path,-1,".tif")

    Button loadROI,pos={10,145},size={120,20},proc=loadRoiProc,title="Load ROI"
    CheckBox zeroRoiCheck,pos={15,200},size={94,14},title="Zero ROI Pixels",value= 1
    Button roiPanelHelp,pos={10,170},size={120,20},proc=roiPanelButtonProc,title="Help"

end

// the following function creates the roi wave and saves it in the same data folder as the top
// image wave.

Function setRoiCopyProc()
	
    String topWave=WMGetImageWave(WMTopImageGraph())
    WAVE/Z ww=$topWave
    if(WaveExists(ww)==0)
        doAlert 0, "no wave exists in the graph"
	return 0
    endif

    String saveDF=GetDataFolder(1)
    String waveDF=GetWavesDataFolder(ww,1 )
    SetDataFolder waveDF

    print "generating ROI mask"
    ControlInfo zeroRoiCheck
    if(V_value)
	ImageGenerateROIMask/E=1/I=0 $WMTopImageName()		
    else
	ImageGenerateROIMask $WMTopImageName()		
    endif

    // draw ROI edge as feedback
    SetDrawLayer/K ProgFront
    KillWaves/Z M_ImageEdges
    ImageEdgeDetection/N/M=2 Sobel, M_ROIMask

    AppendImage M_ImageEdges
    ModifyImage M_ImageEdges explicit=0,ctab= {*,2,Rainbow,0};DelayUpdate
    ModifyImage M_ImageEdges minRGB=0,maxRGB=NaN
    SetDataFolder saveDF

    file_IO#updateImageList()
end

Function WMRoiDrawButtonProc(ctrlName) : ButtonControl
	String ctrlName

	String ImGrfName= WMTopImageGraph()
	if( strlen(ImGrfName) == 0 )
		return 0
	endif
	
    SetDataFolder root:images:
    DoWindow/F $ImGrfName
    if( CmpStr(ctrlName,"StartROI") == 0 )
                RemoveImage/W=$ImGrfName/Z M_ImageEdges
                SetDrawLayer/K ProgFront
		ShowTools/A rect
		SetDrawLayer ProgFront
		Wave w= $WMGetImageWave(ImGrfName)		// the target matrix
		String iminfo= ImageInfo(ImGrfName, NameOfWave(w), 0)
		String xax= StringByKey("XAXIS",iminfo)
		String yax= StringByKey("YAXIS",iminfo)
		SetDrawEnv linefgc= (3,52428,1),fillpat= 0,xcoord=$xax,ycoord=$yax,save
    endif
    if( CmpStr(ctrlName,"FinishROI") == 0 )
        if (FindListItem("M_ImageEdges", ImageNameList(ImGrfName,";")) >= 0)
            return 0    // already updated 
        endif
	GraphNormal
	HideTools/A
	SetDrawLayer UserFront
	DoWindow/F WMImageROIPanel
        setRoiCopyProc()
    endif
    if( CmpStr(ctrlName,"clearROI") == 0 )
        setROI#clearROI()
    endif
End
	
//==================================================================

Function clearROI()
    String ImGrfName= WMTopImageGraph()
    GraphNormal
    SetDrawLayer/K ProgFront
    SetDrawLayer UserFront
    //DoWindow/F WMImageROIPanel
    if (FindListItem("M_ImageEdges", ImageNameList(ImGrfName,";")) >= 0)
        RemoveImage M_ImageEdges
    endif

End

//==================================================================

Function roiPanelButtonProc(ctrlName) : ButtonControl
	String ctrlName

	DisplayHelpTopic "ROI Panel"
End

Function loadRoiProc(ctrlName) : ButtonControl
    String ctrlName

    ControlInfo/W=WMImageROIPanel fileSelectMenu 
    if (cmpstr(S_value, "_none_") == 0) // same string
        doAlert 0, "Could not find any ROI files. Note they should have .tif suffix."
        return 0;
    endif

    SVAR folderPath=root:fileIO:folderPath
    NewPath/O path, folderPath

    print "load " + folderPath + S_value

    KillWaves/Z M_ROIMask
    KillWaves/Z M_ImageEdges
    setROI#clearROI()

    SetDataFolder root:images:
    ImageLoad/O/P=path/T=TIFF/N=M_ROIMask S_value
    if (V_flag == 1)
        WAVE M_ROIMask = root:images:M_ROIMask
        switch (CommonUsage#checkImageDatatype("M_ROIMask"))
        case 72: // 8 bit unsigned int
            M_ROIMask = 255 - M_ROIMask
        break
        case 80: // 16 bit unsigned int
            M_ROIMask = 65535 - M_ROIMask
        break
        default:
            doAlert 0, "caution! the rescaled stack is neither 8 bit nor 16 bit!"
            ImageStats/M=1 M_ROIMask
            M_ROIMask = V_max - M_ROIMask
        break
    endswitch

    else
        return -2
    endif
    
    // draw ROI edge as feedback
    ImageEdgeDetection/N/M=2 Sobel, M_ROIMask

    AppendImage M_ImageEdges
    ModifyImage M_ImageEdges explicit=0,ctab= {0,2,Rainbow,0};DelayUpdate
    ModifyImage M_ImageEdges minRGB=0,maxRGB=NaN

    file_IO#updateImageList()
end

Function saveRoiProc(ctrlName) : ButtonControl
    String ctrlName

    CommonUsage#saveImageAs("M_ROIMask", "roi.tif")

    return 0;
end

