#pragma rtGlobals=1		// Use modern global access method.

// 	2014-08
// 	Calculations of the interactions between n clay platelets

// From Grodinsky, Chapter 4. 
// Repulsive force between two plates at constant surface potential, Vo, in V
// Separation, D, in nm
// Force in N m-2 ???
Function edlPlatePlate_Grod(Vo,D,CIConc,z,RelEps,T)
	Variable Vo,D,CIConc,z,RelEps,T
	
	Variable Dsi 		= D*1e-9
	Variable Epsilon 	= RelEps * cPermittivity
	
	Variable Kap 			= Kappa(CIConc,z,RelEps,T)
	Variable InvDebye 	= ReciprocalDebyeLength(CIConc,z,RelEps,T)
	Variable coshVal 	= cosh(0.5*Kap*Dsi)
	
	print "Kappa",1e9/Kap,"nm"
	print "Debye length",1e9/InvDebye,"nm"
	print "cosh term",coshVal
	
	Variable Force 	= 0.5*Epsilon*Kap^2*Vo^2*(1/coshVal)^2
	
	return Force
End

// Check for errors
// Counterion concentration in molarity (= moles/liter)
Function Kappa(CIConc,z,RelEps,T)
	Variable CIConc,z,RelEps,T
	
	Variable Conc 	= CIConc * 1000
	Variable Eps 	= RelEps*cPermittivity
	
	Variable RelKap 	= sqrt((Eps*cR*T)/(2*z^2*cFaraday^2*Conc))
	
	return 1/RelKap
End

Function UUUU(UU)
	Variable UU
	
	return (UU*cBoltzmann*298)/cElectron
End
	
// From Ohshima
// Model 3, linear superposition
// Electrolyte concentration, CIConc, in M
Function edlPlatePlate_LSA(Vo,D,CIConc,z,RelEps,T)
	Variable Vo,D,CIConc,z,RelEps,T
	
	Variable Dm	 	= D/1e9
	Variable Nm3 		= MolarityToNumberDensity(CIConc) 	// input in moles per liter; output in ions per m3
	Variable Ncm3 	= Nm3/1e6
	Variable PreFac 	= 64*Ncm3*cBoltzmann*T
	Variable uu 		= z*cElectron*Vo/(2*cBoltzmann*T)
	
	Variable DebyeHukel 	= sqrt((2*Nm3*z^2*cElectron^2)/(RelEps*cPermittivity*cBoltzmann*T))
	
	Variable Pressure 	= PreFac*tanh(uu)^2*exp(-DebyeHukel*Dm)
	
	return Pressure
End

// From Zhang
// Distance, D, in nanometers
// Clay sheet thickness, tt, in nm
// The Hamaker constant, A, in zJ = 10-21J
Function vdWPlatePlate_Zhang(A,D,tt)
	Variable A, D, tt
	
	Variable Scale 	= 1e27
	Variable PreFac 	= (1e-21*A)/(6*pi)
	
	Variable Force 	= PreFac * (1/D^3 - 2/(D+tt)^3 + 1/(D+2*tt)^3) * Scale
	
	return Force	 
End

// From Zhang
// Distance, D, in nanometers
// Clay sheet thickness, tt, in nm
// The Hamaker constant, A, in zJ = 10-21J
Function edlPlatePlate_Zhang(Vo,D,CIConc,z,RelEps,T)
	Variable Vo,D,CIConc,z,RelEps,T
	
	Variable u 			= z*cElectron*Vo/(cBoltzmann*T)
	Variable Nm3 		= MolarityToNumberDensity(CIConc) 	// input in moles per liter; output in ions per m3
	Variable Ncm3 	= Nm3/1e6
	Variable PreFac 	= 2*Ncm3*cBoltzmann*T
	
	return PreFac*(cosh(u) - 1)
End
	


// -----------------------------------------------------------------------------

// 	Tadmor, J. Phys.: Condens. Matter 13, 2001)
// Distance, D, in nanometers
// Clay sheet thickness, t, in nm
// The Hamaker constant, A, in zJ = 10-21J
// Pressure in N m-2, presumably
Function EvdWPlanes(D,t,A)
	Variable D, t, A
	
	Variable PreFactor = -1*(1e-21*A)/(12*pi)
	
	Variable Dsi 	= 1e-9 * D
	Variable tsi 	= 1e-9 * t
	
	return PreFactor * (1/Dsi^2  +  1/(Dsi+2*tsi)^2 -  2/(Dsi+tsi)^2)

End



// Distance, D, in nanometers
// Clay sheet thickness, t, in nm
// The Hamaker constant, A, in zJ = 10-21J
// Pressure in N m-2, presumably
Function Pvdw(D,t,A)
	Variable D, t, A
	
	Variable PreFactor = (1e-21*A)/(6*pi)
	
	Variable Dsi 	= 1e-9 * D
	Variable tsi 	= 1e-9 * t
	
	return PreFactor * (1/Dsi^2  +  1/(Dsi+2*tsi)^2 +  2/(Dsi+tsi)^2)
End

// Distance, D, in nanometers
// Surface Potential, P, in V
// Pressure in N m-2, presumably
Function Pel(Conc, Pot, D)
	Variable Conc, Pot, D
	
	Variable Dsi 			= 1e-9 * D	
	Variable No 			= MolarityToNumberDensity(Conc)	// ions /m3			// 6 * 10+25
	Variable kT 			= 298 * cBoltzmann											// 4 * 10-21
	Variable DebyeSqr 	= (2*No*cElectron^2)/(80*cPermittivity*kT)
	Variable Debye 		= sqrt(DebyeSqr)
	
	print "Debye",Debye
	
	Variable Debye2 		= 1/(ReciprocalDebyeLength(Conc,1,80,298))
	
	print "Debye2",Debye2

	Variable Prefactor 	= 64*No*kT
	
	Variable TanhPot 		= tanh((cElectron*Pot)/kt)
	
	return Prefactor * TanhPot^2 * exp(-Debye2*Dsi)
End

// UNITS:

// Distance in nanometers
// The Hamaker constant in zJ = 10-21J
// Energies in eV
	
// *!*!*! This Macro below must be made into a function *!*!*!	
	
//Macro MakeDVLOPotential(DVLOName,Hamaker, SurfaceCharge, Temperature, CIConc, Radius)
//	Variable /G gHamaker, gSurfaceCharge, gTemperature, gCIConc, gRadius
//	String /G gDVLOName
//	//
//	String DVLOName = gDVLOName
//	Prompt DVLOName, "Name the DVLO potential data."
//	Variable Hamaker = gHamaker
//	Prompt Hamaker, "The Hamaker constant [zJ]"
//	Variable SurfaceCharge = gSurfaceCharge
//	Prompt SurfaceCharge, "The surface charge density [C/m2]"
//	Variable Temperature = gTemperature
//	Prompt Temperature, "The Temperature [K]"
//	Variable CIConc = gCIConc
//	Prompt CIConc, "The counterion concentration [M]"
//	Variable Radius = gRadius
//	Prompt Radius, "The particle radius [nm]"
//	PauseUpdate; Silent 1
//	
//	gDVLOName 		= DVLOName
//	gHamaker 			= Hamaker
//	gSurfaceCharge 	= SurfaceCharge
//	gTemperature 		= Temperature
//	gCIConc 			= CIConc
//	gRadius 			= Radius
//	
//	Print " *** Calculating the DLVO Potential curves for particles in an aqueous solution of symmetrical monovalent electrolyte at",Temperature,"K. "
//	Print " 			Spherical particle radius	=",Radius ,"nm. "
//	Print " 			Surface charge density 		=",SurfaceCharge,"mC/m^2"
//	Print " 			Counterion concentration 	=",CIConc*1000,"mM"
//	Print " 			Hamaker constant 			=",Hamaker,"zJ"
//	Print " 			...........	Bjerrum length 	=",(1e9) * BjerrumLength(78, Temperature),"nm"
//	Print " 			...........	Debye length 		=",(1e9)/ReciprocalDebyeLength(CIConc,1,78,Temperature),"nm"
//	Print " 			...........	Surface potential 	=",GrahameSurfacePotential(SurfaceCharge,CIConc,1,78,Temperature),"V"
//	
//	Make /O/D/N=3001 DVLO_Distance, DVLO_Energy, DVLO_Coulomb, DVLO_Waals
//	DVLO_Distance[] = 0.02*x 
//	
//	DVLO_Waals[] 	= DVLOWaals(Hamaker,Radius,DVLO_Distance[p],Temperature)
//	
//	DVLO_Coulomb[] 	= DVLOCoulomb(SurfaceCharge,CIConc,Radius,DVLO_Distance[p],Temperature)
//	
//	DVLO_Energy 		= DVLO_Coulomb + DVLO_Waals
//	
//	// Now adopt the curves. 
//	if (ItemsInList(FunctionList("UnloadSpectrumPackage",";","")) == 0)
//		Execute "LoadSpectrumAnalysisPackage()"
//		DoUpdate
//		Print " *** Try again!"
//		return 0
//	endif
//	
//	String cmd, AxisName, DataName, DataFolderName = ":"
//	
//	DataName = ReplaceString("_data",DVLOName,"") + "_data"
//	AxisName = ReplaceString("_data",DVLOName,"") + "_axis"
//	Duplicate /O DVLO_Energy, $DataName
//	Duplicate /O DVLO_Distance, $AxisName
//	AdoptLocalAxisAndData($AxisName,$DataName)
//	KillWaves /Z $DataName
//	
//	DataName = ReplaceString("_data",DVLOName,"") + "_Cmb_data"
//	Duplicate /O DVLO_Coulomb, $DataName
//	AdoptLocalAxisAndData($AxisName,$DataName)
//	KillWaves /Z $DataName
//	
//	DataName = ReplaceString("_data",DVLOName,"") + "_vdW_data"
//	Duplicate /O DVLO_Waals, $DataName
//	AdoptLocalAxisAndData($AxisName,$DataName)
//	KillWaves /Z $AxisName, $DataName
//End

// 	Input: Concentration in moles per liter
//	Output: Number density in number per cubic meter
Function MolarityToNumberDensity(Concentration)
	Variable Concentration
	
	Variable LiterToM3 	= 1e3
	
	return cAvogadro * Concentration * LiterToM3
End

// Returns the Bjerrum length in meters
Function BjerrumLength(RelEps, T)
	Variable RelEps, T
	
	Variable LB 	= (cElectron)^2 / (4 * pi * RelEps*cPermittivity*cBoltzmann*T)
	
	return LB
End

// *** Correct only for symmetric electrolytes!
Function ReciprocalDebyeLength(CIConc,z,RelEps,T)
	Variable CIConc		// Counterion concentration, M
	Variable z				// Counterion valence
	Variable RelEps		// Relative permittivity
	Variable T				// Temp., in K
	
	// Counterion density, in number per m3. 
	Variable NRho = MolarityToNumberDensity(CIConc)
	
	// Bjerrum length, in meters
	Variable LB 	= BjerrumLength(RelEps, T)
	
	// Young-Shin correction
	Variable Kappa = sqrt(4 * pi * LB * 2 * NRho * z^2)
	
	return Kappa // in units of m-1
End

// Uses the Grahame Eqn to convert surface charge density to surface potential
Function GrahameSurfacePotential(sigma,CIConc,z,RelEps,T)
	Variable sigma		// Surface change density in C m-2
	Variable CIConc		// Counterion concentration, M
	Variable z				// Counterion valence
	Variable RelEps		// Relative permittivity
	Variable T				// Temp., in K
	
	Variable kT = cBoltzmann * T
	
	CIConc	= max(0.00001,CIConc)
	
	// Counterion density, in number per m3. 
	Variable NRho = MolarityToNumberDensity(CIConc)
	
	Variable InvSinhTerm = asinh(sigma/sqrt(8*kT*NRho*RelEps*cPermittivity))
	
	Variable Potential = kT/(2*z*cElectron) * InvSinhTerm
	
//	Print " *** Potential at the surface is",Potential,"Volts"
	
	return Potential // in units of volts = J/C
End

// Calculation of sphere-sphere interaction energies from Elimelech (3.21)
Function DVLOCoulomb(Sigma,CIConc,a,h,T)
	Variable Sigma,CIConc,a,h,T
	
	Variable kT 			= cBoltzmann * T
	Variable a_nm 		= a * 1e-9	// radius
	Variable h_nm 		= h * 1e-9	// separation
	Variable RelEpsWater	= 78			// relative permittivity of water
	
	// Kappa = inverse Debye Length, in m-1
	Variable Kappa = ReciprocalDebyeLength(CIConc,1,RelEpsWater,T)
	
	// Find the surface potential, in volts = J/C
	Variable Phi = GrahameSurfacePotential(Sigma,CIConc,1,RelEpsWater,T)
	
	// Calculate gamma
	Variable G = tanh(cElectron*Phi/(4*kT))
	
	// *** Remove the h from 1/(h + 2a)
	Variable Vc = (64 * pi) * (cPermittivity * RelEpsWater) * ((a_nm^2)/(h_nm + 2*a_nm)) * (kT/cElectron)^2 * G^2 * exp(-Kappa*h_nm)
	
	// Return the Coulomb interaction energy in thermal units
	return Vc/kT
End
	
// Hamaker interactions for small sphere-sphere interactions
// 	Input: 	Hamaker, in zepto-Joules. 1 zJ = 1e-21 J. 
Function DVLOWaals(Hamaker,a,h,T)
	Variable Hamaker,a,h,T
	
	Variable kT = cBoltzmann * T
	
	Variable A_J = (Hamaker * 1e-21)
	
	Variable a_nm = a * 1e-9
	Variable h_nm = h * 1e-9
	
	Variable Term1 = (h_nm^2 + 4*a_nm*h_nm)
	Variable Term2 = (h_nm^2 + 4*a_nm*h_nm + 4*a_nm^2)
	
	Variable Vw = -(A_J/6)*((2*a_nm^2)/Term1 + (2*a_nm^2)/Term2 + ln((Term1)/(Term2)))
	
	// Return the van der Waals interaction energy in thermal units
	return Vw/kT
End












// Hamaker interactions for large spheres. 
Function oldDVLOWaals(Hamaker,a,h,T)
	Variable Hamaker,a,h,T
		
	Variable zJ = 1e-21
	
	Variable e = 1.602176e-19
	Variable kT = 1.380650e-23 * T
	
	Variable Vw = -(Hamaker * zJ * a)/(6*h)
	
	return Vw/kT
End

// Calculation of sphere-sphere interaction energies from Elimelech (3.21)
//Function oldDVLOCoulomb(Charge,CIConc,a,h,T)
//	Variable CIConc,Charge,a,h,T
//	
//	MakePhysicalConstants()
//	NVAR e=gElectron
//	NVAR eps = gPermittivity
//	NVAR kB=gBoltzmann
//	NVAR NA=gAvogadro
//	
//	Variable kT = kB * T
//	Variable RelEpsWater = 78			// relative permittivity of water
//	
//	// K = inverse Debye Length, in nm-1
//	Variable K = ReciprocalDebyeLength(CIConc,1,RelEpsWater,T)
//	
//	// This expression comes from Israelachvilli (12.37)
////	Variable K = sqrt(CIConc)/.304	
//
//	// First find the local ion density associated with the surface charge
//	Variable Rho_surfacepart = Charge^2/(2*RelEpsWater*eps*298*kB*NA*1000)
//	
//	Variable Rho_bulkpart = CIConc
//	
//	Variable SurfaceRho = Rho_surfacepart + Rho_bulkpart
//	
//	Variable RatioRho = SurfaceRho/CIConc
//	
//	// Calculate the surface potential
//	Variable Phi = ln(RatioRho) * (kT/e)
//	
//	Variable Yo = (e*Phi)/kT
//	
////	Variable G = tanh(Yo/4)
//	Variable G = tanh(e*Phi/(4*kT))
//	
//	// Calculate the Coulomb potential
////	Variable Vc = (32 * pi * eps * a * (kT/e)^2 * G^2) * exp(-K*h)
//	// Note that a is in "nm"
//	Variable Vc = (64 * pi * kT * a * CIConc * (G^2/K^2) * exp(-K*h)
//	
//	// Return the Coulomb interaction energy in eV
//	return Vc/kT
//End
