#pragma rtGlobals=1		// Use modern global access method.

// WIdeFlag just makes a wider plot wondow for Tyler's very long acquisition data sets
Function PlotSelectedData(PlotAllFlag,WideFlag)
	Variable PlotAllFlag, WideFlag
	
	WAVE /T DataList 	= root:SPECTRA:wDataList
	WAVE DataSel 	 	= root:SPECTRA:wDataSel
	WAVE DataGroup 	= root:SPECTRA:wDataGroup
	
//	if (DataFolderExists("root:WinGlobals"))
//		NewDataFolder /O root:WinGlobals
//	endif
	
	NVAR PlotAssd 				= root:SPECTRA:Plotting:gPlotAssociatedFlag	
	Variable UserLegend 			= NumVAROrDefault("root:SPECTRA:Plotting:gFileNameLegendFlag",0)
	Variable PlotTime 			= NumVAROrDefault("root:SPECTRA:Plotting:gPlotvsTimeFlag",0)
	
	Variable i=0, DataNum, Num2Plot, AppendFlag = 0, timeFlag=0
	String DataNote, DataFileName, DataType, Folder2DName, Data2DName, Data2DFull, LegendText=""
	String AxisName, AxisAndFolderName, DataName, DataAndFolderName, PlotName, FolderStem = "root:SPECTRA:Data:Load"
	
	WaveStats /Q/M=0 DataSel
	Num2Plot = (PlotAllFlag == 1) ? V_npnts : V_sum
	
	do
		// Bit 0 or 3 is set: Normal selection. 
		if (((DataSel[i] & 2^0) != 0) || ((DataSel[i] & 2^3) != 0) || (PlotAllFlag == 1))
			
			DataNum 				= i
			DataName				= DataList[DataNum]
			DataAndFolderName	= FolderStem + num2str(DataGroup[DataNum]) + ":" + DataName
			DataNote 				= Note($DataAndFolderName)
			DataFileName 			= ReturnTextBeforeNthChar(DataNote,"\r",1)
			DataType 				= StringByKey("Data type",DataNote,"=","\r")
			
			if (PlotTime == 0)
				AxisName 				= AxisNameFromDataName(DataName)
				AxisAndFolderName	= AxisNameFromDataName(DataAndFolderName)
			else
				AxisName 				= AnyNameFromDataName(DataName,"time")
				AxisAndFolderName	= AnyNameFromDataName(DataAndFolderName,"time")
				if (!WaveExists($AxisAndFolderName))
					Print " 	*** No data-point acquisition times were loaded for",DataName
					return 0
				endif
			endif
			
			Folder2DName 		= FolderStem + num2str(DataGroup[DataNum]) + ":TwoD"
			Data2DName 			= AnyNameFromDataName(DataName,"2D")
			Data2DFull 			= ParseFilePath(2,Folder2DName,":",0,0) + Data2DName
			
			if (WaveExists($Data2DFull))
				Plot2DData(Data2DFull,DataNote)
			else
				Display /K=1/W=(525,44,1234,707)  $DataAndFolderName vs $AxisAndFolderName as "Plotted Data"
				ControlBar 180
				PlotName = WinName(0,65)
				
				MakePlotDataFolders(PlotName,1)
				
				PlotDataInWindow(PlotName, DataList, DataSel, DataGroup, "red", PlotAllFlag, AppendFlag, 0, 1, PlotTime, PlotAssd,"",UserLegend)
//				if (cmpstr(DataType,"Complex") == 0)
//					PlotDataInWindow(PlotName, DataList, DataSel, DataGroup, "red", PlotAllFlag, AppendFlag, 1, 1, PlotTime, PlotAssd,"")
//				endif

				
				if (WideFlag)
					CheckWindowPosition(PlotName,0,44,2000,707)
				else
					CheckWindowPosition(PlotName,525,44,1234,707)
				endif
				
				timeFlag = (PlotTime && WideFlag) ? 1 : 0
				SetCommonPlotFormat(PlotName,timeFlag)
				
				TransferPlotPreferences(PlotName,1)
				
				AddPlotFormatControls(PlotName)
				
				UpdateTraceStyle(PlotName,"_all_")
				
				PlotControlButtons("ReplaceCursorsButton")
	
				if (PlotTime && WideFlag)
					// some formatting and display tweaks for wide display of time-axis data
					ModifyGraph /W=$PlotName dateInfo(bottom)={0,1,0}, grid(bottom)=1,nticks(bottom)=12
				endif
				
				return 0
			endif
		endif
		
		i+=1
	while(i<numpnts(DataSel))
End

Function MakePlotDataFolders(PlotName,KillFolder)
	String PlotName
	Variable KillFolder
	
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	String CopyFolderName 	= "root:SPECTRA:Plotting:"+PlotName+":DataDuplicates"
	String PrefsFolderName	= "root:SPECTRA:Plotting:Preferences"
	String MaskFolderName	= "root:SPECTRA:Plotting:Mask"
	
	if (KillFolder)
		KillDataFolder /Z $(PlotFolderName)
	endif
	NewDataFolder/O $(PlotFolderName)
	NewDataFolder/O $(CopyFolderName)
	NewDataFolder/O $(MaskFolderName)
	NewDataFolder/O $(PrefsFolderName)
	
	MakeVariableIfNeeded(PrefsFolderName + ":gLabelPrefs",1)
	MakeVariableIfNeeded(PrefsFolderName + ":gRangePrefs",0)
	MakeVariableIfNeeded(PrefsFolderName + ":gStylePrefs",0)
	MakeVariableIfNeeded(PrefsFolderName + ":gT70Prefs",0)
End

Function TransferPlotPreferences(PlotName,TransferToPlotFlag)
	String PlotName
	Variable TransferToPlotFlag
	
	String SourceFolder,DestinationFolder
	
	if (TransferToPlotFlag == 1)	// Applying the preferences
		SourceFolder		= "root:SPECTRA:Plotting:Preferences"
		DestinationFolder	= "root:SPECTRA:Plotting:" + PlotName
	else								// Saving the preferences
		SourceFolder		= "root:SPECTRA:Plotting:" + PlotName
		DestinationFolder	= "root:SPECTRA:Plotting:Preferences"
	endif
	
	DuplicateAllVarsInDataFolder(SourceFolder,DestinationFolder,"Csr",1)
	DuplicateAllVarsInDataFolder(SourceFolder,DestinationFolder,"Prefs",1)
	
	NVAR gLabelPrefs	= $(SourceFolder+":gLabelPrefs")
	if (gLabelPrefs == 1)
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Label")
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Unit")
		
		NVAR gNoYLabelFlag			= $(DestinationFolder + ":gNoYLabelFlag")
		NVAR gNoY2LabelFlag			= $(DestinationFolder + ":gNoY2LabelFlag")
		
		NoLabelControl("NoYLabelCheckBox",gNoYLabelFlag)
		NoLabelControl("NoY2LabelCheckBox",gNoY2LabelFlag)
	endif
	
	NVAR gRangePrefs	= $(SourceFolder+":gRangePrefs")
	if (gRangePrefs == 1)
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Plot")
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Log")
		
		NVAR gLogYLeft			= $(DestinationFolder + ":gLogYLeft")
		NVAR gLogX				= $(DestinationFolder + ":gLogX")
		NVAR gLogYRight			= $(DestinationFolder + ":gLogYRight")
		
		NVAR gPlotXMin			= $(DestinationFolder + ":gPlotXMin")
		NVAR gPlotXMax			= $(DestinationFolder + ":gPlotXMax")
		
		if (TransferToPlotFlag == 1)
			SetLogAxis("AXES_LogYLeftCheckBox",gLogYLeft)
			SetLogAxis("AXES_LogXCheckBox",gLogX)
			SetLogAxis("RAXES_LogYRightCheckBox",gLogYRight)
			
			SetVariable AXES_MinXAxisSetVar,win=$PlotName,value= gPlotXMin
			SetVariable AXES_MaxXAxisSetVar,win=$PlotName,value= gPlotXMax
		endif
	endif
	
	NVAR gStylePrefs	= $(SourceFolder+":gStylePrefs")
	if (gStylePrefs == 1)
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Trace")
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Marker")
		TransferStrsAndVars(SourceFolder,DestinationFolder,"Portrait")
	endif
	
	NVAR gT70Prefs	= $(SourceFolder+":gT70Prefs")
	if (gT70Prefs == 1)
		TransferStrsAndVars(SourceFolder,DestinationFolder,"T70")
	endif
	
	// Always transfer axis calibration preferences
	TransferStrsAndVars(SourceFolder,DestinationFolder,"AxisTrue")
	TransferStrsAndVars(SourceFolder,DestinationFolder,"AxisValue")
End

// *************************************************************
// ****		Plot Error Bars - quite general! 
// *************************************************************
Function DisplayErrorBars(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String PlotName = WinName(0,65)
	
	String SampleName, DataName, AxisName, Suffix, DataAndFolderName, AxisAndFolderName, FitAndFolderName, ResAndFolderName, FolderName, AxisSigName, DataSigName,FitCmptsList
	Variable i, j, ErrorsFlag=1, NumData = ItemsInList(TraceNameList(PlotName, ";", 1))
	
	for (i=0;i<NumData;i+=1)
		DataName				= NameOfWave(WaveRefIndexed(PlotName,i,1))
		AxisName				= ReplaceString("_data",DataName,"_axis")
		Suffix 					= ReturnLastSuffix(DataName,"_")
		
		// Neglect other kinds of associated waves. 
		if (cmpstr("data",Suffix) == 0)
			SampleName			= SampleNameFromDataName(DataName)
			FolderName			= GetWavesDataFolder(WaveRefIndexed(PlotName,i,1), 1)
			DataAndFolderName 	= GetWavesDataFolder(WaveRefIndexed(PlotName,i,1), 2)
			AxisAndFolderName 	= ReplaceString("_data",DataAndFolderName,"_axis")
			WAVE Axis 			= $AxisAndFolderName
			
			// Control display of Error Bars
			if (StrSearch(ctrlName,"ErrorsCheckBox",-1) > -1)
				AxisSigName		= SampleName+"_axis_sig"
				DataSigName		= SampleName+"_data_sig"
				WAVE AxisErrors	= $(CheckFolderColon(FolderName)+AxisSigName)
				WAVE DataErrors	= $(CheckFolderColon(FolderName)+DataSigName)
				
				if (checked == 0)
					ErrorBars /W=$PlotName $DataName OFF
				else
					if ((waveexists(DataErrors) == 0) && (waveexists(AxisErrors) == 0))
						ErrorsFlag = 0
					elseif ((waveexists(DataErrors) == 1) && (waveexists(AxisErrors) == 0))
						Duplicate /O/D DataErrors, $(FolderName+AxisSigName)
						WAVE AxisErrors	= $(FolderName+AxisSigName)
						AxisErrors = 0
					elseif ((waveexists(DataErrors) == 0) && (waveexists(AxisErrors) == 1))
						Duplicate /O/D AxisErrors, $(FolderName+DataSigName)
						WAVE DataErrors	= $(FolderName+DataSigName)
						DataErrors = 0
					endif
					
					if ((checked == 1) && (ErrorsFlag != 0))
						ErrorBars /W=$PlotName $DataName XY, wave =(AxisErrors,AxisErrors),wave=(DataErrors,DataErrors)
					endif
				endif
			
			// Control display of fit curve
			elseif (cmpstr(ctrlName,"AXES_DisplayFitsCheckBox") == 0)
				FitAndFolderName 	= ReplaceString("_data",DataAndFolderName,"_fit")
				WAVE Fit 				= $FitAndFolderName
				
				ControlFitDisplay(Axis,Fit,PlotName,checked,0,0,65535)
			
			// Control display of residuals
			elseif (cmpstr(ctrlName,"AXES_DisplayResCheckBox") == 0)
				ResAndFolderName 	= ReplaceString("_data",DataAndFolderName,"_res")
				WAVE Resids 			= $ResAndFolderName
				
				ControlFitDisplay(Axis,Resids,PlotName,checked,21845,21845,21845)
			
			// Control display of fit components
			elseif (cmpstr(ctrlName,"AXES_DisplayCmptsCheckBox") == 0)
				// Find associated waves
				FitCmptsList = FolderWaveList(FolderName,SampleName + "_*",";","",-1,0)
				// Remove axis, data, error. fit and resids waves
				FitCmptsList = ExclusiveWaveList(ExclusiveWaveList(ExclusiveWaveList(FitCmptsList,DataName,";"),AxisName,";"),"_sig",";")
				FitCmptsList = ExclusiveWaveList(ExclusiveWaveList(FitCmptsList,"_res",";"),"_fit",";")
				
				for (j=0;j<ItemsInList(FitCmptsList);j+=1)
					String CmptName = ParseFilePath(2,FolderName,":",0,0) + StringFromList(j,FitCmptsList)
					WAVE Cmpt 	= $(ParseFilePath(2,FolderName,":",0,0) + StringFromList(j,FitCmptsList))
					ControlFitDisplay(Axis,Cmpt,PlotName,checked,0,39321,0)
				endfor
		
			endif
		endif
	endfor
End

Function ControlFitDisplay(Axis,Fit,PlotName,checked,r,g,b)
	Wave Axis, Fit
	String PlotName
	Variable checked, r,g,b

	if (WaveExists(Fit))
		CheckDisplayed /W=$PlotName Fit
		if ((checked) && (!V_flag))
			AppendToGraph /W=$PlotName Fit vs Axis
			ModifyGraph rgb($NameOfWave(Fit))=(r,g,b)
		elseif ((!checked) && (V_flag))
			RemoveFromGraph /W=$PlotName $NameOfWave(Fit)
		endif
	endif
End

// *************************************************************
// ****		Tabs in Control Bar
// *************************************************************
Function AddPlotFormatControls(WindowName)
	String WindowName
	
	// Igor Cursor Variables
//	NewDataFolder /O/S root:WinGlobals:$WindowName
//	String /G S_CursorAInfo, S_CursorBInfo
	
	String cmd, OldDF = getDataFolder(1)
	String PlotFolderName = "root:SPECTRA:Plotting:" + WindowName
	SetDataFolder root:
	SetDataFolder $(PlotFolderName)
	
		// PLOT HOOKS
		SetWindow $WindowName, hook(PlotCursorHook)=CursorSelectTrace, hookevents=4
		
		// CONTROLS OUTSIDE THE TABS
		AppendTraceSelectionControls(WindowName)
		AppendCursorControls(WindowName,0)
		
		// CREATING A TABBED AREA IN THE CONTROL BAR
		Variable /G gTabChoice = 0
		TabControl PlotControlTab,pos={84,4},size={620,172},tabLabel(0)="Axes", tabLabel(1)="Styles"
		TabControl PlotControlTab,tabLabel(2)="Labels",tabLabel(3)="Math", tabLabel(4)="Stats"
		TabControl PlotControlTab,tabLabel(5)="X-rays",tabLabel(6)="Titration",tabLabel(7)="FTIR",tabLabel(8)="BDS"
		TabControl PlotControlTab, proc=PlotControlTabFn,value= gTabChoice
		
		// *************************************************************
		// ****		TAB 0:	Plot Axis Options
		// *************************************************************
		AppendPlotAxisControls(WindowName,0)
		
		// *************************************************************
		// ****		TAB 1:	Plot Styles Options
		// *************************************************************
		AppendPlotStyleControls(WindowName)
		
		// *************************************************************
		// ****		TAB 2:	Axis Label Options
		// *************************************************************
		
		// This also executes the plot axis label routines
		AppendPlotLabelControls(WindowName,1,1,1,1,0,0)
			
		// *************************************************************
		// ****		TAB 3:	Data Math Operations
		// *************************************************************
		AppendPlotMathControls(WindowName)
			
		// *************************************************************
		// ****		TAB 4:	Data Statistics
		// *************************************************************
		AppendPlotStatsControls(WindowName)
			
		// *************************************************************
		// ****		TAB 5:	X-ray and Diffraction Stuff
		// *************************************************************
		AppendPlotXRayControls(WindowName)
			
		// *************************************************************
		// ****		TAB 6:	Simple Titration Analysis
		// *************************************************************
		AppendPlotT70Controls(WindowName)
			
		// *************************************************************
		// ****		TAB 7:	FTIR Analysis
		// *************************************************************
		AppendPlotFTIRControls(WindowName)
			
		// *************************************************************
		// ****		TAB 8:	Broadband dielectric spectroscopy
		// *************************************************************
		AppendPlotBDSControls(WindowName)

	SetDataFolder $(OldDF)
End	

Function PlotControlTabFn(name,tab)
	String name
	Variable tab
	
	String PlotName = WinName(0,65)
	
	NVAR gTraceColorStyle		= $("root:SPECTRA:Plotting:"+PlotName+":gTraceColorStyle")
	SVAR gRightPlottedList		= $("root:SPECTRA:Plotting:"+PlotName+":gRightPlottedList")
	NVAR MaskFlag				= $("root:SPECTRA:Plotting:"+PlotName+":gUseMaskFlag")	
	NVAR gTabChoice				= $("root:SPECTRA:Plotting:"+PlotName+":gTabChoice")
	gTabChoice = tab
	
	// This does not seem to be working ... 
	Variable RAXESDisableFlag = (ItemsInList(gRightPlottedList) > 0) ? 0 : 1
	// ... so try this
	GetAxis /Q/W=$PlotName right
	RAXESDisableFlag = (V_flag==1) ? 1 : 0
	
	
	TabControlOfPlotControls(tab,0,"AXES_",PlotName,0)
	TabControlOfPlotControls(tab,0,"RAXES_",PlotName,RAXESDisableFlag)
	TabControlOfPlotControls(tab,1,"STYLE_",PlotName,0)
	TabControlOfPlotControls(tab,1,"C1STYLE_",PlotName,!(gTraceColorStyle == 1))
	TabControlOfPlotControls(tab,1,"C3STYLE_",PlotName,!(gTraceColorStyle == 3))
	TabControlOfPlotLabelControls(tab,2,0,0,0,1,PlotName)
	TabControlOfPlotControls(tab,3,"MATH",PlotName,0)
	
	// The Masking controls appear in the 'Math' tab
	TabControlOfPlotControls(tab,3,"MASK1_",PlotName,0)
	TabControlOfPlotControls(tab,3,"MASK2_",PlotName,(MaskFlag == 0))
	TabControlOfPlotControls(tab,3,"MASK3_",PlotName,1)
	//
	TabControlOfPlotControls(tab,4,"STATS_",PlotName,0)
	TabControlOfPlotControls(tab,5,"XRAY_",PlotName,0)
	TabControlOfPlotControls(tab,6,"T70",PlotName,0)
	TabControlOfPlotControls(tab,7,"FTIR",PlotName,0)
	TabControlOfPlotControls(tab,8,"BDS",PlotName,0)
End

Function TabControlOfPlotControls(ChosenTab,HostTab,ControlStr,WindowName,DisableOrGreyFlag)
	Variable ChosenTab,HostTab, DisableOrGreyFlag
	String ControlStr, WindowName
	
	String ControlList 	= ControlNameList(WindowName,";","*"+ControlStr+"*")
	Variable DisableFlag 	= (ChosenTab == HostTab) ? DisableOrGreyFlag : 1
	
	ModifyControlList ControlList, win=$WindowName, disable=DisableFlag
End

// *************************************************************
// ****		Selecting Displayed Traces for Further Operations. TWO WAYS
// *************************************************************
Function AppendTraceSelectionControls(WindowName)
	String WindowName
	
	String cmd, PlotFolderName = "root:SPECTRA:Plotting:"+WindowName
	String /G gLeftPlottedList 	= AxisTraceListBG(WindowName, "left","data")
	String /G gRightPlottedList	= "" 
	String /G gPlottedDisplayList1 = "_none_;_all_;"+gLeftPlottedList
	String /G gPlottedDisplayList2 = gLeftPlottedList
	String /G gSelection1 = "_all_"
	String /G gSelection2	= StringFromList(0,gLeftPlottedList)
	Variable /G gPlottedSelect1Num = 1
	
	Variable mode1=2, mode2=1, NumTraces = ItemsInList(TraceNameList(WindowName,";",1))
	
	if (NumTraces > 1)
		mode2 			= 2
		gSelection2 	= StringFromList(1,TraceNameList(WindowName,";",1))
	endif
	
	PopupMenu TraceMenu1,fSize=11,pos={115,30},proc=TraceSelectionMenuProcs,title="#1",mode=mode1
	SetControlGlobalVariable(WindowName,"TraceMenu1","root:SPECTRA:Plotting","gPlottedDisplayList1",1)
	
	PopupMenu TraceMenu2,fSize=11,pos={400,30},proc=TraceSelectionMenuProcs,title="#2",mode=mode2
	SetControlGlobalVariable(WindowName,"TraceMenu2","root:SPECTRA:Plotting","gPlottedDisplayList2",1)
	
	//  Reverting to Original data and Removing a trace
	Button RemoveDataButton,pos={88,27},size={19,28},fstyle=9,fColor=(65535,0,0),proc=TraceRemovalButton,title="X"
End

Function TraceRemovalButton(ctrlname):ButtonControl
	String ctrlname
	
	String PlotName = WinName(0,65)
	SVAR gSelection1 			= $("root:SPECTRA:Plotting:"+PlotName+":gSelection1")
	SVAR gLeftPlottedList 	= $("root:SPECTRA:Plotting:"+PlotName+":gLeftPlottedList")

	if ((cmpstr(gSelection1,"_none_") == 0) || (cmpstr(gSelection1,"_all_") == 0))
		return 1
	else
		RemoveWavesInListFromPlot(PlotName, gSelection1+";")
		UpdatePlottedTraceLists(PlotName)
		
		if (ItemsInList(gLeftPlottedList) == 0)
			CloseTopPlotCleanly()
		else								// Reset the displayed trace Selection 1 to the first trace
			UpdateTraceSelection(0)
		endif
	endif
End


Function CatchCursorPositionChanges(PlotName,Csr)
	String PlotName, Csr
	
	WAVE Trace 			= CsrWaveRef($Csr,PlotName)
	if (!WaveExists(Trace))
		return 0
	endif
	
	NVAR gCsrTime 	= $("root:SPECTRA:Plotting:"+PlotName+":gCsr"+Csr+"Time")
	
	Variable Point, TimeStart, TimeCsr
	
	// This does not work!! Igor does not update the S_CursorInfo globals early enough.
//	SVAR CursorInfo 	= $("root:WinGlobals:"+PlotName+":S_Cursor"+Csr+"Info")
//	Point 				= str2num(StringByKey("POINT",CursorInfo))
	
	// So just calculate it from the Cursor commands. 
	TimeCsr 	= CsrXWaveRef($Csr,PlotName)[xcsr($Csr)]
	TimeStart 	= str2num(StringByKey("AcquisitionStart",note(Trace),"=","\r"))
	
	gCsrTime 	= TimeCsr - TimeStart
	
	ControlUpdate /W=$PlotName $("Csr"+Csr+"TimeValue")
End
	
	
	
//Function CatchCursorPositionChanges(PlotName)
//	String PlotName
//
//	SVAR CursorAInfo 	= $("root:WinGlobals:"+PlotName+":S_CursorAInfo")
//	SVAR CursorBInfo 	= $("root:WinGlobals:"+PlotName+":S_CursorBInfo")
//	
//	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
//	NVAR gCsrATime 		= $(PlotFolderName+":gCsrATime")
//	NVAR gCsrBTime 		= $(PlotFolderName+":gCsrBTime")
//	
//	String TraceNameA 	= StringByKey("TNAME",CursorAInfo)
//	String TraceNameB 	= StringByKey("TNAME",CursorBInfo)
//
//	Variable PointA 		= str2num(StringByKey("POINT",CursorAInfo))
//	Variable PointB 		= str2num(StringByKey("POINT",CursorBInfo))
//	
//	WAVE TraceA 			= CsrWaveRef(A,PlotName)
//	WAVE TraceB 			= CsrWaveRef(B,PlotName)
//	
//	
//	
//	gCsrATime 			= CsrXWaveRef(A,PlotName)[PointA]
//	gCsrBTime 			= CsrXWaveRef(B,PlotName)[PointB]
//	
//	
//	
//	Variable XA, XB
//	
//	XA 			= XWaveRefFromTrace(PlotName,TraceNameA)[PointA]
//	XB 			= XWaveRefFromTrace(PlotName,TraceNameA)[PointB]
//	
//	gCsrATime 			= XWaveRefFromTrace(PlotName,TraceNameA)[PointA]
//	gCsrBTime 			= XWaveRefFromTrace(PlotName,TraceNameA)[PointB]
//	
//	WAVE AxisA 	= XWaveRefFromTrace(PlotName,TraceNameA)
//	WAVE AxisB 	= XWaveRefFromTrace(PlotName,TraceNameB)
//	
//End

// All the plot panel hooks are in this badly-named routine. 
Function CursorSelectTrace(H_Struct) 
	STRUCT WMWinHookStruct &H_Struct 

	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	NVAR gPlottedSelect1Num		= $(PlotFolderName+":gPlottedSelect1Num")

	if (StrSearch(PlotName,"Graph",0) > -1)
		PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
		
		Variable keyCode 		= H_Struct.keyCode
		Variable eventCode	= H_Struct.eventCode
		Variable Modifier 		= H_Struct.eventMod
		Variable CmdBit 		= 3
		Variable CursorMove 	= 0
		
		// 
		if (eventCode == 7)								// A cursor was moved
				CatchCursorPositionChanges(PlotName,"A")
				CatchCursorPositionChanges(PlotName,"B")
				
				return 1
//		elseif ((keyCode==28) || (keyCode==29))			// Left or Right arrow keys MAY have moved a cursor
//			if ((CsrIsOnPlot(PlotName,"A") == 1) || (CsrIsOnPlot(PlotName,"B") == 1))
//				CursorMove = 1
//			endif
		endif
		
		
		// Using up and down arrow keys to change displayed trace
		if (keyCode ==30)
			UpdateTraceSelection(max(0,gPlottedSelect1Num-1))
			return 1
			
		elseif (keyCode == 31)
			UpdateTraceSelection(gPlottedSelect1Num+1)
			return 1
		endif
		
		// Hiding "h" and Showing "s" the plot panels for tidiness
		if ((keyCode == 72) || (keyCode == 104))
//			print keyCode
//			return 1
			
			ModifyGraph /W=$PlotName expand=0.5
			ShowHideAllControls(PlotName,1)
//			ControlBar 0
			return 1
			
		elseif ((keyCode == 83) || (keyCode == 115))	// S or s
			// Check whether the Apple Command key is also pressed. 
//			print keyCode
//			return 1
			
			if ((Modifier & 2^CmdBit) == 0)
//				ControlBar 180
				ModifyGraph /W=$PlotName expand=1
				ShowHideAllControls(PlotName,0)
				PlotControlTabFn("PlotControlTab",1)
				return 1
			else
				return 0	 	// Allow Igor to save on Cmd-S
			endif
		
		elseif (keyCode == 49)	// The number 1
			AddTagToCursorPosition(PlotName)
			return 1
			
		elseif ((keyCode == 76) || (keyCode == 108))
			return 0
		endif
		
		// Killing the window cleanly
		if (eventCode == 2)
			CloseTopPlotCleanly()
			return 1
		endif
	endif
	
	return 0
End

Function AddTagToCursorPosition(PlotName)
	String PlotName
	
	if (CsrIsOnPlot(PlotName,"A") == 0)
		return 0
	endif
	
	Variable tagX, tagSecs
	String tagText, tagName, tagFile, tagDate, tagTime
	
	WAVE Data 	= CsrWaveRef(A,PlotName)
	
	tagX 		= GetCursorPositionOrValue(PlotName,"A",0)
	tagName 	= CsrWave(A,PlotName)+"_"+num2str(tagX)
	tagFile 	= ReturnTextBeforeNthChar(note(Data),"\r",1)
	
	tagSecs 	= str2num(WaveNoteStrByKey(note(Data),"AcquisitionStart"))
	
	tagDate 	= secs2date(tagSecs,0)
	tagTime 	= secs2time(tagSecs,2)
		
	tagText 	= "File = "+tagFile+"\r"
	tagText	 	+= "Acqn time = "+tagDate+" "+tagTime+"\r"
	
	Tag/C/N=$tagName/F=0/L=1/TL=0/W=$PlotName $CsrWave(A,PlotName), tagX,tagText
End

Function TraceSelectionMenuProcs(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	String PlotName = WinName(0,65)
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	
	Variable CsrA	= min(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
	Variable CsrB	= max(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
	
	if (cmpstr("TraceMenu1",ctrlName) == 0)
		SVAR gSelection1 	= $("root:SPECTRA:Plotting:"+PlotName+":gSelection1")
		
		// First UnHighlight the current selection
		NVAR gTabChoice					= $("root:SPECTRA:Plotting:"+PlotName+":gTabChoice")
		NVAR gTraceThick					= $("root:SPECTRA:Plotting:"+PlotName+":gTraceThick")
		NVAR gMarkerThick				= $("root:SPECTRA:Plotting:"+PlotName+":gMarkerThick")
		SVAR gPlottedDisplayList1		= $("root:SPECTRA:Plotting:"+PlotName+":gPlottedDisplayList1")
		NVAR gPlottedSelect1Num		= $("root:SPECTRA:Plotting:"+PlotName+":gPlottedSelect1Num")
		
		Variable PrevSelection1Num = WhichListItem(gSelection1,gPlottedDisplayList1)
		if (PrevSelection1Num > 1)
			UpdateSingleTrace(PlotName,gSelection1,"LineThickness",gTraceThick,"")
			UpdateSingleTrace(PlotName,gSelection1,"MarkerThickness",gMarkerThick,"")
		endif
		
		gSelection1 =  popStr
		gPlottedSelect1Num	= WhichListItem(popStr,gPlottedDisplayList1)
		
		if (gPlottedSelect1Num > 1)
			UpdateSingleTrace(PlotName,popStr,"LineThickness",gTraceThick+1,"")
			UpdateSingleTrace(PlotName,popStr,"MarkerThickness",gMarkerThick+1,"")
		endif
		
//		if (gTabChoice == 4) 	// Redundant here I think. 
//			CalculateTraceStatistics(PlotName,gSelection1)
//		endif
			
	elseif (cmpstr("TraceMenu2",ctrlName) == 0)
		SVAR gSelection2 	= $("root:SPECTRA:Plotting:"+PlotName+":gSelection2")
		gSelection2 =  popStr
		
	endif
End

Function UpdateTraceSelection(TraceNum)
	Variable TraceNum

	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	Variable CsrA	= min(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
	Variable CsrB	= max(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
	
	NVAR gTabChoice				= $("root:SPECTRA:Plotting:"+PlotName+":gTabChoice")
	NVAR gTraceThick				= $("root:SPECTRA:Plotting:"+PlotName+":gTraceThick")
	NVAR gMarkerThick			= $("root:SPECTRA:Plotting:"+PlotName+":gMarkerThick")
	NVAR gPlottedSelect1Num	= $("root:SPECTRA:Plotting:"+PlotName+":gPlottedSelect1Num")
	SVAR gSelection1				= $("root:SPECTRA:Plotting:"+PlotName+":gSelection1")
	SVAR gPlottedDisplayList1	= $("root:SPECTRA:Plotting:"+PlotName+":gPlottedDisplayList1")
	
	// First UnHighlight the previous selection ... 
	Variable PrevSelection1Num = WhichListItem(gSelection1,gPlottedDisplayList1)
	if (PrevSelection1Num > 1)
		UpdateSingleTrace(PlotName,gSelection1,"LineThickness",gTraceThick,"")
		UpdateSingleTrace(PlotName,gSelection1,"MarkerThickness",gMarkerThick,"")
	endif
	
	gPlottedSelect1Num = min(TraceNum,ItemsInList(gPlottedDisplayList1)-1)
	gSelection1 = StringFromList(gPlottedSelect1Num,gPlottedDisplayList1)
	
	// ... and highlight the new selection
	if (gPlottedSelect1Num > 1)
		UpdateSingleTrace(PlotName,gSelection1,"LineThickness",gTraceThick+1,"")
		UpdateSingleTrace(PlotName,gSelection1,"MarkerThickness",gMarkerThick+1,"")
	endif
	
	PopupMenu TraceMenu1,mode=(gPlottedSelect1Num+1),win=$PlotName; DoUpdate
	
	if (gTabChoice == 4)
		CalculateTraceStatistics(PlotName,gSelection1)
	endif
	
	// If there are any mask values in the plot folder, update the mask onto the axis of the selected trace. 
	DisplayMaskBeneathData(PlotName,PlotFolderName,gSelection1)
End

// *************************************************************
// ****		Cursor Controls
// *************************************************************
Function AppendCursorControls(PlotName,CursorsOnly)
	String PlotName
	Variable CursorsOnly
	
	GroupBox CsrGroupBox,pos={2,19},size={81,109},fColor=(39321,1,1),title="cursors"
	
	Button CsrFullButton,fsize=12,pos={5,35},size={76,20},proc=PlotCursorsButtons,title="Full range"
	Button CsrSaveButton,fsize=12,pos={5,58},size={32,20},proc=PlotCursorsButtons,title="Save"
	Button CsrLastButton,fsize=12,pos={39,58},size={42,20},proc=PlotCursorsButtons,title="Recall"
	
	PopupMenu CsrTypeMenu,fSize=12,pos={10,102},size={38,20},proc=ChooseCursorType,title="   type"
	PopupMenu CsrTypeMenu,mode=0,popvalue="",value= #"\"symbols;crosshairs;\""
	
	if (!CursorsOnly)
		Button ExportTopPlotbutton,pos={7,153},size={70,20},proc=ExportTopPlot,title="Export"
	endif
	
	// Create Value displays that are only visible in the "Wide Plots" showing cursor information
	GroupBox CsrTimeInfoBox,pos={707,113},size={167,64},fColor=(39321,1,1),title="Cursor positions"
	
	Variable /G gCsrATime=0, gCsrBTime=0
	ValDisplay CsrATimeValue pos={710,130},size={150,17}, title="Time at A",fSize=12
	ValDisplay CsrBTimeValue pos={710,154},size={150,17}, title="Time at B",fSize=12
	SetControlGlobalVariable(PlotName,"CsrATimeValue","root:SPECTRA:Plotting","gCsrATime",2)
	SetControlGlobalVariable(PlotName,"CsrBTimeValue","root:SPECTRA:Plotting","gCsrBTime",2)
End

Function PlotCursorsButtons(ctrlname):ButtonControl
	String ctrlname
	
	String PlotName = WinName(0,65)
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	String PrefsFolderName	= "root:SPECTRA:Plotting:Preferences"
	
	SVAR gSelection1 	= $(PlotFolderName+":gSelection1")
	String CsrTraceName 	= gSelection1
		
	if (cmpstr(ctrlname,"CsrFullButton")==0)
		FullDisplayedRange(PlotName,1)
		return 1
		
	elseif (cmpstr(ctrlname,"CsrSaveButton")==0)
		Variable /G $(PlotFolderName+":gCsrAValue")=GetCursorPositionOrValue(PlotName,"A",1)
		Variable /G $(PlotFolderName+":gCsrBValue")=GetCursorPositionOrValue(PlotName,"B",1)
		return 1
		
	elseif (cmpstr(ctrlname,"CsrLastButton")==0)
		// Find the saved range from the Plot folder ... 
		NVAR gCsrAValue	= $(PlotFolderName+":gCsrAValue")
		NVAR gCsrBValue	= $(PlotFolderName+":gCsrBValue")
		
		// ... or the Preferences folder
		if ((NVAR_Exists(gCsrAValue) == 0) || (NVAR_Exists(gCsrBValue) == 0))
			NVAR gCsrAValue	= $(PrefsFolderName+":gCsrAValue")
			NVAR gCsrBValue	= $(PrefsFolderName+":gCsrBValue")
		endif
		
		if ((cmpstr("_all_",gSelection1) == 0) || (cmpstr("_none_",gSelection1) == 0))
			CsrTraceName 	= StringFromList(0,TraceNameList(PlotName,";",1))
		endif
		Cursor /W=$PlotName A $CsrTraceName BinarySearch(XWaveRefFromTrace(PlotName,CsrTraceName),gCsrAValue)
		Cursor /W=$PlotName B $CsrTraceName BinarySearch(XWaveRefFromTrace(PlotName,CsrTraceName),gCsrBValue)
		return 1
		
	endif
End

Function ChooseCursorType(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	Cursor /M/L=(popNum-1)/H=(popNum-1)/S=(2*(popNum-1)) A
	Cursor /M/L=(popNum-1)/H=(popNum-1)/S=(2*(popNum-1)) B
End

// *************************************************************
// ****		TAB 0:	Plot Axis Options
// *************************************************************
Function AppendPlotAxisControls(PlotName,RangeOnly)
	String PlotName
	Variable RangeOnly

	// *** Need to first calculate the axis ranges
	SetAxis /A; DoUpdate
	
	GetAxis /Q/W=$PlotName left
	Variable LeftInc = (V_max - V_min)/100
	Variable OffsetIncInc = (V_max-V_min)/200
	
	Variable /G gPlotLeftMin=V_min, gPlotLeftMax=V_max
	
	GetAxis /Q/W=$PlotName bottom
	Variable XInc = (V_max - V_min)/200
	
	Variable PlotXMin=NumVarOrDefault("gPlotXMin",V_min), PlotXMax=NumVarOrDefault("gPlotXMax",V_max)
	
	// Make Global variables in the current data folder = plot data folder. 
	Variable /G gPlotXMin=PlotXMin, gPlotXMax=PlotXMax, gPlotRightMin=NAN, gPlotRightMax=NAN, gPlotRightLeftScale
			
	SetAxis bottom, gPlotXMin, gPlotXMax
	SetAxis /A=2 left
	
	// 	Axis range group boxes
	GroupBox AXES_GroupLeft,pos={90,56},size={139,85},fColor=(39321,1,1),title="Left Axis"
	GroupBox AXES_GroupLeft1,pos={235,56},size={139,85},fColor=(39321,1,1),title="X Axis"
	GroupBox RAXES_GroupRight,pos={379,56},size={146,85},fColor=(39321,1,1),title="Right Axis", disable=1
	
	SetVariable AXES_MaxLeftAxisSetVar,limits={-Inf,Inf,LeftInc}, fSize=12,pos={98,71},size={106,19},bodyWidth=86,proc=VaryPlotAxisRange,title="Max", format="%0.2e", value=gPlotLeftMax
	SetVariable AXES_MinLeftAxisSetVar,limits={-Inf,Inf,LeftInc}, fSize=12,pos={101,91},size={103,19},bodyWidth=86,proc=VaryPlotAxisRange,title="Min",format="%0.2e", value= gPlotLeftMin	
	
	SetVariable AXES_MaxXAxisSetVar,limits={-Inf,Inf,XInc}, fSize=12,pos={243,71},size={106,19},bodyWidth=86,proc=VaryPlotAxisRange,title="Max",format="%0.2e", value= gPlotXMax
	SetVariable AXES_MinXAxisSetVar,limits={-Inf,Inf,XInc}, fSize=12,pos={246,91},size={103,19},bodyWidth=86,proc=VaryPlotAxisRange,title="Min",format="%0.2e", value= gPlotXMin
	
	SetVariable RAXES_MaxRightAxisSetVar,pos={387,71}, fSize=12,size={106,19},bodyWidth=86,proc=VaryPlotAxisRange,title="Max", format="%0.2e",value=gPlotRightMax, disable=1
	SetVariable RAXES_MinRightAxisSetVar,pos={390,91}, fSize=12,size={103,19},bodyWidth=86,proc=VaryPlotAxisRange,title="Min", format="%0.2e",value=gPlotRightMin, disable=1
	
	SetVariable RAXES_ScaleRightAxisSetVar,limits={0,Inf,1},pos={384,113}, fSize=12,size={95,15}, disable=1
	SetVariable RAXES_ScaleRightAxisSetVar,proc=VaryPlotAxisRange,title="% Left", value=gPlotRightLeftScale, disable=1

	// Axis AutoScale buttons
	Button AXES_FullScaleLeftButton,pos={207,71}, fSize=24,proc=FullAxisScale,size={18,40},title="\\F'Wingdings'"
	Button AXES_FullScaleXButton,pos={352,71}, fSize=24,proc=FullAxisScale,size={18,40},title="\\F'Wingdings'"
	Button RAXES_FullScaleRightButton,pos={496,70}, fSize=24,proc=FullAxisScale,size={18,40},title="\\F'Wingdings'", disable=1
	
	
	
//	MakeVariableIfNeeded("root:SPECTRA:Plotting:DRSFitPanel:gRealImag",1)
//	NVAR gRealImag = root:SPECTRA:Plotting:DRSFitPanel:gRealImag
//	CheckBox AXES_RICheckBox1,pos={10,15},proc=DRSRealOrImagCheckProc,title="Real",fSize=14,mode=1, value=(gRealImag==1)
//	CheckBox AXES_RICheckBox2,pos={65,15},proc=DRSRealOrImagCheckProc,title="Imaginary",fSize=14,mode=1, value=(gRealImag==2)
	
	// Disable automatic scaling of left or right axes when x-axis range is changed
	Variable /G gFixLeftAxis, gFixRightAxis
	CheckBox AXES_FixLeftCheckBox,pos={107,114}, fSize=12,size={72,14},title="fix scale", variable=gFixLeftAxis
	CheckBox RAXES_FixRightCheckBox,pos={484,124}, fSize=12,size={78,14},title="fix", variable=gFixRightAxis, disable=1
	
	Button AXES_CsrScaleXButton,pos={265,113},proc=FullAxisScale,size={60,20},title="cursors"
	
	// Log-Linear Axis selections
	Variable /G gLogYLeft, gLogX, gLogYRight
	CheckBox AXES_LogYLeftCheckBox,pos={190,114}, fSize=12,size={72,14},proc=SetLogAxis,title="log", variable=gLogYLeft
	CheckBox AXES_LogXCheckBox,pos={333,114}, fSize=12,size={64,14},proc=SetLogAxis,title="log", variable=gLogX
	CheckBox RAXES_LogYRightCheckBox,pos={484,111}, fSize=12,size={78,14},proc=SetLogAxis,title="log", variable=gLogYRight, disable=1
	
	// Add a horizontal zero line
	Variable /G gHorzZero
	CheckBox AXES_HorzZeroCheckBox,pos={500,130}, fSize=12,size={72,14},title="0",proc=SetLogAxis, variable=gHorzZero
	
	if (RangeOnly)
		return 1
	endif

	// Stacking controls
	Variable/G gStackInc = OffsetIncInc*5, gStackNum = 0,  gStackGroup=1 //gTraceCycleNum = 1
	
	SetVariable AXES_SetStackNumVar,limits={-Inf,Inf,1},value= gStackNum, pos={92,149},size={84,19},proc=SetStackNumVar,fsize=13,title="Stack"
	SetVariable AXES_SetStackIncVar,limits={-Inf,Inf,OffsetIncInc},value= gStackInc, pos={183,149},size={150,19},fsize=13,title="offset"
	SetVariable AXES_SetStackGroupVar,limits={-Inf,Inf,1},fsize=13,pos={342,149},size={92,19},proc=VaryPlotAxisRange,title="Group",value= gStackGroup
	
//	Variable /G gTimeStackFlag				// Display Error Bars
//	CheckBox AXES_TimeStackCheckBox,pos={440,151},fsize=12,size={103,15},proc=TimeStack,title="Time stack?", value=gTimeStackFlag
	
	// Disable automatic scaling of left or right axes when x-axis range is changed
	Variable /G gFixLeftAxis, gFixRightAxis
	CheckBox AXES_FixLeftCheckBox,pos={107,114}, fSize=12,size={72,14},title="fix scale", variable=gFixLeftAxis
	CheckBox RAXES_FixRightCheckBox,pos={484,124}, fSize=12,size={78,14},title="fix", variable=gFixRightAxis, disable=1

	// Display a vertical line at a specified axis position
	Variable/G gBarNumber=1, gBar1Position=NAN, gBar2Position=NAN, gBar3Position=NAN, gBar4Position=NAN
	SetVariable AXES_BarNumberSetVar,pos={533,149},limits={1,4,1}, format=" ", fSize=12,size={18,17},proc=SetBarNumber,title=" ", value=gBarNumber
	SetVariable AXES_BarPositionSetVar,pos={555,149},limits={-Inf,Inf,XInc/5},  fSize=12,size={118,15},proc=SetPositionBar,title="Vert 1", value=gBar1Position
	Button AXES_RemoveBarButton,fsize=13, pos={675,150},proc=RemovePositionBar,size={16,16},title="x"
	
	// Display a HORIZONTAL line position
	Variable/G gHBarNumber=1, gHBar1Position=NAN, gHBar2Position=NAN, gHBar3Position=NAN, gHBar4Position=NAN
	SetVariable AXES_HBarNumberSetVar,pos={533,128},limits={1,4,1}, format=" ", fSize=12,size={18,17},proc=SetBarNumber,title=" ", value=gHBarNumber
	SetVariable AXES_HBarPositionSetVar,pos={554,128},limits={-Inf,Inf,XInc/5}, fSize=12,size={119,15},proc=SetPositionBar,title="Horiz 1", value=gHBar1Position
	Button AXES_RemoveHBarButton,fsize=13, pos={675,129},proc=RemovePositionBar,size={16,16},title="x"
	
	Variable /G gErrorsFlag				// Display Error Bars
	CheckBox AXES_ErrorsCheckBox,pos={564,57},fsize=12,size={103,15},proc=DisplayErrorBars,title="Display errors", value=gErrorsFlag
	
	Variable /G gDisplayFitsFlag			// Display any associated fit waves
	CheckBox AXES_DisplayFitsCheckBox,pos={564,74},fsize=12,size={103,15},proc=DisplayErrorBars,title="Display fit results", value=gDisplayFitsFlag
	
	Variable /G gDisplayResFlag			// Display any associated fit waves
	CheckBox AXES_DisplayResCheckBox,pos={564,91},fsize=12,size={103,15},proc=DisplayErrorBars,title="Display residuals", value=gDisplayResFlag
	
	Variable /G gDisplayCmptsFlag			// Display any associated fit waves
	CheckBox AXES_DisplayCmptsCheckBox,pos={564,108},fsize=12,size={103,15},proc=DisplayErrorBars,title="Display components", value=gDisplayCmptsFlag
End

Function LargeLabelsCheckBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	if (checked == 1)
		ModifyGraph axThick=4, fSize=24
//		ModifyGraph axThick=4, fSize=32
	else
		ModifyGraph axThick=2, fSize=18
//		ModifyGraph axThick=2, fSize=24
	endif
End

Function PortraitPlotCheckBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	if (checked == 1)
		ModifyGraph margin(right)=259
	else
		ModifyGraph margin(right)=0
	endif
End

// *************************************************************
// ****		Setting Axis Ranges
// *************************************************************
Function VaryPlotAxisRange(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	Variable NumLeftTraces, AutoScaleRightAxis = 0
	Variable LogLeftFlag 		= NumberByKey("log(x)", AxisInfo(PlotName,"left"),"=")
	Variable LogRightFlag 	= NumberByKey("log(x)", AxisInfo(PlotName,"right"),"=")
	
	NVAR gFixLeftAxis		= $(PlotFolderName + ":gFixLeftAxis")
	NVAR gFixRightAxis		= $(PlotFolderName + ":gFixRightAxis")
	
	NVAR gPlotLeftMin		= $(PlotFolderName + ":gPlotLeftMin")
	NVAR gPlotLeftMax		= $(PlotFolderName + ":gPlotLeftMax")
	NVAR gPlotXMin			= $(PlotFolderName + ":gPlotXMin")
	NVAR gPlotXMax			= $(PlotFolderName + ":gPlotXMax")
	NVAR gPlotRightMin		= $(PlotFolderName + ":gPlotRightMin")
	NVAR gPlotRightMax		= $(PlotFolderName + ":gPlotRightMax")
	NVAR gPlotRightLeftScale = $(PlotFolderName + ":gPlotRightLeftScale")
	
	Variable StackInc 			= NumVarOrDefault((PlotFolderName + ":gStackInc"),0)
	Variable StackNum 		= NumVarOrDefault((PlotFolderName + ":gStackNum"),0)
	Variable StackGroup 		= NumVarOrDefault((PlotFolderName + ":gStackGroup"),1)
	
	if (StrSearch(ctrlName,"Left",0) > -1)
		SetAxis left, gPlotLeftMin, gPlotLeftMax
		
	elseif (StrSearch(ctrlName,"XAxis",0) > -1)
		GetAxis /Q left
		SetAxis bottom, gPlotXMin, gPlotXMax
		
		if(NVAR_Exists(gFixLeftAxis) && (gFixLeftAxis == 0))
			// When changing the x-axis, automatically update the left and right ranges ...
			SetAxis /A=2 left
			
			GetAxis /Q left					// ... and remember the new range.
			gPlotLeftMin	= V_min
			gPlotLeftMax	= V_max
		else
			SetAxis left, V_min, V_max
		endif
		
		if(NVAR_Exists(gFixRightAxis) & (gFixRightAxis == 1))
			SetAxis /Z/A=2 right
		endif
		
		if ((StrSearch(AxisList(PlotName),"right",0) > -1) && (AutoScaleRightAxis == 1))
			// Recall that the right axis is never stacked. 
			
			if(NVAR_Exists(gFixRightAxis) & (gFixRightAxis == 1))
				SetAxis /A=2 right
				
				GetAxis /Q right
				gPlotRightMin	= V_min
				gPlotRightMax	= V_max
			endif
		endif
		
	elseif (StrSearch(ctrlName,"Right",0) > -1)
		if (StrSearch(AxisList(PlotName),"right",0) > -1)
			if (cmpstr("RAXES_ScaleRightAxisSetVar",ctrlName) == 0)
				gPlotRightMin	= gPlotLeftMin*(gPlotRightLeftScale/100)
				gPlotRightMax	= gPlotLeftMax*(gPlotRightLeftScale/100)
			endif
			
			SetAxis right, gPlotRightMin, gPlotRightMax
		endif
		
	elseif (cmpstr("AXES_SetStackGroupVar",ctrlName) == 0)
		NVAR gStackGroup			= $(PlotFolderName + ":gStackGroup")
		
		NumLeftTraces	= ItemsInList(AxisTraceListBG(PlotName, "left",""))
		gStackGroup 		= min(varNum,NumLeftTraces)
		
	endif
End

Function FullAxisScale(B_Struct)
	STRUCT WMButtonAction &B_Struct
	
	String ctrlName 		= B_Struct.ctrlName
	String WindowName 	= B_Struct.win
	Variable eventCode 	= B_Struct.eventCode
	Variable eventMod 	= B_Struct.eventMod
	
	if (eventCode != 2)
		return 0	// Mouse up after pressing
	endif
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	NVAR gPlotXMin			= $(PlotFolderName + ":gPlotXMin")
	NVAR gPlotXMax			= $(PlotFolderName + ":gPlotXMax")
	NVAR gPlotLeftMin		= $(PlotFolderName + ":gPlotLeftMin")
	NVAR gPlotLeftMax		= $(PlotFolderName + ":gPlotLeftMax")
	NVAR gFixLeftAxis		= $(PlotFolderName + ":gFixLeftAxis")
	NVAR gFixRightAxis		= $(PlotFolderName + ":gFixRightAxis")
	
	if (cmpstr(ctrlName,"AXES_FullScaleLeftButton")==0)
		SetAxis /A=2 left
		
		if ((eventMod & 2^1) != 0)	// Bit 1 = shift key down
			GetAxis /Q left
			gPlotLeftMin 	= V_min
			gPlotLeftMax 	= V_max
		endif
		
	elseif (StrSearch(ctrlName,"ScaleX",0)>-1)
		if (cmpstr(ctrlName,"AXES_FullScaleXButton")==0)
			SetAxis /A bottom
			
			if ((eventMod & 2^1) != 0)	// Bit 1 = shift key down
				GetAxis /Q bottom
				gPlotXMin 	= V_min
				gPlotXMax 	= V_max
			endif
			
		elseif (cmpstr(ctrlName,"AXES_CsrScaleXButton")==0)
			gPlotXMin	= min(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
			gPlotXMax	= max(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
			
			SetAxis bottom, gPlotXMin, gPlotXMax
		endif
		
		if(NVAR_Exists(gFixLeftAxis) && (gFixLeftAxis == 0))
			SetAxis /A=2 left
		endif
		
		if(NVAR_Exists(gFixRightAxis) && (gFixRightAxis == 0))
			SetAxis /Z/A=2 right
		endif
		
	elseif (cmpstr(ctrlName,"RAXES_FullScaleRightButton")==0)
		SetAxis /Z/A=2 right
	endif
End

// *************************************************************
// ****		Use a HORIZONTAL OFFSET to "stack" time-dependent traces on absolute date/time axis
// 	****							*** OBSELETE ***
// *************************************************************
//Function TimeStack(CB_Struct) : CheckBoxControl 
//	STRUCT WMCheckboxAction &CB_Struct 
//	
//	Variable checked 			= CB_Struct.checked
//	String WindowName 		= CB_Struct.win
//	Variable eventCode		= CB_Struct.eventCode
//	if (eventCode != 2)
//		return 0
//	endif
//	
//	if (!checked)
//		ModifyGraph offset={0,0}
//		return 1
//	endif
//
//	String ListOfTraces, TraceName, DataNote
//	Variable i, NumTraces, IgorStart, MinStart=1e99
//	
//	ListOfTraces 	= AxisTraceListBG(WindowName, "left","")
//	NumTraces 	= ItemsInList(ListOfTraces)
//	
//	// First find the start
//	for (i=0;i<NumTraces;i+=1)
//		TraceName		= StringFromList(i,ListOfTraces)
//		WAVE Data 	= TraceNameToWaveRef(WindowName,TraceName)
//		IgorStart 		= ParseWaveNote(note(Data),"In Igor date convention")
//		if (numtype(IgorStart) == 0)
//			MinStart 		= (IgorStart < MinStart) ? IgorStart : MinStart
//		endif
//	endfor
//	
//	// Either apply or remove horizontal offset. 
//	for (i=0;i<NumTraces;i+=1)
//		TraceName		= StringFromList(i,ListOfTraces)
//		WAVE Data 	= TraceNameToWaveRef(WindowName,TraceName)
//		
//		IgorStart 	= ParseWaveNote(note(Data),"In Igor date convention")
//		if (numtype(IgorStart) == 0)
//			ModifyGraph offset($TraceName)={IgorStart-MinStart,0}
//		endif
//	endfor
//	
//	return 0
//End

Function SetStackNumVar(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	NVAR gStackInc			= $(PlotFolderName + ":gStackInc")
	NVAR gStackNum			= $(PlotFolderName + ":gStackNum")
	NVAR gStackGroup			= $(PlotFolderName + ":gStackGroup")
	NVAR gPlotLeftMin		= $(PlotFolderName + ":gPlotLeftMin")
	NVAR gPlotLeftMax		= $(PlotFolderName + ":gPlotLeftMax")
	
	Traces_Stacking(PlotName,gStackInc,gStackNum,gStackGroup)
	
	GetAxis /Q left
	gPlotLeftMin = V_min
	gPlotLeftMax = V_max
End

Function SetLogAxis(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String AxisName="null", ButtonName, PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	if (cmpstr(ctrlName,"AXES_LogYLeftCheckBox")==0)
		NVAR gLogFlag	= $(PlotFolderName + ":gLogYLeft")
		AxisName 		= "left"
		ButtonName	= "AXES_FullScaleLeftButton"
		
	elseif (cmpstr(ctrlName,"AXES_LogXCheckBox")==0)
		NVAR gLogFlag	= $(PlotFolderName + ":gLogX")
		AxisName 		= "bottom"
		ButtonName	= "AXES_FullScaleXButton"
		
	elseif (cmpstr(ctrlName,"RAXES_LogYRightCheckBox")==0)
		NVAR gLogFlag	= $(PlotFolderName + ":gLogYRight")
		AxisName 		= "right"
		ButtonName	= "RAXES_FullScaleRightButton"
	endif
	
	if (StrSearch(AxisList(""),AxisName,0) > -1)
		gLogFlag = checked
		ModifyGraph log($AxisName)=checked
	endif
	
	if (cmpstr(ctrlName,"AXES_HorzZeroCheckBox")==0)
		ModifyGraph zero(left)=(8*checked),zeroThick(left)=1
	endif
	
//	FullAxisScale(ButtonName) 
End

// *************************************************************
// ****		Vertical Position Bars
// *************************************************************
Function SetBarNumber(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	String SetVarName, BarType="Bar", BarTitle, PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	BarTitle 	= "Vert "+num2str(varNum)
	if (StrSearch(ctrlName,"HBar",0) > -1)
		BarType = "HBar"
		BarTitle 	= "Horiz "+num2str(varNum)
	endif
	
	NVAR gBarNumber	= $(CheckFolderColon(PlotFolderName) + "g" + BarType + "Number")
	gBarNumber = varNum
	
	SetVarName 	= "AXES_"+BarType+"PositionSetVar"
	NVAR gBarPosition	= $(CheckFolderColon(PlotFolderName) + "g" + BarType + num2str(gBarNumber) + "Position")
	SetVariable $SetVarName,title=BarTitle, value=gBarPosition
End

Function SetPositionBar(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	String BarType="Bar", PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	if (StrSearch(ctrlName,"HBar",0) > -1)
		BarType 	= "HBar"
	endif
	
	NVAR gBarNumber		= $(CheckFolderColon(PlotFolderName) + "g" + BarType + "Number")
	NVAR gBarPosition		= $(CheckFolderColon(PlotFolderName) + "g" + BarType + num2str(gBarNumber)  + "Position")
	
	String BarPosnName 		= BarType + num2str(gBarNumber) + "Position"
	String BarHeightName 	= BarType + num2str(gBarNumber) + "Height"
	
	WAVE BarHeight		= $(CheckFolderColon(PlotFolderName) + BarHeightName)
	if (WaveExists(BarHeight) == 0)
		Make /N=2 $(CheckFolderColon(PlotFolderName) + BarPosnName)=NAN
		Make /N=2 $(CheckFolderColon(PlotFolderName) + BarHeightName)=NAN
	endif
	
	if (StrSearch(ctrlName,"HBar",0) > -1)
		// Swap the wave assignments to change data and axis
		WAVE BarPosn		= $(CheckFolderColon(PlotFolderName) + BarHeightName)
		WAVE BarHeight		= $(CheckFolderColon(PlotFolderName) + BarPosnName)
	else
		WAVE BarPosn		= $(CheckFolderColon(PlotFolderName) + BarPosnName)
		WAVE BarHeight		= $(CheckFolderColon(PlotFolderName) + BarHeightName)
	endif
	
	gBarPosition	= varNum
	BarPosn[0] 	= gBarPosition
	BarHeight[0] 	= Inf
	BarPosn[1] 	= gBarPosition
	BarHeight[1] 	= -Inf
	
	// Revert back to normal wave assignments for plotting. 
	WAVE BarHeight		= $(CheckFolderColon(PlotFolderName) + BarHeightName)
	WAVE BarPosn		= $(CheckFolderColon(PlotFolderName) + BarPosnName)
	
	CheckDisplayed /W=$PlotName BarHeight
	if (V_flag == 0)
		AppendToGraph /W=$PlotName BarHeight vs BarPosn
		ReorderTraces $(StringFromList(0,TraceNameList(PlotName,";",1))),{$(NameOfWave(BarHeight))}
		ModifyGraph /W=$PlotName rgb($(NameOfWave(BarHeight)))=(17476,17476,17476)
//		ModifyGraph /W=$PlotName mode($(NameOfWave(BarHeight)))=1
	endif
End

// *************************************************************
// ****		Add a Shift-Click option for extracting values intersected by a horizontal or vertical bar
// *************************************************************
Function RemovePositionBar(B_Struct)
	STRUCT WMButtonAction &B_Struct
	
	String ctrlName 		= B_Struct.ctrlName
	String WindowName 	= B_Struct.win
	Variable eventCode 	= B_Struct.eventCode
	Variable eventMod 	= B_Struct.eventMod
	
	if (eventCode != 2)
		return 0	// Mouse up after pressing
	endif
	
	Variable YFlag = 0

	String msg, BarType="Bar"
	if (StrSearch(ctrlName,"HBar",0) > -1)
		YFlag 		= 1
		BarType 	= "HBar"
	endif
	
	String PlotName 		= WinName(0,65)
	String PlotFolder 		= "root:SPECTRA:Plotting:" + PlotName
	NVAR gBarNumber	= $(CheckFolderColon(PlotFolder) + "g" + BarType + "Number")
	NVAR gBarPosition	= $(CheckFolderColon(PlotFolder) + "g" + BarType + num2str(gBarNumber) + "Position")
	WAVE BarHeight		= $(CheckFolderColon(PlotFolder) + BarType + num2str(gBarNumber) + "Height")
	
	CheckDisplayed /W=$PlotName BarHeight
	if (!V_flag)
		return 0
	endif
	
	if (YFlag)
		msg 		= "Extracting trace data intersecting the horizontal bar at y = "+num2str(gBarPosition)
	else
		msg 		= "Extracting trace data intersecting the vertical bar at x = "+num2str(gBarPosition)
	endif
	
	if ((eventMod & 2^1) != 0)	// Bit 1 = shift key down
		Print  " *** 	"+msg
		WaveFromTraceIntersection(PlotName,gBarPosition, YFlag)
	else
		gBarPosition = NAN
		RemoveFromGraph /Z /W=$PlotName $(BarType + num2str(gBarNumber) + "Height")
	endif
	
	return 1
End

// *************************************************************
// ****		TAB 1:	Plot Styles
// *************************************************************
Function AppendPlotStyleControls(PlotName)
	String PlotName

	GroupBox STYLE_Group0,fColor=(39321,1,1),pos={599,50},size={101,115}, disable=1,title="Save Preferences", disable=1
	GroupBox STYLE_Group1,fColor=(39321,1,1),pos={88,51},size={227,120},title="Select Trace Style", disable=1
	GroupBox STYLE_Group2,fColor=(39321,1,1),pos={319,52},size={172,82},title="Select Subset of Traces", disable=1
	GroupBox STYLE_Group3,fColor=(39321,1,1),pos={497,52},size={95,82},title="Plot Margins", disable=1
	
	String PlotFolderName	= "root:SPECTRA:Plotting:" + PlotName
	
	MakeVariableIfNeeded(PlotFolderName+":gTraceThick",1)
	MakeVariableIfNeeded(PlotFolderName+":gMarkerSize",0)
	MakeVariableIfNeeded(PlotFolderName+":gMarkerThick",1)
	
	NVAR gTraceThick			= $(PlotFolderName+":gTraceThick")
	NVAR gMarkerSize		= $(PlotFolderName+":gMarkerSize")
	NVAR gMarkerThick		= $(PlotFolderName+":MarkerThick")
	
	// Set Variable controls
	SetVariable STYLE_TraceLineThickVar,fSize=14,pos={226,99},size={46,20},title=" ", disable=1
	SetVariable STYLE_TraceLineThickVar,limits={0,6,0.5},value= $("root:SPECTRA:Plotting:"+PlotName+":gTraceThick")
	
	SetVariable STYLE_TraceMarkerSizeVar,fSize=14,pos={181,123},size={36,20},title=" ", disable=1
	SetVariable STYLE_TraceMarkerSizeVar,limits={0,9,0.5},value= $("root:SPECTRA:Plotting:"+PlotName+":gMarkerSize")
	
	SetVariable STYLE_TraceMarkerThickVar,fSize=14,pos={226,123},size={46,20},title=" ", disable=1
	SetVariable STYLE_TraceMarkerThickVar,limits={0,6,0.5},value= $("root:SPECTRA:Plotting:"+PlotName+":gMarkerThick")
	
	// VARIABLE popup menues
	MakeVariableIfNeeded(PlotFolderName+":gTraceLine",0)
	MakeVariableIfNeeded(PlotFolderName+":gTraceStyle",1)
	MakeVariableIfNeeded(PlotFolderName+":gTraceMarker",1)
	MakeVariableIfNeeded(PlotFolderName+":gTraceColorStyle",2)
	
	NVAR gTraceLine			= $(PlotFolderName+":gTraceLine")
	NVAR gTraceStyle			= $(PlotFolderName+":gTraceStyle")
	NVAR gTraceMarker		= $(PlotFolderName+":gTraceMarker")
	NVAR gTraceColorStyle	= $(PlotFolderName+":gTraceColorStyle")
	
	PopupMenu STYLE_TraceStyleMenu,win=$PlotName,fSize=12,pos={104,75},size={112,20},proc=PlotStyleMenuProcs,title="Style"
	PopupMenu STYLE_TraceStyleMenu,mode=gTraceStyle,bodyWidth= 78,value= #"\"line;stick;dots;markers;line & markers;bars;\"", disable=1
	
	PopupMenu STYLE_TraceLineMenu,win=$PlotName,fSize=12,pos={111,98},size={105,20},proc=PlotStyleMenuProcs,title="Line"
	PopupMenu STYLE_TraceLineMenu,mode=gTraceLine,bodyWidth= 78,value= #"\"*LINESTYLEPOP*\"", disable=1
	
	PopupMenu STYLE_TraceMarkerMenu,win=$PlotName,fSize=12,pos={139,123},size={38,20},proc=PlotStyleMenuProcs,title="Marker"
	PopupMenu STYLE_TraceMarkerMenu,mode=gTraceMarker,bodyWidth= 38,value= #"\"*MARKERPOP*\"", disable=1
	
	PopupMenu STYLE_TraceColorStyleMenu,fSize=12,pos={104,147},size={109,20},proc=PlotStyleMenuProcs,title="Color", disable=1
	PopupMenu STYLE_TraceColorStyleMenu,mode=gTraceColorStyle,bodyWidth= 75,value= #"\"Single;Rainbow;Table;\""

	// STRING popup menus
	MakeStringIfNeeded(PlotFolderName+":gTraceRGB","(65535,0,0)")
	MakeStringIfNeeded(PlotFolderName+":gTraceColorTable","Rainbow16")
	MakeStringIfNeeded(PlotFolderName+":gTraceType","data")
	
	SVAR gTraceRGB			= $(PlotFolderName+":gTraceRGB")
	SVAR gTraceColorTable	= $(PlotFolderName+":gTraceColorTable")
	
	// Some infernal coding to get these color menus to display preferences!!
	PopupMenu C1STYLE_TraceColorMenu,win=$PlotName,pos={218,147},size={57,20},fSize=12,proc=PlotStyleMenuProcs,title=" "
	PopupMenu C1STYLE_TraceColorMenu,bodyWidth= 50,mode=1,value= #"\"*COLORPOP*\"", disable=1
	SetMenuControlValue(PlotName,"C1STYLE_TraceColorMenu",gTraceRGB,2)
	
	Variable m = 1 + WhichListItem(gTraceColorTable, CTabList())
	PopupMenu C3STYLE_TraceColorTableMenu,win=$PlotName,pos={218,147},size={57,20},fSize=12,proc=PlotStyleMenuProcs,title=" "
	PopupMenu C3STYLE_TraceColorTableMenu,bodyWidth= 50,mode=m,value= #"\"*COLORTABLEPOPNONAMES*\"", disable=1
	
	// Routines to select a subset of plotted traces to change styles
	PopupMenu STYLE_TraceTypeMenu,win=$PlotName,fSize=12,pos={324,68},size={123,20},proc=PlotStyleMenuProcs,title="Update"
	PopupMenu STYLE_TraceTypeMenu,mode=1,value= #"\"data;fits;residuals;backgrounds;peaks;all;match text;\"", disable=1
	
	String /G gTraceMatchStr = ""
	SetVariable STYLE_TraceMatchStr,pos={324,92},size={160,19},title="Match String", disable = 1
	SetVariable STYLE_TraceMatchStr,fSize=12,value=$("root:SPECTRA:Plotting:"+PlotName+":gTraceMatchStr")
	
	// Preferences	
	Variable /G gLabelPrefs=1, gRangePrefs=1, gStylePrefs=1, gT70Prefs=1
//	Button STYLE_GraphPrefsButton,size={70,20},pos={7,130},proc=PlotControlButtons,title="Set Prefs", disable=1
	Button GraphPrefsButton,size={70,20},pos={7,130},proc=PlotControlButtons,title="Set Prefs"
	CheckBox STYLE_LabelPrefsCheckBox,fSize=12,pos={604,90},size={86,15},title="Axis Labels",mode=0, variable=gLabelPrefs, disable=1
	CheckBox STYLE_RangePrefsCheckBox,fSize=12,pos={605,107},size={94,15},title="X-axis Range",mode=0, variable=gRangePrefs, disable=1
	CheckBox STYLE_StylePrefsCheckBox,fSize=12,pos={605,123},size={95,15},title="Trace Styles",mode=0, variable=gStylePrefs, disable=1
	CheckBox STYLE_TitrationPrefsCheckBox,fSize=12,pos={605,139},size={95,15},title="Titration",mode=0, variable=gT70Prefs, disable=1
	
	Variable /G gUpdateStyle=1, gUpdateLine=1, gUpdateMarker=1, gUpdateColor=1
	CheckBox STYLE_UpdateStyleCheckBox,fSize=12,pos={295,81},size={95,15},title=" ", variable=gUpdateStyle,disable=1
	CheckBox STYLE_UpdateLineCheckBox,fSize=12,pos={295,104},size={95,15},title=" ", variable=gUpdateLine,disable=1
	CheckBox STYLE_UpdateMarkerCheckBox,fSize=12,pos={295,128},size={95,15},title=" ", variable=gUpdateMarker,disable=1
	CheckBox STYLE_UpdateColorCheckBox,fSize=12,pos={295,151},size={95,15},title=" ", variable=gUpdateColor,disable=1
	
	Button STYLE_UpdateAllStylesButton,pos={291,63},size={20,12},fSize=9,proc=PlotControlButtons,title="all",disable=1
	Button STYLE_UpdateNoStylesButton,pos={271,63},size={20,12},fSize=9,proc=PlotControlButtons,title="no",disable=1
	
	Button STYLE_UpdatePlotStyleButton,pos={320,140},size={120,24},fSize=13,fColor=(21845,21845,21845),proc=PlotControlButtons,title="Apply Trace Style",disable=1
	
	Button STYLE_FixMarginsButton,pos={502,69},size={35,20},fSize=13,proc=PlotControlButtons,title="Fix",disable=1
	Button STYLE_AutoMarginsButton,pos={545,69},size={40,20},fSize=13,proc=PlotControlButtons,title="Auto",disable=1
	
	NVAR gPortraitFlag	= gPortraitFlag
	if (NVAR_Exists(gPortraitFlag) == 0)
		Variable /G gPortraitFlag=0
	endif
	CheckBox STYLE_PortraitPlotCheckBox title="Portrait format",size={103,40},pos={477,153},proc=PortraitPlotCheckBox,fSize=12,variable=gPortraitFlag,disable=1
	PortraitPlotCheckBox("AXES_PortraitPlotCheckBox",gPortraitFlag)
	
	Variable /G gLargeLabelsFlag=0
	CheckBox STYLE_LargeLabelsCheckBox title="Large labels",size={103,40},pos={477,135},proc=LargeLabelsCheckBox,fSize=12,variable=gLargeLabelsFlag,disable=1
End

Function PlotControlButtons(ctrlname):ButtonControl
	String ctrlname
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	SVAR gSelection1 		= $(PlotFolderName+":gSelection1")
	NVAR gPortraitFlag 	= $(PlotFolderName+":gPortraitFlag")
	
	if (cmpstr("_none_",gSelection1) == 0)
		return 0
	elseif (strlen(gSelection1) == 0)
		return 0
	endif
	
	if (cmpstr(ctrlname,"STYLE_UpdatePlotStyleButton") == 0)
		UpdateTraceStyle(PlotName,gSelection1)
		
//	elseif (cmpstr(ctrlname,"STYLE_GraphPrefsButton")==0)
	elseif (cmpstr(ctrlname,"GraphPrefsButton")==0)
		TransferPlotPreferences(PlotName,0)
		
	elseif (cmpstr(ctrlname,"STYLE_UpdateAllStylesButton")==0)
		SetAllStyleCheckBoxes(PlotFolderName,1)
		
	elseif (cmpstr(ctrlname,"STYLE_UpdateNoStylesButton")==0)
		SetAllStyleCheckBoxes(PlotFolderName,0)
		
	elseif (cmpstr(ctrlname,"STYLE_FixMarginsButton")==0)
		SetAxisMargin("left",PlotName,90)
		if (gPortraitFlag)
			SetAxisMargin("right",PlotName,259)
		else
			SetAxisMargin("right",PlotName,90)
		endif
		
	elseif (cmpstr(ctrlname,"STYLE_AutoMarginsButton")==0)
		SetAxisMargin("left",PlotName,0)
//		SetAxisMargin("right",PlotName,0)
		if (gPortraitFlag)
			SetAxisMargin("right",PlotName,259)
		else
			SetAxisMargin("right",PlotName,90)
		endif
	endif
End

Function SetAxisMargin(AxisName,PlotName,margin)
	String AxisName,PlotName
	Variable margin
	
	if (strlen(AxisInfo(PlotName,AxisName)) > 0)
		ModifyGraph margin($AxisName) = margin
	endif
	
	ModifyGraph lblMargin=5
End

Function SetAllStyleCheckBoxes(PlotFolderName,check)
	String PlotFolderName
	Variable check

	NVAR gUpdateStyle 	= $(PlotFolderName+":gUpdateStyle")
	NVAR gUpdateLine 	= $(PlotFolderName+":gUpdateLine")
	NVAR gUpdateMarker 	= $(PlotFolderName+":gUpdateMarker")
	NVAR gUpdateColor 	= $(PlotFolderName+":gUpdateColor")
	
	gUpdateStyle	 	= check
	gUpdateLine		= check
	gUpdateMarker	= check
	gUpdateColor		= check
End

Function PlotStyleMenuProcs(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	if (cmpstr("STYLE_TraceStyleMenu",ctrlName) == 0) 			// BG menu
		NVAR gTraceStyle	= $(PlotFolderName + ":gTraceStyle")
		gTraceStyle = popNum
		
	elseif (cmpstr("STYLE_TraceLineMenu",ctrlName) == 0) 		// WM menu
		NVAR gTraceLine	= $(PlotFolderName + ":gTraceLine")
		gTraceLine = popNum-1
		
	elseif (cmpstr("STYLE_TraceMarkerMenu",ctrlName) == 0) 		// WM menu
		NVAR gTraceMarker	= $(PlotFolderName + ":gTraceMarker")
		gTraceMarker = popNum-1
		
	elseif (cmpstr("STYLE_TraceColorStyleMenu",ctrlName) == 0) // BG menu
		NVAR gTraceColorStyle	= $(PlotFolderName + ":gTraceColorStyle")
		gTraceColorStyle = popNum
		
		ModifyControl C1STYLE_TraceColorMenu, win=$PlotName, disable = !(gTraceColorStyle == 1)
		ModifyControl C3STYLE_TraceColorTableMenu, win=$PlotName, disable = !(gTraceColorStyle == 3)
		
	elseif (cmpstr("C1STYLE_TraceColorMenu",ctrlName) == 0) 		// WM string menu
		SVAR gTraceRGB	= $(PlotFolderName + ":gTraceRGB")
		gTraceRGB = popStr
		
	elseif (cmpstr("C3STYLE_TraceColorTableMenu",ctrlName) == 0) // WM string menu
		SVAR gTraceColorTable	= $(PlotFolderName + ":gTraceColorTable")
		gTraceColorTable = popStr
		
	elseif (cmpstr("STYLE_TraceTypeMenu",ctrlName) == 0) 		// BG menu for selecting subset of traces
		SVAR gTraceType	= $(PlotFolderName + ":gTraceType")
		gTraceType = popStr
	endif
End

// *************************************************************
// ****		TAB 3:	Data Math Operations
// *************************************************************
Function AppendPlotMathControls(WindowName)
	String WindowName
	
	String PlotFolderName	= "root:SPECTRA:Plotting:" + WindowName
	
	GroupBox MATH_Group0 title="Normalize",fColor=(39321,1,1),pos={95,56},size={87,38}, disable = 1
	GroupBox MATH_Group1 title="Modify selected trace",fColor=(39321,1,1),pos={184,56},size={138,93}, disable = 1
	GroupBox MATH_Group2 title="Fit poly to #1, do math",fColor=(39321,1,1),pos={324,128},size={124,42}, disable = 1
	GroupBox MATH_Group3 title="Align #1 to #2",fColor=(39321,1,1),pos={449,56},size={128,87}, disable = 1
	GroupBox MATH_Group4 title="Spectrum math",fColor=(39321,1,1),pos={323,56},size={124,70}, disable = 1
	
	//  ***   Simple Math Operations
	// ------------------------------------------------------------------------------------
	Variable /G gDataAddConstant = 0.05
	SetVariable MATH_SetAddVar,fSize=13,pos={188,103},size={64,20},title=" ", disable = 1
	SetVariable MATH_SetAddVar,limits={-inf,inf,0.05},format="%1.3f",value= $(PlotFolderName+":gDataAddConstant")
	
	Button MATH_1AddButton,fSize=16,pos={260,104},size={23,18},proc=DataControlButtons,title="+", disable = 1
	Button MATH_1SubtractButton,fSize=16,pos={294,104},size={23,18},proc=DataControlButtons,title="-", disable = 1
	// ------------------------------------------------------------------------------------
	
	// ------------------------------------------------------------------------------------
	Variable /G gDataMultConstant = 1.05
	SetVariable MATH_SetMultVar,fSize=13,pos={188,127},size={64,20},title=" ", disable = 1
	SetVariable MATH_SetMultVar,limits={-inf,inf,0.01},format="%1.3f",value= $(PlotFolderName+":gDataMultConstant")
	
	Button MATH_1MultiplyButton,fSize=16,pos={260,127},size={23,18},proc=DataControlButtons,title="x", disable = 1
	Button MATH_1DivideButton,fSize=16,pos={294,127},size={23,18},proc=DataControlButtons,title="", disable = 1
	// ------------------------------------------------------------------------------------
	
	// ------------------------------------------------------------------------------------
	Variable /G gDataSmoothConstant = 1
	SetVariable MATH_SetSmoothVar,fSize=13,pos={93,125},size={44,20},title=" ", disable = 1
	SetVariable MATH_SetSmoothVar,limits={1,32767,1},value= $(PlotFolderName+":gDataSmoothConstant")
	
	Button MATH_1SmoothButton,pos={141,124},size={40,20},proc=DataControlButtons,title="Smth", disable=1
	// ------------------------------------------------------------------------------------
	
	// ------------------------------------------------------------------------------------
	Variable /G gSelectionNum = 1
	CheckBox MATH_SelectionRadio1,pos={190,68},size={61,15},proc=SelectionNumCheckProc,title="#1 or",fSize=12,mode=1, value=1, disable=1
	CheckBox MATH_SelectionRadio2,pos={251,68},size={47,15},proc=SelectionNumCheckProc,title="#2",fSize=12,mode=1, value=0, disable=1
	
	Variable /G gOperandNum = 1
	CheckBox MATH_OperandRadio1,pos={190,85},proc=OperandNumCheckProc,title="Data",fSize=12,mode=1, value=1, disable=1
	CheckBox MATH_OperandRadio2,pos={251,85},proc=OperandNumCheckProc,title="Axis",fSize=12,mode=1, value=0, disable=1
	// ------------------------------------------------------------------------------------
	
	//  ***   Other Math Operations
	Button MATH_AreaNormButton,pos={99,71},size={35,18},proc=DataControlButtons,title="Area", disable=1
	Button MATH_IntNormButton,pos={144,71},size={35, 18},proc=DataControlButtons,title="Int", disable=1
	
	Button MATH2_DerivativeButton,pos={99,100},size={40,20},proc=DataControlButtons,title="d/dx", disable=1
	Button MATH2_IntegrateButton,pos={144,100},size={16,20},proc=DataControlButtons,title="", disable=1
	Button MATH2_IntegrateDownButton,pos={164,100},size={16,20},proc=DataControlButtons,title="", disable=1

	Button MATH_MoreMathButton,pos={99,150},size={80,18},proc=DataControlButtons,title="More Math", disable=1
	
	//  ***   Buttons for Io or Polynomial Removal
	Button MATH_PolyFitButton,pos={329,142},size={54,22},fColor=(32769,65535,32768),proc=IzeroButtons,title="Poly BG", disable=1
	
	Button MATH_AlignButton,pos={452,72},size={54,20},fColor=(65535,65534,49151),proc=IzeroButtons,title="Align", disable=1
	
	//  ***   Reverting to Original data. Undo scaling, shifting and truncation
	Button MATH_RevertButton,pos={515,72},size={54,20},fColor=(65535,65535,65535),proc=DataControlButtons,title="Undo", disable=1
	
	Button MATH_ManualNormButton,pos={384,72},size={54,20},fColor=(32768,65535,65535),proc=IzeroButtons,title="Manual", disable=1
	
	Variable /G gPolyOrder=0
	SetVariable MATH_PolyOrderVar,pos={390,144},size={50,18},title="#",fSize=12
	SetVariable MATH_PolyOrderVar,limits={0,9,1},value= gPolyOrder, disable=1
	
	// OBSELETE: CheckBoxes to determine which simple math operation to perform on 2 spectra
	Variable /G gPlotIoDiff = 0, gPlotIoMult = 0, gPlotIoRatio = 0, gIoRatioMinus1=0, gOverwriteFlag=0, gAutoKeepFlag=0
	CheckBox MATH_PlotIoDiffCheck,pos={325,69},size={166,15},title="1 - 2",fSize=12,proc=IzeroCheckProcs,variable= gPlotIoDiff, disable=1
	CheckBox MATH_PlotIoMultCheck,pos={325,86},size={166,15},title="1 x 2",fSize=12,proc=IzeroCheckProcs,variable= gPlotIoMult, disable=1
	CheckBox MATH_PlotIoRatioCheck,pos={325,103},size={54,15},title="1/2 - 1",fSize=12,proc=IzeroCheckProcs,variable= gPlotIoRatio, disable=1
	CheckBox MATH_IoRatioMinus1Check,pos={392,104},size={54,15},title=" ",fSize=12,variable= gIoRatioMinus1, disable=1
	
	String /G gSpectrumMath = "1 - 2"
//	PopupMenu MATH_SpecMath,fSize=12,pos={325,100},proc=SpectrumMathPopupProcs,title=" ",mode=1, disable=1
//	PopupMenu MATH_SpecMath, popmatch=gSpectrumMath, value="1 - 2;1 + 2;1 x 2;1/2;1/2 - 1;1 interp 2;"
	
	// *** 	"Keeping" the results means adopting them. 
	Button MATH_KeepNormButton,pos={452,147},size={54,20},proc=IzeroButtons,title="Adopt", disable=1
	CheckBox MATH_AutoKeepCheck,pos={524,150},size={54,15},title="Auto",fSize=12,variable= gAutoKeepFlag, disable=1
	
	// Allow only a partial range of the spectrum to be affected by any transformation. 
	Variable /G gCsrRangeChoice = 1
	PopupMenu MATH_CsrRangeChoiceMenu,fSize=12,pos={198,151},size={112,20},proc=MathLimitsMenuProc,title="limits"
	PopupMenu MATH_CsrRangeChoiceMenu,mode=gCsrRangeChoice,bodyWidth= 78,value= #"\"full;cursors;truncate;\"", disable=1
	
	// *** 	"Saving" the results means exporting as a text file. NO SPACE ANYMORE!
//	Button MATH_SaveNormButton,pos={456,130},size={40,20},fColor=(65535,49151,62258),proc=IzeroButtons,title="Save", disable=1
//	CheckBox MATH_OverWriteCheck,pos={503,132},size={54,15},title="Overwrite",fSize=12,variable= gOverwriteFlag, disable=1
	// ************************************************************************
	
	// *** Shifting the trace along the axis using interpolation
	Variable /G gAxisOffset = 0 			// This is the INCREMENTAL offset applied to the saved original axis
	GetAxis /Q bottom
	Variable AxisOffsetInc = (V_max - V_min)/1000
	SetVariable MATH_SetAxisOffsetVar,pos={451,97},size={102,18},proc=SetAxisOffset,title="Shift",fSize=12
	SetVariable MATH_SetAxisOffsetVar,limits={-inf,inf,AxisOffsetInc},format="%1.4f",value= gAxisOffset, disable=1
	
	Variable /G gFitAxisOffsetFlag = 0	// The option to FIX or FIT the axis shift offset
	CheckBox MATH_AlignCheck5,pos={559,98},size={34,15},title=" ",fSize=12,variable= gFitAxisOffsetFlag, disable=1
//	CheckBox MATH_AlignCheck5,pos={457,120},size={34,15},title="Find x-shift",fSize=12,variable= gFitAxisOffsetFlag, disable=1
	
	Variable /G gFitScaleFactorFlag = 1	// The option to FIX or FIT the intensity scale factor
	CheckBox MATH_AlignCheck6,pos={458,123},size={34,15},title="Fit scale factor",fSize=12,variable= gFitScaleFactorFlag, disable=1
	// ************************************************************************
	
	// *** Masking Routines
	Variable /G gUseMaskFlag = 0
	CheckBox MASK1_PLOT_MaskCheck,pos={600,58},size={74,15},title="Mask",proc=MaskCheckProc,fSize=14, disable=1
	
	Button MASK2_PLOT_EditMaskButton,pos={588,80},size={106,18},proc=MaskRegionButtons,title="Edit", disable=1
	Button MASK2_PLOT_MaskPoly2DataButton,pos={588,100},size={106,18},proc=MaskRegionButtons,title="Poly to Data", disable=1
	
	Button MASK3_PLOT_UpdateMaskButton,pos={588,80},size={52,18},proc=MaskRegionButtons,title="Update", disable=1
	Button MASK3_PLOT_CancelMaskButton,pos={642,80},size={52,18},proc=MaskRegionButtons,title="Cancel", disable=1
	Button MASK3_PLOT_MaskButton,pos={588,100},size={52,18},proc=MaskRegionButtons,fsize=12,title="Mask", disable=1
	Button MASK3_PLOT_UnMaskButton,pos={642,100},size={52,18},proc=MaskRegionButtons,fsize=12,title="Unmask", disable=1
	Button MASK3_PLOT_KeepButton,pos={588,120},size={52,18},proc=MaskRegionButtons,fsize=12,title="Keep", disable=1
	Button MASK3_PLOT_SaveButton,pos={642,120},size={52,18},proc=MaskRegionButtons,fsize=12,title="Save", disable=1
	Button MASK3_PLOT_RecallButton,pos={588,140},size={52,18},proc=MaskRegionButtons,fsize=12,title="Recall", disable=1
	Button MASK3_PLOT_ImportButton,pos={642,140},size={52,18},proc=MaskRegionButtons,fsize=12,title="Import", disable=1
	// ************************************************************************
End

// OBSELETE - but leave them active for backwards compatibility. 
Function IzeroCheckProcs(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	SVAR gSelection1 	= $(PlotFolderName+":gSelection1")
	SVAR gSelection2 	= $(PlotFolderName+":gSelection2")
	
	if (cmpstr(ctrlName,"MATH_PlotIoDiffCheck") == 0)
		NVAR CheckVar	= $(PlotFolderName+":gPlotIoDiff")
		if (checked == 0)
			RemoveFromGraph /Z DataMinusBG, polynomial
		endif
	elseif (cmpstr(ctrlName,"MATH_PlotIoMultCheck") == 0)
		NVAR CheckVar	= $(PlotFolderName+":gPlotIoMult")
		if (checked == 0)
			RemoveFromGraph /Z DataMultiplyBG, polynomial
		endif
	elseif (cmpstr(ctrlName,"MATH_PlotIoRatioCheck") == 0)
		NVAR CheckVar	= $(PlotFolderName+":gPlotIoRatio")
		if (checked == 0)
			RemoveFromGraph /Z DataDivideBG, polynomial
		endif
	endif
	MirrorLeftAxisIfRightAxisUnused(PlotName)
End

Function SpectrumMathPopupProcs(PU_Struct) : PopupMenuControl 
	STRUCT WMPopupAction &PU_Struct 
	
	Variable popNum 			= PU_Struct.popNum
	Variable eventCode 		= PU_Struct.eventCode
	String popStr 				= PU_Struct.popStr
	String ctrlName 			= PU_Struct.ctrlName
	
	if (eventCode!=2)
		return 0
	endif
	
	RemoveFromGraph /Z SpectrumMath, DataMinusBG, DataMultiplyBG, DataDivideBG, polynomial
End

Function MathLimitsMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	String PlotName = WinName(0,65)
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	NVAR gCsrRangeChoice	= $("root:SPECTRA:Plotting:"+PlotName+":gCsrRangeChoice")
	
	gCsrRangeChoice = popNum
End

Function SelectionNumCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String PlotName = WinName(0,65)
	NVAR gSelectionNum		= $("root:SPECTRA:Plotting:"+PlotName+":gSelectionNum")
	
	gSelectionNum = ReturnLastNumber(ctrlName)
	
	CheckBox MATH_SelectionRadio1,value=(gSelectionNum==1)
	CheckBox MATH_SelectionRadio2,value=(gSelectionNum==2)
End

Function OperandNumCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String PlotName = WinName(0,65)
	NVAR gOperandNum		= $("root:SPECTRA:Plotting:"+PlotName+":gOperandNum")
	
	gOperandNum = ReturnLastNumber(ctrlName)
	
	CheckBox MATH_OperandRadio1,value=(gOperandNum==1)
	CheckBox MATH_OperandRadio2,value=(gOperandNum==2)
End

Function RealOrImagCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String PlotName = WinName(0,65)
	NVAR gRealImag		= $("root:SPECTRA:Plotting:"+PlotName+":gRealImag")
	
	gRealImag = ReturnLastNumber(ctrlName)
	
	CheckBox AXES_RICheckBox1,value=(gRealImag==1)
	CheckBox AXES_RICheckBox2,value=(gRealImag==2)
	
	ModifyGraph cmplxMode=(gRealImag)
End

Function DataControlButtons(ctrlname):ButtonControl
	String ctrlname
	
	String PlotName = WinName(0,65)
	String PrefsFolderName 	= "root:SPECTRA:Plotting:Preferences"
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	String CopyFolderName 	= "root:SPECTRA:Plotting:"+PlotName+":DataDuplicates"
	
	NVAR gOperandNum 		= $(PlotFolderName+":gOperandNum")
	NVAR gSelectionNum 		= $(PlotFolderName+":gSelectionNum")
	NVAR gAxisOffset 			= $(PlotFolderName+":gAxisOffset")
	NVAR gCsrRangeChoice 	= $(PlotFolderName+":gCsrRangeChoice")
	SVAR gSelection 			= $(PlotFolderName+":gSelection"+num2str(gSelectionNum))
	
	String ExportStyle, SmoothName, MathOpName, TraceName, AxisName
	Variable NumTraces, TraceIndex,  AllTracesFlag, i, MathOpFlag, NumConst, CsrAValue, CsrBValue, MinPt,MaxPt
			
	if (StrSearch(ctrlname,"MATH_1",0) > -1)
		if (cmpstr("_none_",gSelection) == 0)
			return 0
		elseif (strlen(gSelection) == 0)
			return 0
		endif
		
		if (cmpstr(ctrlname,"MATH_1AddButton")==0)
			NVAR	gDataAddConstant	= $(PlotFolderName+":gDataAddConstant")
			NumConst		= gDataAddConstant
			MathOpName 	= "Addition"
		elseif (cmpstr(ctrlname,"MATH_1SubtractButton")==0)
			NVAR 	gDataAddConstant	= $(PlotFolderName+":gDataAddConstant")
			NumConst		= -gDataAddConstant
			MathOpName 	= "Addition"
		elseif (cmpstr(ctrlname,"MATH_1MultiplyButton")==0)
			NVAR	gDataMultConstant	= $(PlotFolderName+":gDataMultConstant")
			NumConst		= gDataMultConstant
			MathOpName 	= "Multiplication"
		elseif (cmpstr(ctrlname,"MATH_1DivideButton")==0)
			NVAR	gDataMultConstant	= $(PlotFolderName+":gDataMultConstant")
			NumConst		= gDataMultConstant
			MathOpName 	= "Division"
		elseif (cmpstr(ctrlname,"MATH_1SmoothButton")==0)
			NVAR	gDataSmoothConstant	= $(PlotFolderName+":gDataSmoothConstant")
			NumConst		= gDataSmoothConstant
			MathOpName 	= "Smooth"
		endif
		
		// OK, we must be performing some simple Math operations on one or all traces. 
		AllTracesFlag = (cmpstr("_all_",gSelection) == 0) ? 1 : 0
	
		NumTraces = ItemsInList(TraceNameList(PlotName,";",1))
		for (i=0;i<NumTraces;i+=1)
			
			WAVE Trace 	= WaveRefIndexed(PlotName,i,1)
			TraceName 	= NameOfWave(Trace)
			
			WAVE Axis 	= XWaveRefFromTrace(PlotName,TraceName)
			AxisName 		= NameOfWave(Trace)
			
			if (!TraceIsSpecial(TraceName))
				if ((AllTracesFlag == 1) || (cmpstr(TraceName,gSelection) == 0))
				
					// Debug .... problems when the axis values are sorted inversely (I think)
				
					if (gCsrRangeChoice == 1)
						MinPt 		= 0
						MaxPt 		= numpnts(Trace)-1

//						CsrAValue 	= -inf
//						CsrBValue 	= inf		
//						MinPt 	= GetCursorPositionOrValue(PlotName,"A",0)
//						MaxPt 	= GetCursorPositionOrValue(PlotName,"B",0)
					else
						CsrAValue 	= CsrIsOnPlot(PlotName,"A") ? GetCursorPositionOrValue(PlotName,"A",1) : 0
						CsrBValue 	= CsrIsOnPlot(PlotName,"B") ? GetCursorPositionOrValue(PlotName,"B",1) : inf
						
						MinPt = min(AxisValueToPoint(Axis,CsrAValue),AxisValueToPoint(Axis,CsrBValue))
						MaxPt = max(AxisValueToPoint(Axis,CsrBValue),AxisValueToPoint(Axis,CsrBValue))
					endif
						
//					MinPt = min(AxisValueToPoint(Axis,CsrAValue),AxisValueToPoint(Axis,CsrBValue))
//					MaxPt = max(AxisValueToPoint(Axis,CsrBValue),AxisValueToPoint(Axis,CsrBValue))
	
					if (cmpstr("Smooth",MathOpName) == 0)
						Duplicate /FREE/O/D Trace, SmoothTrace
						Smooth /B NumConst, SmoothTrace			// This can only operate on data, not axes
						Trace[MinPt,MaxPt] 	= SmoothTrace[p]
					else
						
						if (gOperandNum == 1)
							ApplyMathOpToDataAndErrors(Trace, MathOpName, NumConst,0,MinPt,MaxPt,1,0)
						else
							ApplyMathOpToDataAndErrors(Axis, MathOpName, NumConst,0,MinPt,MaxPt,1,0)
						endif
					endif
					
					// ... and update the Graph-stored duplicate copy. This can be undone by the Revert or Undo buttons. 
					CopyIndexTraceToDataFolder(PlotName,CopyFolderName,i, gOperandNum)
				endif
			endif
		endfor	
		
	else
		SVAR gSelection1 		= $(PlotFolderName+":gSelection1")
		
		if (StrSearch(ctrlname,"MATH2",0) > -1)
			// Mathematical transformations that require both axis and data. 
			if (cmpstr(ctrlname,"MATH2_DerivativeButton") == 0)
				TransformPlotAxisAndData(PlotName,gSelection1,1)
			elseif (cmpstr(ctrlname,"MATH2_IntegrateButton") == 0)
				TransformPlotAxisAndData(PlotName,gSelection1,2)
			elseif (cmpstr(ctrlname,"MATH2_IntegrateDownButton") == 0)
				TransformPlotAxisAndData(PlotName,gSelection1,3)
			endif
			
			// Do not update the saved traces in the Graph datafolder
		else
			if (cmpstr(ctrlname,"STATS_AvgButton")==0)
				AverageLeftPlottedData(PlotName)
			elseif (cmpstr(ctrlname,"STATS_InterpButton")==0)
				InterpolateLeftPlottedData(PlotName)
			elseif (cmpstr(ctrlname,"MATH_AreaNormButton")==0)
				NormalizePlottedData(PlotName,1)
			elseif (cmpstr(ctrlname,"MATH_IntNormButton")==0)
				NormalizePlottedData(PlotName,2)
			elseif (cmpstr(ctrlname,"MATH_MoreMathButton")==0)
				ApplyMathOpToPlotDataOrAxis(PlotName)
			elseif (cmpstr(ctrlname,"MATH_RevertButton")==0)
				// Replace the data and DataDuplicates with the stored Originals
				RevertPlottedData(PlotName,gSelection1)
				
				// Tidy up any fitting or alignment traces
				RemoveFromGraph /Z/W=$PlotName DataMinusBG, DataDivideBG, DataMultiplyBG, polynomial
				MirrorLeftAxisIfRightAxisUnused(PlotName)
				
				// Reset certain fitting parameters
				gAxisOffset 	= 0
			endif
			
			// Update our saved traces in the Graph datafolder
			DuplicateAllTracesToDataFolder(PlotName,CopyFolderName,"*",1)
			
		endif
	endif
End

Function IzeroButtons(B_Struct)
	STRUCT WMButtonAction &B_Struct
	
	String ctrlName 		= B_Struct.ctrlName
	String panelName 		= B_Struct.win
	Variable eventMod 	= B_Struct.eventMod
	
	Variable eventCode 	= B_Struct.eventCode
	if (eventCode != 2)
		return 0	// Mouse up after pressing
	endif
	
	Variable BGType, ShiftDown = 0
	if ((eventMod & 2^1) != 0)	// Bit 1 = shift key down
		ShiftDown = 1
	endif
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	SVAR gDisplayList			= $(PlotFolderName+":gPlottedDisplayList1")
	SVAR gSelection1 			= $(PlotFolderName+":gSelection1")
	SVAR gSelection2 			= $(PlotFolderName+":gSelection2")
	
	NVAR gAxisOffset			= $(PlotFolderName+":gAxisOffset")
	NVAR gIoRatioMinus1		= $(PlotFolderName+":gIoRatioMinus1")
	NVAR gFitAxisOffsetFlag	= $(PlotFolderName+":gFitAxisOffsetFlag")
	NVAR gFitScaleFactor		= $(PlotFolderName+":gFitScaleFactorFlag")
	NVAR gPolyOrder			= $(PlotFolderName+":gPolyOrder")
	NVAR gUseMaskFlag		= $(PlotFolderName+":gUseMaskFlag")
	NVAR gCsrRangeChoice	= $(PlotFolderName+":gCsrRangeChoice")
	NVAR gAutoKeepFlag		= $(PlotFolderName+":gAutoKeepFlag")
	
	NVAR gOverwriteFlag		= $(PlotFolderName+":gOverwriteFlag")
	NVAR gAutoNameFlag		= $(PlotFolderName+":gAutoNameFlag")
	
	String tSelection1 	= gSelection1
	String IzeroFolderName = "root:SPECTRA:Fitting:IzeroFit"

	// If only a single trace is plotted, more convenient to pass the exact name, not "all"
	if (ItemsInList(gDisplayList) == 3)
		tSelection1 = StringFromList(2,gDisplayList)
	endif
	
	if (cmpstr(ctrlname,"MATH_KeepNormButton")==0)
		KeepIzeroResults(PlotName,PlotFolderName,tSelection1,gSelection2,0)	// Actually, we don't really know what BGType is in this instance. 
		
	elseif (cmpstr(ctrlname,"MATH_ManualNormButton")==0)
		// This will be updated to handle complex waves
		ApplyManualNormMethod(PlotName,PlotFolderName,gSelection1,gSelection2,gAutoKeepFlag)

	else
		if (cmpstr(ctrlname,"MATH_PolyFitButton")==0)
			BGType 	= (ShiftDown == 1) ? 3 : 1
			BackgroundRemovalProc(PlotName,PlotFolderName,tSelection1,gSelection2,BGType,gAxisOffset,gFitAxisOffsetFlag,gFitScaleFactor,gPolyOrder,gUseMaskFlag,gCsrRangeChoice,gIoRatioMinus1,gAutoKeepFlag)
		elseif (cmpstr(ctrlname,"MATH_AlignButton")==0)
			BackgroundRemovalProc(PlotName,PlotFolderName,gSelection1,gSelection2,2,gAxisOffset,gFitAxisOffsetFlag,gFitScaleFactor,gPolyOrder,gUseMaskFlag,gCsrRangeChoice,gIoRatioMinus1,gAutoKeepFlag)
		endif
	endif
	
	DoWindow /F PlotName
End

//Function IzeroButtons(ctrlname):ButtonControl
//	String ctrlname
//	
//	String PlotName = WinName(0,65)
//	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
//	
//	SVAR gDisplayList			= $(PlotFolderName+":gPlottedDisplayList1")
//	SVAR gSelection1 			= $(PlotFolderName+":gSelection1")
//	SVAR gSelection2 			= $(PlotFolderName+":gSelection2")
//	
//	NVAR gAxisOffset			= $(PlotFolderName+":gAxisOffset")
//	NVAR gIoRatioMinus1		= $(PlotFolderName+":gIoRatioMinus1")
//	NVAR gFitAxisOffsetFlag	= $(PlotFolderName+":gFitAxisOffsetFlag")
//	NVAR gFitScaleFactor		= $(PlotFolderName+":gFitScaleFactorFlag")
//	NVAR gPolyOrder			= $(PlotFolderName+":gPolyOrder")
//	NVAR gUseMaskFlag		= $(PlotFolderName+":gUseMaskFlag")
//	NVAR gCsrRangeChoice	= $(PlotFolderName+":gCsrRangeChoice")
//	NVAR gAutoKeepFlag		= $(PlotFolderName+":gAutoKeepFlag")
//	
//	NVAR gOverwriteFlag		= $(PlotFolderName+":gOverwriteFlag")
//	NVAR gAutoNameFlag		= $(PlotFolderName+":gAutoNameFlag")
//	
//	String tSelection1 	= gSelection1
//	String IzeroFolderName = "root:SPECTRA:Fitting:IzeroFit"
//
//	// If only a single trace is plotted, more convenient to pass the exact name, not "all"
//	if (ItemsInList(gDisplayList) == 3)
//		tSelection1 = StringFromList(2,gDisplayList)
//	endif
//	
//	if (cmpstr(ctrlname,"MATH_KeepNormButton")==0)
//		KeepIzeroResults(PlotName,PlotFolderName,tSelection1,gSelection2,0)	// Actually, we don't really know what BGType is in this instance. 
//		
//	elseif (cmpstr(ctrlname,"MATH_ManualNormButton")==0)
//		ApplyManualNormMethod(PlotName,PlotFolderName,gSelection1,gSelection2,gAutoKeepFlag)
//
//	else
//		if (cmpstr(ctrlname,"MATH_PolyFitButton")==0)
//			BackgroundRemovalProc(PlotName,PlotFolderName,tSelection1,gSelection2,1,gAxisOffset,gFitAxisOffsetFlag,gFitScaleFactor,gPolyOrder,gUseMaskFlag,gCsrRangeChoice,gIoRatioMinus1,gAutoKeepFlag)
//		elseif (cmpstr(ctrlname,"MATH_AlignButton")==0)
//			BackgroundRemovalProc(PlotName,PlotFolderName,gSelection1,gSelection2,2,gAxisOffset,gFitAxisOffsetFlag,gFitScaleFactor,gPolyOrder,gUseMaskFlag,gCsrRangeChoice,gIoRatioMinus1,gAutoKeepFlag)
//		endif
//	endif
//	
//	DoWindow /F PlotName
//End
			
// *************************************************************
// ****		TAB 4:	Data Statistics
// *************************************************************
Function AppendPlotStatsControls(WindowName)
	String WindowName
	
	String cmd, PlotFolderName = "root:SPECTRA:Plotting:"+WindowName
	
	GroupBox STATS_GroupStatsY,pos={96,114},size={167,54},fColor=(39321,1,1),title="X Axis Statistics", disable=1
	GroupBox STATS_GroupStatsX,pos={96,54},size={167,54},fColor=(39321,1,1),title="Y Axis Statistics", disable=1
	GroupBox STATS_GroupStats,pos={267,54},size={255,115},fColor=(39321,1,1),title="X Axis Statistics for a Distribution P(x)", disable=1
	
	Button STATS_AvgButton,pos={590,60},size={100,18},proc=DataControlButtons,title="Average", disable=1
	Button STATS_InterpButton,pos={590,120},size={100,18},proc=DataControlButtons,title="Interpolate", disable=1
	
	// Go buttons
	Button STATS_GoButton1,pos={243,67},size={16,16},proc=StatsControlButtons,title="   ", disable=1
	Button STATS_GoButton2,pos={243,126},size={16,16},proc=StatsControlButtons,title="   ", disable=1
	Button STATS_GoButton3,pos={500,67},size={16,16},proc=StatsControlButtons,title="   ", disable=1
	
	// Calculate simple statistical measures for the Y-axis data. 
	String /G gYStatisticChoice="sum of values"
	PopupMenu STATS_YStatisticMenu pos={102,66},title="Statistic ",fSize=12, proc=DistStatsMenuProcs, disable=1
	PopupMenu STATS_YStatisticMenu,value="sum of values;mean of values;standard deviation;average;integral;integral - bg;", mode=1
	
	Variable /G gYStatistic=0
	ValDisplay STATS_YStatistic pos={101,87},size={150,17}, title="Value ",bodyWidth=110,fSize=12, disable=1
	SetControlGlobalVariable(WindowName,"STATS_YStatistic","root:SPECTRA:Plotting","gYStatistic",2)
	
	// Calculate simple statistical measures for the X-axis data. 
	String /G gXStatisticChoice="sum of values"
	PopupMenu STATS_XStatisticMenu pos={102,126},title="Statistic ",fSize=12, proc=DistStatsMenuProcs, disable=1
	PopupMenu STATS_XStatisticMenu,value="sum of values;mean of values;", mode=1
	
	Variable /G gXStatistic=0
	ValDisplay STATS_XStatistic pos={101,147},size={150,17}, title="Value ",bodyWidth=110,fSize=12, disable=1
	SetControlGlobalVariable(WindowName,"STATS_XStatistic","root:SPECTRA:Plotting","gXStatistic",2)
	
	
	// Calculate certain statistical measures for the X-axis data, assuming that the Y-axis data is actually a distribution of the form P(x)
	String /G gDistStatsChoice="mean"
	PopupMenu STATS_DistStatsMenu pos={274,66},title="Statistic ",fSize=12, proc=DistStatsMenuProcs, disable=1
	PopupMenu STATS_DistStatsMenu,value="mean;variance;std dev;", mode=1
	
	Variable /G gDistStatsVariable=0
	ValDisplay STATS_DistStatsVar pos={277,97},size={204,17}, title="Selected trace ",fSize=12, disable=1
	SetControlGlobalVariable(WindowName,"STATS_DistStatsVar","root:SPECTRA:Plotting","gDistStatsVariable",2)
	
//	Variable /G gDistMeanVariable=0
//	ValDisplay STATS_DistMeanVar pos={304,121},size={177,17},title=" All traces ",title=" ",fSize=12, disable=1
//	SetControlGlobalVariable(WindowName,"STATS_DistMeanVar","root:SPECTRA:Plotting","gDistMeanVariable",2)
//	
//	Variable /G gDistSDVariable=0
//	ValDisplay STATS_DistSDVar pos={381,145},size={100,20},title=" ",fSize=12, disable=1
//	SetControlGlobalVariable(WindowName,"STATS_DistSDVar","root:SPECTRA:Plotting","gDistSDVariable",2)
End

Function DistStatsMenuProcs(PU_Struct) : PopupMenuControl
	STRUCT WMPopupAction &PU_Struct
	
	String ctrlName 			= PU_Struct.ctrlName
	String plotName 			= PU_Struct.win
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	SVAR gSelection1 	= $(PlotFolderName+":gSelection1")
	
	if (PU_Struct.eventCode > 0)
		if (StrSearch(ctrlName,"X",0) > -1)
			SVAR gStatisticChoice 	= $(PlotFolderName + ":gXStatisticChoice")
		elseif (StrSearch(ctrlName,"Y",0) > -1)
			SVAR gStatisticChoice 	= $(PlotFolderName + ":gYStatisticChoice")
		else
			SVAR gStatisticChoice 	= $(PlotFolderName + ":gDistStatsChoice")
		endif
		
		gStatisticChoice 	= PU_Struct.popStr
	endif
	
	CalculateTraceStatistics(PlotName,gSelection1)
End

Function StatsControlButtons(ctrlname):ButtonControl
	String ctrlname
	
	String PlotName = WinName(0,65)
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	SVAR gSelection1 	= $(PlotFolderName+":gSelection1")
		
	CalculateTraceStatistics(PlotName,gSelection1)
End

Function CalculateTraceStatistics(PlotName,Selection1)
	String PlotName,Selection1

	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	
	SVAR gYStatisticChoice	= $("root:SPECTRA:Plotting:"+PlotName+":gYStatisticChoice")
	NVAR gYStatistic			= $("root:SPECTRA:Plotting:"+PlotName+":gYStatistic")
	
	SVAR gXStatisticChoice	= $("root:SPECTRA:Plotting:"+PlotName+":gXStatisticChoice")
	NVAR gXStatistic			= $("root:SPECTRA:Plotting:"+PlotName+":gXStatistic")
	
	SVAR gDistStatsChoice	= $("root:SPECTRA:Plotting:"+PlotName+":gDistStatsChoice")
	NVAR gDistStatistic		= $("root:SPECTRA:Plotting:"+PlotName+":gDistStatsVariable")
	
	if ((cmpstr("_all_",Selection1) == 0) || (cmpstr("_none_",Selection1) == 0))
		return 0
	endif

	Variable CsrA	= min(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
	Variable CsrB	= max(GetCursorPositionOrValue(PlotName,"A",1),GetCursorPositionOrValue(PlotName,"B",1))
	
	WAVE Axis 	= XWaveRefFromTrace(PlotName,Selection1)
	WAVE Data		= TraceNameToWaveRef(PlotName,Selection1)
	
	Variable p1 = AxisValueToPoint(Axis, CsrA)
	Variable p2 = AxisValueToPoint(Axis, CsrB)
	
	Variable pMin = min(p1,p2)
	Variable pMax = max(p1,p2)
	
	// Backwards compatibility
	if (!SVAR_Exists(gDistStatsChoice))
		return 0
	endif
	
	// Calculate the Y statistic
	gYStatistic 	= CalculateStatistic(Axis,Data,pMin,pMax,gYStatisticChoice)
	
	// Calculate the X statistic
	gXStatistic 	= CalculateStatistic(Data,Axis,pMin,pMax,gXStatisticChoice)
	
	// Calculate the DISTRIBUTION statistic
	gDistStatistic 	= DistributionStatistics(Data,Axis,pMin,pMax,gDistStatsChoice)

End

Function DistributionStatistics(Data,Axis,pMin,pMax,Choice)
	Wave Axis,Data
	Variable pMin,pMax
	String Choice
	
	Variable Integral=NaN, PDArea, PDMean
	
	// Calculate the area of the probability distribution for normalization
	PDArea = IntegrateWave(Axis,Data,pMin,pMax)
	
	// Calculate the mean
	Duplicate /FREE/D Data, Integrand, AxisMean
	Integrand	= Axis * Data
	PDMean 	= IntegrateWave(Axis,Integrand,pMin,pMax)/PDArea
	AxisMean 	= Axis - PDMean
	
	if (cmpstr("mean",Choice) == 0)
		return PDMean
		
	else
		Integrand[]	= AxisMean[p]^2 * Data[p]
		Integral 		= IntegrateWave(AxisMean,Integrand,pMin,pMax)
		
		if (cmpstr("variance",Choice) == 0)
			return Integral/PDArea
		else
			return sqrt(Integral/PDArea)
		endif
	endif
	
End


Function CalculateStatistic(Axis,Data,pMin,pMax,Choice)
	Wave Axis,Data
	Variable pMin,pMax
	String Choice
	
	Variable value, factor, base, background
	
	if (cmpstr("sum of values",Choice) == 0)
		value 	= SumWaveWithNANs(Data,pMin,pMax)
		
		return value
		
	elseif (cmpstr("mean of values",Choice) == 0)
		value 	= SumWaveWithNANs(Data,pMin,pMax)
		factor 	= (pMax - pMin)
		
		return value/factor
		
	elseif (cmpstr("standard deviation",Choice) == 0)
		WaveStats /Q/R=[pMin,pMax] Data
		return V_sdev
		
	elseif (cmpstr("average",Choice) == 0)
		return faverageXY(Axis,Data,Axis[pMin],Axis[pMax])
		
	elseif (cmpstr("integral",Choice) == 0)
		return areaXY(Axis,Data,Axis[pMin],Axis[pMax])

	elseif (cmpstr("integral - bg",Choice) == 0)
		value 	= areaXY(Axis,Data,Axis[pMin],Axis[pMax])
		base 	= Axis[pMax] - Axis[pMin]
		background = 0.5 * base * (Data[pMin] + Data[pMax])
		
		return value - background
	endif
End

// *************************************************************
// ****		TAB 5:	X-ray Diffraction Operations
// *************************************************************
Function AppendPlotXRayControls(WindowName)
	String WindowName
	
	String PlotFolderName 	= "root:SPECTRA:Plotting:"+WindowName
	String CalFolderName 	= "root:SPECTRA:Plotting:Calibration"
	
	MakeVariableIfNeeded(PlotFolderName+":gAxisValue1",0)
	MakeVariableIfNeeded(PlotFolderName+":gAxisTrueValue1",0)
	MakeVariableIfNeeded(PlotFolderName+":gAxisValue2",0)
	MakeVariableIfNeeded(PlotFolderName+":gAxisTrueValue2",0)
	MakeVariableIfNeeded(PlotFolderName+":gAxisManualValue",0)
	MakeVariableIfNeeded(PlotFolderName+":gAxisTrueValue",0)
	
	NVAR gAxisValue1				= $(PlotFolderName+":gAxisValue1")
	NVAR gAxisTrueValue1		= $(PlotFolderName+":gAxisTrueValue1")
	NVAR gAxisValue2				= $(PlotFolderName+":gAxisValue2")
	NVAR gAxisTrueValue2		= $(PlotFolderName+":gAxisTrueValue2")
	NVAR gAxisManualValue		= $(PlotFolderName+":gAxisManualValue")
	NVAR gAxisTrueValue			= $(PlotFolderName+":gAxisTrueValue")
	
	SetVariable XRAY_SetAxis1 ,pos={90,84},size={150,15},title="Measured value",fSize=10,limits={0,Inf,0.1},value= gAxisValue1, disable=1
	SetVariable XRAY_SetAxis1True ,pos={113,101},size={127,15},title="True value",fSize=10,limits={0,Inf,0.1},value= gAxisTrueValue1, disable=1
	SetVariable XRAY_SetAxis2 ,pos={90,118},size={150,15},title="Measured value",fSize=10,limits={0,Inf,0.1},value= gAxisValue2, disable=1
	SetVariable XRAY_SetAxis2True ,pos={112,135},size={127,15},title="True value",fSize=10,limits={0,Inf,0.1},value= gAxisTrueValue2, disable=1
	
	SetVariable XRAY_SetManualAxis ,pos={91,154},size={72,15},title=" ",fSize=10,limits={0,Inf,0.1},value= gAxisManualValue, proc=ShowTrueAxis,disable=1
	ValDisplay XRAY_ShowTrueAxis, pos={175,154},size={74,17},title="= ",fSize=10, disable=1
	SetControlGlobalVariable(WindowName,"XRAY_ShowTrueAxis","root:SPECTRA:Plotting","gAxisTrueValue",2)
	
	Button XRAY_SaveAxisCal,pos={96,62},size={140,18},proc=XRayControlButtons,title="Save axis calibration", disable=1
End

Function ShowTrueAxis(ctrlName,varNum,varStr,varName) : SetVariableControl 
	String ctrlName 
	Variable varNum
	String varStr
	String varName
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	NVAR gAxisValue1				= $(PlotFolderName+":gAxisValue1")
	NVAR gAxisTrueValue1		= $(PlotFolderName+":gAxisTrueValue1")
	NVAR gAxisValue2				= $(PlotFolderName+":gAxisValue2")
	NVAR gAxisTrueValue2		= $(PlotFolderName+":gAxisTrueValue2")
	NVAR gAxisManualValue		= $(PlotFolderName+":gAxisManualValue")
	NVAR gAxisTrueValue			= $(PlotFolderName+":gAxisTrueValue")
	
	Variable m = (gAxisTrueValue1-gAxisTrueValue2)/(gAxisValue1-gAxisValue2)
	
	Variable c = gAxisTrueValue1 - m*gAxisValue1
	
	gAxisTrueValue = m*gAxisManualValue + c
End

Function ScaleAndShift(Axis,PromptFlag)
	Wave Axis
	Variable PromptFlag
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	NVAR gAxisValue1				= $(PlotFolderName+":gAxisValue1")
	NVAR gAxisTrueValue1		= $(PlotFolderName+":gAxisTrueValue1")
	NVAR gAxisValue2				= $(PlotFolderName+":gAxisValue2")
	NVAR gAxisTrueValue2		= $(PlotFolderName+":gAxisTrueValue2")
	
	if (PromptFlag == 1)
		Variable Axis1 = gAxisValue1
		Prompt Axis1, "First measured axis value"
		Variable Axis1True = gAxisTrueValue1
		Prompt Axis1True, "Desired axis value"
		Variable Axis2 = gAxisValue2
		Prompt Axis2, "Second measured axis value"
		Variable Axis2True = gAxisTrueValue2
		Prompt Axis2True, "Desired axis value"
		DoPrompt "Scale and shift axis", Axis1, Axis1True, Axis2, Axis2True
		if (V_flag)
			return 0
		endif
		
		gAxisValue1 		= Axis1
		gAxisTrueValue1 	= Axis1True
		gAxisValue2		= Axis2
		gAxisTrueValue2	= Axis2True
	endif
		
	Variable m	= (gAxisTrueValue1 - gAxisTrueValue2)/(gAxisValue1 - gAxisValue2)
	
	Variable c	= gAxisTrueValue1 - m*gAxisValue1
	
	Axis[]	 = m*Axis[p] + c
End

Function XRayControlButtons(ctrlname):ButtonControl
	String ctrlname
	
	String PlotName = WinName(0,65)
	String PrefsFolderName 	= "root:SPECTRA:Plotting:Preferences"
	String PlotFolderName 	= "root:SPECTRA:Plotting:" + PlotName
	String CopyFolderName 	= "root:SPECTRA:Plotting:" + PlotName+":DataDuplicates"
	SVAR gSelection1 	= $(PlotFolderName+":gSelection1")
		
	String ExportStyle
	Variable NumTraces, TraceIndex,  i
	
	if (cmpstr(ctrlname,"XRAY_SaveAxisCal") == 0)
		TransferStrsAndVars(PlotFolderName,PrefsFolderName,"Axis")
	endif
	
//	if (StrSearch(ctrlname,"SaveXRD",0) > -1)
//		if  (cmpstr(ctrlname,"XRD_SaveGSASButton")==0)
//			ExportStyle = "GSAS"
//		elseif (cmpstr(ctrlname,"XRD_SavePanButton")==0)
//			ExportStyle = "PANA"
//		endif
//			
//		if (cmpstr("_all_",gSelection1) == 0)
//			NumTraces = ItemsInList(TraceNameList(PlotName,";",1))
//			for (i=0;i<NumTraces;i+=1)
//				WAVE Axis 	= WaveRefIndexed(PlotName, i, 2)
//				WAVE Data 	= WaveRefIndexed(PlotName, i, 1)
//				SaveGSASFXY(Axis,Data)
//				//
//				CopyIndexTraceToDataFolder(PlotName,CopyFolderName,i, 1)
//			endfor		
//		else
//			SaveGSASFXY(XWaveRefFromTrace(PlotName,gSelection1),TraceNameToWaveRef(PlotName,gSelection1))
//			//
//			TraceIndex = WhichListItem(gSelection1,TraceNameList(PlotName, ";", 1))
//			CopyIndexTraceToDataFolder(PlotName,CopyFolderName,TraceIndex, 1)
//		endif
//	endif
End

// *************************************************************
// ****		TAB 6:	Titration Analysis
// *************************************************************
Function AppendPlotT70Controls(WindowName)
	String WindowName
	
	String PlotFolderName 	= "root:SPECTRA:Plotting:"+WindowName
	
	MakeVariableIfNeeded(PlotFolderName+":gT70SampleMass",1)
	MakeVariableIfNeeded(PlotFolderName+":gT70SurfaceArea",100)
	MakeVariableIfNeeded(PlotFolderName+":gT70AcidVinit",0)
	MakeVariableIfNeeded(PlotFolderName+":gT70AcidCinit",0)
	MakeVariableIfNeeded(PlotFolderName+":gT70BaseVinit",0)
	MakeVariableIfNeeded(PlotFolderName+":gT70BaseCinit",0)
	MakeVariableIfNeeded(PlotFolderName+":gT70TitrantConc",0.1)
	MakeVariableIfNeeded(PlotFolderName+":gT70TitrantFactor",-1)
	
	NVAR gT70SampleMass			= $(PlotFolderName+":gT70SampleMass")
	NVAR gT70SurfaceArea			= $(PlotFolderName+":gT70SurfaceArea")
	NVAR gT70AcidVinit				= $(PlotFolderName+":gT70AcidVinit")
	NVAR gT70AcidCinit				= $(PlotFolderName+":gT70AcidCinit")
	NVAR gT70BaseVinit				= $(PlotFolderName+":gT70BaseVinit")
	NVAR gT70BaseCinit				= $(PlotFolderName+":gT70BaseCinit")
	NVAR gT70TitrantConc			= $(PlotFolderName+":gT70TitrantConc")
	NVAR gT70TitrantFactor			= $(PlotFolderName+":gT70TitrantFactor")
		
	// Volume of initial ACID addition
	SetVariable SetT70AcidVinitVar,pos={94,73},size={89,15},title="Volume",fSize=10
	SetVariable SetT70AcidVinitVar,limits={0,100,0.1},value= gT70AcidVinit, disable=1
	
	// Concentration of initial ACID addition
	SetVariable SetT70AcidCinitVar,pos={105,96},size={79,15},title="Conc.",fSize=10
	SetVariable SetT70AcidCinitVar,limits={0,14,0.1},value= gT70AcidCinit, disable=1
	
	// Volume of initial BASE addition
	SetVariable SetT70BaseVinitVar,pos={218,73},size={89,15},title="Volume",fSize=10
	SetVariable SetT70BaseVinitVar,limits={0,100,0.1},value= gT70BaseVinit, disable=1
	
	// Concentration of initial BASE addition
	SetVariable SetT70BaseCinitVar,pos={229,96},size={79,15},title="Conc.",fSize=10
	SetVariable SetT70BaseCinitVar,limits={0,14,0.1},value= gT70BaseCinit, disable=1
	
	//Sample Mass
	SetVariable SetT70MassVar,pos={380,72},size={104,18},title="Mass",fSize=12
	SetVariable SetT70MassVar,limits={0,1000,0.1},value= gT70SampleMass, disable=1
	
	// Surface Area
	SetVariable SetT70SAVar,pos={346,92},size={138,18},title="Surf. Area",fSize=12
	SetVariable SetT70SAVar,limits={0,1000,0.1},value= gT70SurfaceArea, disable=1
	
	// Titrant Concentration
	SetVariable SetT70ConcVar,pos={533,72},size={139,18},title="Concentration",fSize=12
	SetVariable SetT70ConcVar,limits={0,10,0.1},value= gT70TitrantConc, disable=1
	
	// Titrant Acid or Base?
	CheckBox SetT70AcidRadio,pos={555,96},size={61,15},proc=TitrationCheckProcs,title="Acid or",fSize=12
	CheckBox SetT70AcidRadio,mode=1, value=0, disable=1
	CheckBox SetT70BaseRadio,pos={624,96},size={47,15},proc=TitrationCheckProcs,title="Base",fSize=12
	CheckBox SetT70BaseRadio,mode=1, value=1, disable=1
	
	// Parameter Units
	TitleBox T70MassUnits title="g",pos={490,73},frame=0,fSize=12, disable=1
	TitleBox T70SAUnits title="m\\S2\\M/g",pos={490,90},frame=0,fSize=12, disable=1
	TitleBox T70ConcUnits title="M",pos={680,73},frame=0,fSize=12, disable=1
	TitleBox T70AcidConcUnits,pos={187,97},size={9,12},title="M",fSize=10,frame=0, disable=1
	TitleBox T70BaseConcUnits,pos={311,97},size={9,12},title="M",fSize=10,frame=0, disable=1
	TitleBox T70AcidVolUnits,pos={187,74},size={15,12},title="mL",fSize=10,frame=0, disable=1
	TitleBox T70BaseVolUnits,pos={311,74},size={15,12},title="mL",fSize=10,frame=0, disable=1
	
	// Control Boxes
	GroupBox T70AcidInitGroup,pos={90,55},size={120,61},title="Initial Acid Addition", fColor=(52428,1,1),fSize=10, disable=1
	GroupBox T70BaseInitGroup,pos={214,55},size={120,61},title="Initial Base Addition", fColor=(52428,1,1),fSize=10, disable=1
	GroupBox T70SampleGroup,pos={338,55},size={187,62},title="Sample parameters", fColor=(52428,1,1),fSize=10, disable=1
	GroupBox T70TitrantGroup,pos={530,55},size={167,62},title="Titrant Parameters",fColor=(52428,1,1),fSize=10, disable=1
	
	TitleBox T70Instruction title="Plot pH vs 'Volume added'\rPlace round cursor at start of sample titration data",pos={103,121},frame=0,fSize=13,fColor=(1,4,52428), disable=1
	
	Button T70CalculateQButton,pos={462,120},size={100,18},proc=CalculateQButtonProc,title="Calculate Q", disable=1
	Button T70KeepQButton,pos={570,120},size={60,18},proc=CalculateQButtonProc,title="Keep", disable=1
End

Function CalculateQButtonProc(ctrlname):ButtonControl
	String ctrlname
	
	if (cmpstr(ctrlname,"T70KeepQButton") == 0)
		KeepTitrationCurve("la")
		return 0
	endif
	
	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	SVAR gSelection1 		= $(PlotFolderName+":gSelection1")
	SVAR gSelection2 		= $(PlotFolderName+":gSelection2")
	
	NVAR gAcidVinit		= $(PlotFolderName+":gT70AcidVinit")
	NVAR gAcidCinit		= $(PlotFolderName+":gT70AcidCinit")
	NVAR gBaseVinit		= $(PlotFolderName+":gT70BaseVinit")
	NVAR gBaseCinit		= $(PlotFolderName+":gT70BaseCinit")
	NVAR gSampleMass	= $(PlotFolderName+":gT70SampleMass")
	NVAR gSurfaceArea	= $(PlotFolderName+":gT70SurfaceArea")
	NVAR gTitrantConc	= $(PlotFolderName+":gT70TitrantConc")
	NVAR gTitrantFactor	= $(PlotFolderName+":gT70TitrantFactor")
	
	Variable pHstart, pHstartIdx, pHend, pHendIdx, SampleVoffset, BlankVindex, BlankVoffset
	Variable TotalArea	= gSampleMass * gSurfaceArea
	
	if (cmpstr(CsrWave(A, PlotName),gSelection1) != 0)
		DoAlert 0, "Please place the round cursor at the start of the selected sample data"
		return 0
	elseif (cmpstr(gSelection2,gSelection1) == 0)
		DoAlert 0, "Please use trace selection menus to choose separate curves for sample and blank data"
		return 0
	endif
	
	
	// ------------------------	Here, we assume we are plotting pH vs volume GRAM plot
	
	// The sample titration data
	WAVE SamplepHData 		= TraceNameToWaveRef(PlotName,gSelection1)
	WAVE SampleVolAxis		= XWaveRefFromTrace(PlotName,gSelection1)
	// The blank titration data
	WAVE BlankpHData		= TraceNameToWaveRef(PlotName,gSelection2)
	WAVE BlankVolAxis		= XWaveRefFromTrace(PlotName,gSelection2)
	Variable NPts 				= DimSize(SamplepHData,0)
	
	// Look up the titration starting pH
	pHstart 			= SamplepHData[GetCursorPositionOrValue(PlotName,"A",1)]
	pHstartIdx 		= GetCursorPositionOrValue(PlotName,"A",0)
	SampleVoffset 		= SampleVolAxis[GetCursorPositionOrValue(PlotName,"A",0)]
	
	BlankVindex 		= BinarySearchInterp(BlankpHData,pHstart)
	if (numtype(BlankVoffset) != 0)
		DoAlert 0, "The blank data does not extend to this sample pH value!"
		return 0
	endif
	BlankVoffset 		= BlankVolAxis[BlankVindex]
	
	// Make a FREE wave to shift the y-axis volume data
	Duplicate /D/O SampleVolAxis, SampleVol_offset
	Duplicate /D/O BlankVolAxis, BlankVol_offset
	
	SampleVol_offset -= SampleVoffset
	BlankVol_offset -= BlankVoffset
	
	// A new common Volume curve, incorporating the offset. 
	Variable NTitPts 	= NPts - pHstartIdx
	Make /O/D/N=(NTitPts) $(CheckFolderColon(PlotFolderName) + "TitrantVolume") /WAVE=TitrantVolume
	TitrantVolume[] 	= SampleVolAxis[p+pHstartIdx] - SampleVoffset
	
	// A new Sample titration curve. 
	Make /O/D/N=(NTitPts) $(CheckFolderColon(PlotFolderName) + "SamplepH") /WAVE=SamplepH
	Make /O/D/N=(NTitPts) $(CheckFolderColon(PlotFolderName) + "BlankpH") /WAVE=BlankpH
	
	// Interpolate the shifted Sample and Blank curves onto the common titration axis
	Interpolate2 /T=1/I=3/X=TitrantVolume/Y=SamplepH SampleVol_offset, SamplepHData
	Interpolate2 /T=1/I=3/X=TitrantVolume/Y=BlankpH BlankVol_offset, BlankpHData
	
	// A new Gram curve
	Make /O/D/N=(NTitPts) $(CheckFolderColon(PlotFolderName) + "SampleHConc") /WAVE=SampleHConc
	Make /O/D/N=(NTitPts) $(CheckFolderColon(PlotFolderName) + "BlankHConc") /WAVE=BlankHConc
	
	
	
	// ------------------------	
	
	
	
	
	
	
	
	
	
	
	
	

	String msg = " 	*** Calculating surface density of protons adsorbed to "+num2str(TotalArea)+"m2 of sample  --"+gSelection1+"-- "
	if (gTitrantFactor)
		msg+= " during acid titration"
	else
		msg+= " during base titration"
	endif
	msg+= ", subtracting the background from solution --"+gSelection2+"-- "
	print msg
	
	// Adding ACID is a positive contribution; BASE is negative
	
//	// Moles of protons added to or removed from the solution due to sample pre-titration
//	SampledQ[pHstartIdx,]	= gAcidVinit * gAcidCinit - gBaseVinit * gBaseCinit
//	
//	// Moles of protons added to or removed from the solution due to titrant addition to sample
//	SampledQ[pHstartIdx,]	+= gTitrantFactor * gTitrantConc * Sample[p]
//	
//	// Moles of protons added to or removed from the solution due to titrant addition to blank
//	SampledQ[pHstartIdx,]	-= gTitrantFactor * (interp(SamplepH[p],BlankpH,Blank) + BlankVoff)
//	
	//
	
	
	// Moles of protons added to or removed from the solution due to titrant addition to blank
//	SampleQ[pHstartIdx,]	= Sample[p] - (interp(SamplepH[p],BlankpH,Blank) - BlankVoff)
//	SampleQ[pHstartIdx,]	*= (gTitrantFactor * gTitrantConc)
//	
//	// Moles of protons added to or removed from the solution due to sample pre-titration
//	SampleQ[pHstartIdx,]	+= gAcidVinit * gAcidCinit - gBaseVinit * gBaseCinit
//	
//	
//	SampleQ *= cFaraday/TotalArea
//	SampleQ *= 1e9			// Convert to Coulombs per nm2
//	
//	CheckDisplayed /W=$PlotName SampleQ
//	if (V_flag == 0)
//		AppendToGraph /R/W=$PlotName SampleQ vs SamplepH
//	endif
End

Function KeepTitrationCurve(ctrlname):ButtonControl
	String ctrlname

	String PlotName = WinName(0,65)
	String PlotFolderName = "root:SPECTRA:Plotting:" + PlotName
	
	SVAR gSelection1 			= $(PlotFolderName+":gSelection1")
	String SampleQName		= ReplaceString("_data",gSelection1,"") + "_dQ"
	String SamplepHName	= ReplaceString("_data",gSelection1,"") + "_axis"
	
	String AxisFolderName = GetWavesDataFolder(XWaveRefFromTrace(PlotName,gSelection1),1)
	
	AdoptAxisAndDataFromMemory(SamplepHName,"null",AxisFolderName,SampleQName,"null",PlotFolderName,SampleQName,"",1,0,1)
End

Function TitrationCheckProcs(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	String PlotName = WinName(0,65)
	NVAR gTitrantFactor		= $("root:SPECTRA:Plotting:"+PlotName+":gT70TitrantFactor")
	
	if (StrSearch(ctrlName,"Acid",0) > -1)
		gTitrantFactor = 1
		CheckBox SetT70AcidRadio,value=1
		CheckBox SetT70BaseRadio,value=0
	else
		gTitrantFactor = -1
		CheckBox SetT70AcidRadio,value=0
		CheckBox SetT70BaseRadio,value=1
	endif
End

Function Langmuir() : FitFunc

End