﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.


// 
Function RemoveSeasonalTrends()

	SetDataFolder root:
	
	String CTrendType = StrVAROrDefault("root:Geochem:gCTrendType","all")
	Prompt CTrendType, "Choose concentration data", popup, "all;single;"
	String ConcFolder 	= StrVAROrDefault("root:Geochem:gConcFolder","PH_Isco")
	String CTrendFolder 	= StrVAROrDefault("root:Geochem:gCTrendFolder","Fluxes")
	String FolderList1 	= ListOfObjectsInFolder(4,"root:")
	String FolderList2 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
	String NewFolderName = ""
	
	Prompt ConcFolder, "Data Source", popup, FolderList1
	Prompt CTrendFolder, "Destination", popup, FolderList2
	Prompt NewFolderName, "Destination"
	DoPrompt "Calculate single or all Concentration trends", CTrendType, ConcFolder, CTrendFolder, NewFolderName
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gCTrendType = CTrendType
	String /G root:Geochem:gConcFolder = ConcFolder
	
	if (cmpstr(CTrendFolder,"new - enter below")==0)
		NewDataFolder/O $("root:"+NewFolderName)
		CTrendFolder 	= NewFolderName
	endif
	String /G root:Geochem:gCTrendFolder = CTrendFolder
	
	// List all the data in the Source and Discharge directories
	String ConcList = ListOfObjectsInFolder(1,"root:"+ConcFolder)
	
	ConcList 	= ExclusiveWaveList(ConcList,"_d",";")
	Variable n, NSpecies = ItemsInList(ConcList)
	
	// Select the Concentration data
//	String CCData 	= StringFromList(0,ConcList)
	String CCData = StrVAROrDefault("root:Geochem:gCCData",StringFromList(0,ConcList))
	Prompt CCData, "Concentration data", popup, ConcList
	Variable SSpline = NUMVarOrDefault("root:Geochem:gSSpline",0.4)
	Prompt SSpline, "Smoothing spline parameter"
	Variable TPlot = NUMVarOrDefault("root:Geochem:gTPlot",1)
	Prompt TPlot, "Plot trends", popup, "no;yes;"
	if (cmpstr(CTrendType,"all") == 0)
		DoPrompt "Trend removel", SSpline
	else
		DoPrompt "Choose concentration data and trend removal", CCData, SSpline, TPlot
	endif
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gCCData = CCData
	Variable /G root:Geochem:gSSpline = SSpline
	Variable /G root:Geochem:gTPlot = TPlot
	
	
	SetDataFolder $CTrendFolder
	
		if (cmpstr(CTrendType,"all") == 0)
			KillWaves /A/Z
			print "Removing seasonal trends from the data in",ConcFolder,"and saving to",CTrendFolder,"using smoothing spline factor",SSpline
			
			for (n=0;n<NSpecies;n+=1)
				String CCName 		= StringFromList(n,ConcList)
				String CCUnit 		= GeochemUnitFromName(CCName)
				String CCParam 	= GeochemParameterFromName(CCName)
				String CCdName 	= CCParam + "_d"
				
				WAVE CC 	= $("root:"+ConcFolder+":"+CCName)
				WAVE CCd 	= $("root:"+ConcFolder+":"+CCdName)
				if (!WaveExists(CCd))
					WAVE CCd 	= $("root:"+ConcFolder+":"+CCName+"_d")
				endif
				
				CTrendCalculation(SSpline,1,CCParam,CC,CCd)
			endfor
		else
				CCName 		= CCData
				CCUnit 		= GeochemUnitFromName(CCName)
				CCParam 	= GeochemParameterFromName(CCName)
				CCdName 	= CCParam + "_d"
				
				WAVE CC 	= $("root:"+ConcFolder+":"+CCName)
				WAVE CCd 	= $("root:"+ConcFolder+":"+CCdName)
				if (!WaveExists(CCd))
					WAVE CCd 	= $("root:"+ConcFolder+":"+CCName+"_d")
				endif
				
				print "Calculating Concentration-Discharge relationships for the",CCName,"in",ConcFolder,"and saving to",CTrendFolder,"using smoothing spline factor",SSpline
				CTrendCalculation(SSpline,TPlot,CCParam,CC,CCd)
				
		endif
		
	SetDataFolder root:
End

Function PlotTrends(Ele,cc,cdate,SSp,Res)
	String Ele
	Wave cc,cdate,SSp,Res
	
	Display /K=1/W=(200,262,1572,913) cc vs cdate as "Trend for "+Ele
	AppendToGraph SSp,Res
	
	String ResName = NameOfWave(Res), SSpName = NameOfWave(SSp)
	ModifyGraph mode($ResName)=4, marker($ResName)=8, msize($ResName)=2, opaque($ResName)=1
	ModifyGraph rgb($SSpName)=(1,16019,65535)
	
	ModifyGraph mirror=2
	ModifyGraph dateInfo(bottom)={0,0,0}
	Label left " Concentration"
	Label bottom " Date"
End

Function MakeSplineDestinations(Param,SSuffix,cdate)
	String Param, SSuffix
	Wave cdate
	
	Variable Ncc=DimSize(cdate,0)
	Variable nDays = (cdate[Ncc-1] - cdate[0])/(60*60*24)
	
	String sdate = Param+"s_d"
	
	// Make a date wave 
	Make /O/D/N=(nDays) $(Param+SSuffix+"_d") /WAVE=SpD
	SpD[] = cdate[0] + p*86400
	
	// Make an interpolated parameter wave with daily scale 
	Make /O/D/N=(nDays) $(Param+SSuffix) /WAVE=Sp
	SetScale /P x, cdate[0], 86400, Sp, SpD
	
End

Function CTrendCalculation(SSpF,TPlot,Ele,cc,cdate)
	Variable SSpF, TPlot
	String Ele
	Wave cc,cdate
	
	// Smoothing spline
	MakeSplineDestinations(Ele,"SSp",cdate)
	WAVE SSp = $(Ele+"SSp")
	Interpolate2/T=3/F=(SSpF)/Y=SSp/I=3 cdate,cc
	
	// Linear spline
	MakeSplineDestinations(Ele,"LSp",cdate)
	WAVE LSp = $(Ele+"LSp")
	Interpolate2/T=1/Y=LSp/I=3 cdate,cc
	
	WAVE LSp_d = $(Ele+"LSp_d")
	Duplicate /O LSp_d, $(Ele+"Res_d")
	
	Duplicate /O SSp, $(Ele+"Res")
	WAVE Res = $(Ele+"Res")
	Note Res, "Smoothing factor = "+num2str(SSpF)
	
	Res = LSp - SSp
	
	if (TPlot == 2)
		PlotTrends(Ele,cc,cdate,SSp,Res)
	endif
End





// 
Function ChooseDataColorScheme()

	SetDataFolder root:
	
	String ConcFolder 	= StrVAROrDefault("root:Geochem:gConcFolder","PH_Isco")
	Prompt ConcFolder, "Data Source", popup, ListOfObjectsInFolder(4,"root:")
	String ColorScheme 	= StrVAROrDefault("root:Geochem:gColorScheme","Water Year")
	Prompt ColorScheme, "Color scheme", popup, "Water Year;Limb;Month;"
	DoPrompt "Choose folder and color scheme", ConcFolder, ColorScheme
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gConcFolder = ConcFolder
	String /G root:Geochem:gColorScheme = ColorScheme
	
	// List all the RGB waves in the Source directory
	String RGBList = ListOfObjectsInFolder(1,"root:"+ConcFolder)
	RGBList 	= InclusiveWaveList(RGBList,"_RGB",";")
	Variable n, NSpecies = ItemsInList(RGBList)
		
		for (n=0;n<NSpecies;n+=1)
			String RGBName 		= StringFromList(n,RGBList)
			WAVE RGB 	= $("root:"+ConcFolder+":"+RGBName)
			
			String DateName = ReplaceString("_RGB",RGBName,"_d")
			WAVE CCd 	= $("root:"+ConcFolder+":"+DateName)
			
			if (cmpstr(ColorScheme,"Water Year")==0)
				RGBWaterYear(RGB,CCd)
			elseif (cmpstr(ColorScheme,"Limb")==0)
			elseif (cmpstr(ColorScheme,"Month")==0)
		endif
		endfor
		
	SetDataFolder root:
End

Function RGBWaterYear(RGB,CCd)
	Wave RGB,CCd
	
	Variable i, NDays=DimSize(CCd,0)
	
	Variable Year1,Year2, NYears, RGBScale, Month,Day
	ReturnYearMonthDay(CCd[0],Year1,Month,Day)
	ReturnYearMonthDay(CCd[NDays-1],Year2,Month,Day)
	NYears 		= Year2-Year1
	RGBScale 	= 256/NYears
	
	for (i=0;i<NDays;i+=1)
		Year2 = str2num(StringFromList(0,Secs2Date(CCd[i],-2,";")))
		RGB[i] = RGBScale * ((Year2 - Year1)/NYears)
	endfor
End





Function MeanFlux()

	WAVE AWave = CsrWaveRef(A)
	WAVE AXWave = CsrXWaveRef(A)
	WAVE BWave = CsrWaveRef(B)
	WAVE BXWave = CsrXWaveRef(B)
	
	if (WaveExists(AWave) && WaveExists(BWave))
		print "Averaging",NameOfWave(AWave),"between", secs2Date(AXWave[pcsr(A)],0), "and",secs2Date(BXWave[pcsr(B)],0)
	else 
		return 0
	endif
	Variable APoint = min(pcsr(A),pcsr(B))
	Variable BPoint 	= max(pcsr(A),pcsr(B))
	Variable AXVal = min(AXWave[pcsr(A)],BXWave[pcsr(B)])
	Variable BXVal = max(AXWave[pcsr(A)],BXWave[pcsr(B)])
	
	WaveStats /R=[APoint,BPoint ]/Q/M=1 AWave
	
	print V_avg
End

Function ConvertSingleParameterUnits()

	SetDataFolder root:
	
	String UnitList 	= "ppb;ppm;uM;mM;ugpL;mgpL;Pc;"
	String ConcFolder 	= StrVAROrDefault("root:Geochem:gConcFolder","PH_Isco")
	ConcFolder 	= ChooseAFolder("",ConcFolder,"Data Source","Choose data folder for conversion",0)
	if (strlen(ConcFolder) == 0)
		return 0
	endif
	String /G $("root:Geochem:gConcFolder") = ConcFolder
	String SpeciesList 	= ExclusiveWaveList(ListOfObjectsInFolder(1,ConcFolder),"_d",";")
	
	String Species, InputUnit, OutputUnit
	Prompt Species, "Parameter", popup, SpeciesList
	Prompt InputUnit, "Input unit", popup, UnitList
	Prompt OutputUnit "Output unit", popup, UnitList
	DoPrompt "Choose parameter and units", Species, OutputUnit
	if (V_flag)
		return 0
	endif
	
	String Unit = GeochemUnitFromName(Species)
	String Param = GeochemParameterFromName(Species)
	String CCConvert 	= Param + "_" + OutputUnit
	if (strsearch(UnitList,Unit,0) == -1)
		return 0
	endif
	
	WAVE CC 	= $("root:"+ConcFolder+":"+Species)
	Variable NDays = DimSize(CC,0)
		
	Make /FREE/D/N=(NDays) CCppb
	Make /O/D/N=(NDays) $("root:"+ConcFolder+":"+CCConvert) /WAVE=Convert
	
	ConvertDataBetweenUnitAndPPB(1,Param,Unit,CC,CCppb)
	
	ConvertDataBetweenUnitAndPPB(0,Param,OutputUnit,Convert,CCppb)

End

Function ConvertDataBetweenUnitAndPPB(ScaleFlag,Param,Unit,CC,CCppb)
	Variable ScaleFlag
	String Param,Unit
	Wave CC,CCppb
	
	Variable Z, M, scale=1
	
	strswitch(Unit)	
		case "ppb":
			break	
		case "ppm":
			scale=1000
			break
		case "ugpL":
			break	
		case "mgpL":
			scale=1000
			break
		case "uM":
			M = SoluteFormulaMass(Param)
			scale=M / 1000
		case "mM":
			scale = M
			break
		default:
			scale=1
	endswitch
	
	if (ScaleFlag==1) 	// Convert TO ppb
		CCppb = scale * CC
	else 					// Convert FROM ppb
		CC = CCppb / scale		
	endif
End

Function ConvertFromPPBToUnit(Param,Unit,CCppb,CC)
	String Param,Unit
	Wave CCppb, CC

	Variable Z = AtomToZNumber(ReturnTextBeforeNumber(Param),1)
	
	Variable M = ZNumberToMass(Z)
End



// 
Function CalculateCQ()

	SetDataFolder root:
	
	String CQType = StrVAROrDefault("root:Geochem:gCQType","all")
	Prompt CQType, "Choose CQ data", popup, "all;single;"
	String ConcFolder 	= StrVAROrDefault("root:Geochem:gConcFolder","PH_Isco")
	String DischargeFolder 	= StrVAROrDefault("root:Geochem:gDischargeFolder","PH_Isco")
	String CQFolder 	= StrVAROrDefault("root:Geochem:gCQFolder","Fluxes")
	String FolderList1 	= ListOfObjectsInFolder(4,"root:")
	String FolderList2 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
	String NewFolderName = ""
	Prompt ConcFolder, "Data Source", popup, FolderList1
	Prompt DischargeFolder, "Discharge Data Source", popup, FolderList1
	Prompt CQFolder, "Destination", popup, FolderList2
	Prompt NewFolderName, "Destination"
	DoPrompt "Calculate single or all CQ", CQType, ConcFolder, DischargeFolder, CQFolder, NewFolderName
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gCQType = CQType
	String /G root:Geochem:gConcFolder = ConcFolder
	String /G root:Geochem:gDischargeFolder = DischargeFolder
	
	if (cmpstr(CQFolder,"new - enter below")==0)
		NewDataFolder/O $("root:"+NewFolderName)
		CQFolder 	= NewFolderName
	endif
	String /G root:Geochem:gCQFolder = CQFolder
	
	// List all the data in the Source and Discharge directories
	String ConcList = ListOfObjectsInFolder(1,"root:"+ConcFolder)
	String DischList = ListOfObjectsInFolder(1,"root:"+DischargeFolder)
	
	// Select the Concentration and Discharge data
	String CCData 	= StringFromList(0,ConcList)
	Prompt CCData, "Concentration data", popup, ConcList
	String DischName 	= StringFromList(0,DischList)
	Prompt DischName, "Discharge Data", popup, DischList
	if (cmpstr(CQType,"all") == 0)
		DoPrompt "Choose discharge data", DischName
	else
		DoPrompt "Choose discharge and concentration data", DischName, CCData
	endif
	if (V_flag)
		return 0
	endif
	String DischUnit = GeochemUnitFromName(DischName)
	
	WAVE QQ = $("root:"+DischargeFolder+":"+DischName)
	WAVE QQd 	= $("root:"+DischargeFolder+":"+GeochemDateFromName(DischName))
	
	// Remove discharge from the list of concentrations
	ConcList 	= ExclusiveWaveList(ExclusiveWaveList(ExclusiveWaveList(ConcList,"_d",";"),DischName,";"),"Species",";")
	Variable n, NSpecies = ItemsInList(ConcList)
	
	SetDataFolder $CQFolder
	
		if (cmpstr(CQType,"all") == 0)
			KillWaves /A/Z
			print "Calculating Concentration-Discharge relationships for the data in",ConcFolder,"and saving to",CQFolder
			
			for (n=0;n<NSpecies;n+=1)
				String CCName 		= StringFromList(n,ConcList)
				String CCUnit 		= GeochemUnitFromName(CCName)
				String CCParam 	= GeochemParameterFromName(CCName)
				String CCdName 	= CCParam + "_d"
				
				WAVE CC 	= $("root:"+ConcFolder+":"+CCName)
				WAVE CCd 	= $("root:"+ConcFolder+":"+CCdName)
				if (!WaveExists(CCd))
					WAVE CCd 	= $("root:"+ConcFolder+":"+CCName+"_d")
				endif
				
				CQCalculation(CCParam,QQ,QQd,CC,CCd)
			endfor
		else
				CCName 		= CCData
				CCUnit 		= GeochemUnitFromName(CCName)
				CCParam 	= GeochemParameterFromName(CCName)
				CCdName 	= CCParam + "_d"
				
				WAVE CC 	= $("root:"+ConcFolder+":"+CCName)
				WAVE CCd 	= $("root:"+ConcFolder+":"+CCdName)
				if (!WaveExists(CCd))
					WAVE CCd 	= $("root:"+ConcFolder+":"+CCName+"_d")
				endif
				
				print "Calculating Concentration-Discharge relationships for the",CCName,"in",ConcFolder,"and saving to",CQFolder
				CQCalculation(CCParam,QQ,QQd,CC,CCd)
		endif
		
	SetDataFolder root:
End

Function CQCalculation(Ele,qq,qdate,cc,cdate)
	String Ele
	Wave qq,qdate,cc,cdate
	
	Variable i, qDay, qVal, cDay, cVal, nDays=0
	Variable Nqq=DimSize(qdate,0), Ncc=DimSize(cdate,0)
	
	Make /O/N=0 $(Ele+"_C") /WAVE=CWave
	Make /O/N=0 $(Ele+"_Q") /WAVE=QWave
	Make /O/N=0 $(Ele+"_d") /WAVE=DWave
	Make /O/N=0 $(Ele+"_RGB") /WAVE=RGBWave
	
	// Loop through all valid Element dates
	for(i=0;i<Ncc;i+=1)
		cDay 	= cdate[i]
		if (numtype(cDay) == 0)
			cVal 	= cc[i]
			if (numtype(cVal) == 0)
				FindValue /V=(cDay) qdate
				if (V_value != -1)
					qVal 	= qq[V_value]
					if (numtype(qVal) == 0)
						nDays += 1
						Redimension /N=(nDays) CWave, QWave, DWave, RGBWave
						DWave[nDays-1] 	= cDay
						CWave[nDays-1] 	= cVal
						QWave[nDays-1] 	= qVal
					endif
				endif
			endif
		endif
	endfor
End
	

// Convert Concentration-Discharge values to Fluxes. Also calculate Exports for n Water Years
Function AllSpeciesFluxes()

	SetDataFolder root:
	
	String ConcFolder 	= StrVAROrDefault("root:Geochem:gConcFolder","PH_Isco")
	String DischargeFolder 	= StrVAROrDefault("root:Geochem:gDischargeFolder","PH_Isco")
	String FluxFolder 	= StrVAROrDefault("root:Geochem:gFluxFolder","Fluxes")
	String FolderList1 	= ListOfObjectsInFolder(4,"root:")
	String FolderList2 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
	String NewFolderName = ""
	Prompt DischargeFolder, "Discharge Source", popup, FolderList1
	Prompt ConcFolder, "Data Source", popup, FolderList1
	Prompt FluxFolder, "Destination", popup, FolderList2
	Prompt NewFolderName, "Destination"
	DoPrompt "Choose data to normalize", DischargeFolder, ConcFolder, FluxFolder, NewFolderName
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gConcFolder = ConcFolder
	String /G root:Geochem:gDischargeFolder = DischargeFolder
	
	if (cmpstr(FluxFolder,"new - enter below")==0)
		NewDataFolder/O $("root:"+NewFolderName)
		FluxFolder 	= NewFolderName
	endif
	String /G root:Geochem:gFluxFolder = FluxFolder
	
	SetDataFolder $FluxFolder
		KillWaves /A/Z
	
		print "Converting Concentration data in",ConcFolder,"to Fluxes in",FluxFolder
		
		// First, look up the position of cursors on the top plot. 
		WAVE AWave = CsrWaveRef(A)
		WAVE AXWave = CsrXWaveRef(A)
		WAVE BWave = CsrWaveRef(B)
		WAVE BXWave = CsrXWaveRef(B)
		Variable AXVal=-1, BXVal=-1
		
		if (WaveExists(AWave) && WaveExists(BWave))
			String ExpStr =  "Calculating the average exports between "+secs2Date(AXWave[pcsr(A)],0) + " and " + secs2Date(BXWave[pcsr(B)],0)
			AXVal = min(AXWave[pcsr(A)],BXWave[pcsr(B)])
			BXVal = max(AXWave[pcsr(A)],BXWave[pcsr(B)])
		else 
			ExpStr = "Calculating the average exports for the full range"
		endif
		print ExpStr
		
		// Make a list of all the data in the Source folder. 
		String Ele, ConcList = ListOfObjectsInFolder(1,"root:"+ConcFolder)
		String DischList 	= InclusiveWaveList(ListOfObjectsInFolder(1,"root:"+DischargeFolder),"discharge",";")
		String DischName 	= StringFromList(0,DischList)
		Prompt DischName, "Discharge Data", popup, DischList
		DoPrompt "Choose discharge data", DischName
		if (V_flag)
			return 0
		endif
		String DischUnit 		= GeochemUnitFromName(DischName)
		
		WAVE QQ 	= $("root:"+DischargeFolder+":"+DischName)
		WAVE QQd 	= $("root:"+DischargeFolder+":"+GeochemDateFromName(DischName))

		Duplicate /O QQ, $("root:"+FluxFolder+":"+NameOfWave(QQ))
		Duplicate /O QQd, $("root:"+FluxFolder+":"+NameOfWave(QQd))
		
		ConcList 	= ExclusiveWaveList(ExclusiveWaveList(ExclusiveWaveList(ConcList,"_d",";"),DischName,";"),"Species",";")
		
		Variable NSpecies = ItemsInList(ConcList)
		Make /O/T/N=(NSpecies) $("root:"+ConcFolder+":SpeciesNames")
		Make /O/D/N=(NSpecies) $("root:"+ConcFolder+":SpeciesExports"), $("root:"+ConcFolder+":SpeciesSource")
		WAVE /T Names 	= $("root:"+ConcFolder+":SpeciesNames")
		WAVE Exports 	= $("root:"+ConcFolder+":SpeciesExports")
		WAVE Sources 	= $("root:"+ConcFolder+":SpeciesSource")
		
		Variable i, n, nCC, pStart, pStop, Day1, Val1, Val2, NDays=0

		for (n=0;n<NSpecies;n+=1)
			String CCName 		= StringFromList(n,ConcList)
			String CCUnit 		= GeochemUnitFromName(CCName)
			String CCParam 	= GeochemParameterFromName(CCName)
			String CCdName 	= CCParam + "_d"
			String FFName		= CCParam + "_" + CCUnit + "_" + DischUnit
			String FFdName 	= FFName + "_d"
			
			WAVE CC 	= $("root:"+ConcFolder+":"+CCName)
			WAVE CCd 	= $("root:"+ConcFolder+":"+CCdName)
			
			Make /O/D/N=0 $FFName /WAVE=FF
			Make /O/D/N=0 $FFdName /WAVE=FFd
				
			nDays = 0
			nCC = DimSize(CC,0)
			for(i=0;i<nCC;i+=1)
				Day1 	= CCd[i]
				if (numtype(Day1) == 0)
					Val1 	= CC[i]
					if (numtype(Val1) == 0)
						FindValue /V=(Day1) QQd
						if (V_value != -1)
							Val2 	= QQ[V_value]
							if (numtype(Val2) == 0)
								nDays += 1
								Redimension /N=(nDays) FF, FFd
								FF[nDays-1] 	= Val1 * Val2
								FFd[nDays-1] 	= Day1
							endif
						endif
					endif
				endif
			endfor
			
			Names[n] = CCParam + " (" + CCUnit + " x " + DischUnit+")"
			
			if (DimSize(FF,0) == 0)
				KillWaves /Z FF, FFd
			else
		//		SetScale d, FFd[0], FFd[nDays-2],"dat", FFd
				SetScale d, FFd[0], FFd[nDays-2],"dat", FFd
				
				pStart = -1
				if (AXVal > -1)
					FindValue /V=(AXVal) FFd
					pStart 	= V_value
				endif
				if (pStart == -1)
					pStart 	= 0
				endif
				pStop = -1
				if (BXVal > -1)
					FindValue /V=(BXVal) FFd
					pStop 		= V_value
				endif
				if (pStop == -1)
					pStop = DimSize(FFd,0)-1
				endif
				
				WaveStats /Q/R=[pStart,pStop] /M=1 FF
				Variable Export = V_avg
				
				Exports[n] = Export
				Note /K Exports, ExpStr
			endif
			
		endfor
	
	SetDataFolder root:
End

















//// Calculate the Average Export of a Species
//Function SpeciesExport()
//	
//	SetDataFolder root:
//	
//	// Select the Concentration and Discharge folder
//	String ConcFolder 	= StrVAROrDefault("root:gConcFolder","PH_Isco")
//	String FluxFolder 	= StrVAROrDefault("root:gFluxFolder","Flux")
//	ConcFolder 	= ChooseAFolder("",ConcFolder,"Data Source","Choose data folder",0)
//	if (strlen(ConcFolder) == 0)
//		return 0
//	endif
//	String /G gConcFolder = ConcFolder
//	
//	// List all the data in the folder. 
//	String ConcList 	= ListOfObjectsInFolder(1,ConcFolder)
//	ConcList 			= ExclusiveWaveList(ConcList,"_d",";")
//	
//	// Select the Element and Discharge data and Choose a Destination
//	String Species1 	= StrVAROrDefault("root:gSpecies1","")
//	String Species2 	= StrVAROrDefault("root:gSpecies2","")
//	String SaveFolder = StrVAROrDefault("root:gSaveFolder","Correlations")
//	String FolderList 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
//	String NewFolderName = ""
//	
//	
//	Prompt Species1, "Species 1", popup, ConcList
//	Prompt Species2, "Discharge", popup, ConcList
//	Prompt FluxFolder, "Destination", popup, FolderList
//	Prompt NewFolderName, "Destination"
//	DoPrompt "Choose correlation pair", Species1, Species2, FluxFolder, NewFolderName
//	if (V_flag)
//		return 0
//	endif
//	String /G gSpecies1 = Species1
//	String /G gSpecies2 = Species2
//	
//	// Create or Open the Destination
//	if (cmpstr(FluxFolder,"new - enter below")==0)
//		NewDataFolder/O $("root:"+NewFolderName)
//		FluxFolder 	= NewFolderName
//	endif
//	String /G gFluxFolder = FluxFolder
//		
//	SetDataFolder gFluxFolder
//	
//	
//	String Ele 	= StripSuffixBySeparator(Species1,"_")
//	String Unit1 	= ReturnLastSuffix(Species1,"_")
//	String Discharge 	= StripSuffixBySeparator(Species2,"_")
//	String Unit2 	= ReturnLastSuffix(Species2,"_")
//	
//	WAVE CC 	= $("root:"+ConcFolder+":"+Species1)
//	WAVE CCd 	= $("root:"+ConcFolder+":"+Ele+"_d")
//	WAVE QQ 	= $("root:"+ConcFolder+":"+Species2)
//	WAVE QQd 	= $("root:"+ConcFolder+":"+Discharge+"_d")
//	SpeciesFlux(Ele,CC,CCd,Discharge,QQ,QQd)
//	
//	SetDataFolder root:
//End

//Function SpeciesFlux(Ele,CC,CCd,Discharge,QQ,QQd)
//	String Ele, Discharge
//	Wave CC,CCd,QQ,QQd
//	
//	Variable i, Day1, Val1, Day2, Val2, nDays=0
//	
//	Variable Nqq=DimSize(CCd,0), Ncc=DimSize(QQd,0)
//	
//	Make /D/O/N=0 $(Ele+"_Flux") /WAVE=FluxWave
//	Make /D/O/N=0 $(Ele+"_d") /WAVE=DateWave
//	
//	// Loop through all valid Element1 dates
//	for(i=0;i<Ncc;i+=1)
//		Day1 	= CCd[i]
//		if (numtype(Day1) == 0)
//			Val1 	= CC[i]
//			if (numtype(Val1) == 0)
//				FindValue /V=(Day1) QQd
//				if (V_value != -1)
//					Val2 	= QQ[V_value]
//					if (numtype(Val2) == 0)
//						nDays += 1
//						Redimension /N=(nDays) FluxWave, DateWave
//						FluxWave[nDays-1] 	= Val1 * Val2
//						DateWave[nDays-1] 	= Day1
//					endif
//				endif
//			endif
//		endif
//	endfor
//	
//	SetScale d, DateWave[0], DateWave[nDays-1],"dat", DateWave
//End







// Normalize to unity at a selected date range and then shifted to zero
Function ScaleAllConcentrations()

	SetDataFolder root:
	SetDataFolder root:Geochem
	
	String ConcFolder 	= StrVAROrDefault("root:Geochem:gConcFolder","PH_Isco")
	String NormFolder 	= StrVAROrDefault("root:Geochem:gNormFolder","Scaled")
	String FolderList1 	= ListOfObjectsInFolder(4,"root:")
	String FolderList2 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
	String NewFolderName = ""
	Prompt ConcFolder, "Data Source", popup, FolderList1
	Prompt NormFolder, "Destination", popup, FolderList2
	Prompt NewFolderName, "Destination"
	DoPrompt "Choose data to normalize", ConcFolder, NormFolder, NewFolderName
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gConcFolder = ConcFolder
	
	WAVE AWave = CsrWaveRef(A)
	WAVE AXWave = CsrXWaveRef(A)
	WAVE BWave = CsrWaveRef(B)
	WAVE BXWave = CsrXWaveRef(B)
	
	if (WaveExists(AWave) && WaveExists(BWave))
		String Msg = "Scaling concentrations to the average values between  "+secs2Date(AXWave[pcsr(A)],0)+"  and  "+secs2Date(BXWave[pcsr(B)],0) + "  and saving in "+NormFolder 
	else 
		return 0
	endif
	Variable AXVal = min(AXWave[pcsr(A)],BXWave[pcsr(B)])
	Variable BXVal = max(AXWave[pcsr(A)],BXWave[pcsr(B)])
	
	if (cmpstr(NormFolder,"new - enter below")==0)
		NewDataFolder/O/S $("root:"+NewFolderName)
		NormFolder 	= NewFolderName
	endif
	String /G root:Geochem:gNormFolder = NormFolder
	
	String Ele, ConcList = ListOfObjectsInFolder(1,"root:"+ConcFolder)
	ConcList 			= ExclusiveWaveList(ConcList,"_d",";")
	Variable i, j, k=0, NConcs 	= ItemsInList(ConcList), pStart, pStop
	
	for (i=0;i<NConcs;i+=1)
		String CCName 	= StringFromList(i,ConcList)
		String EleName 	= StripSuffixBySeparator(CCName,"_")
		String NmName	= EleName + "_Nm"
		String DDName 	= EleName + "_d"
		
		WAVE CC 	= $("root:"+ConcFolder+":"+CCName)
		WAVE DD 	= $("root:"+ConcFolder+":"+DDName)
		
		// FindValue looks for a specific day value
		FindValue /V=(AXVal) DD
		pStart 	= V_value
		FindValue /V=(BXVal) DD
		pStop 	= V_value
		
		if (pStart == -1)
			FindLevel /Q/P/EDGE=1 DD, AXVal
			pStart = (V_flag == 0) ? floor(V_LevelX) : pStart
		endif
		
		if  (pStop == -1)
			FindLevel /Q/P/EDGE=1 DD, BXVal
			pStop = (V_flag==0) ? floor(V_LevelX) : pStop
		endif
		
		if ((pStart == -1) || (pStop == -1))
			Print " 	- no overlap so quit"
//		elseif (cmpstr(CCName,"DOC_ppm") != 0)	// CRUDE to do a single trace. 
		elseif (cmpstr(CCName,"SO4_uM") != 0)	// CRUDE to do a single trace. 
		
		else
			print Msg
			
			Duplicate /O/D CC, $("root:"+NormFolder+":"+NmName)
			Duplicate /O/D DD, $("root:"+NormFolder+":"+DDName)
			WAVE NN 	= $("root:"+NormFolder+":"+NmName)
			WAVE ND 	= $("root:"+NormFolder+":"+DDName)
			WaveStats /Q/R=[pStart,pStop] /M=1 CC
			NN /= V_avg
			
			k = 0 	// Is this ever necessary??
			Variable NVals = DimSize(NN,0)
			for (j=0;j<NVals;j+=1)
				if ((numtype(NN[k]) != 0) || (numtype(ND[k]) != 0))
					DeletePoints k,1, NN, ND
				else
					k+=1
				endif
			endfor
			
		endif
	endfor
	
	SetDataFolder root:
End



// Create pairs of concentration data for any two species. Also the common date values. 
// Would be worth to also create the common discharge values? 
Function TwoSpeciesCorrelations()
	
	SetDataFolder root:

	String ConcFolder 		= StrVAROrDefault("root:Geochem:gConcFolder","ER_PH_ISCO")
	String ConcFolder2 	= StrVAROrDefault("root:Geochem:gConcFolder2","same")
	Prompt ConcFolder, "Data source", popup, ListOfObjectsInFolder(4,"root:")
	Prompt ConcFolder2, "Second source", popup, "same;"+ListOfObjectsInFolder(4,"root:")
	DoPrompt "Choose data folders for correlations", ConcFolder, ConcFolder2
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gConcFolder = ConcFolder
	if (cmpstr(ConcFolder2,"same") == 0)
		ConcFolder2 	= ConcFolder
	endif
	String /G root:Geochem:gConcFolder2 = ConcFolder2
	
	String ConcList 	= ListOfObjectsInFolder(1,ConcFolder)
	ConcList 				= ExclusiveWaveList(ConcList,"_d",";")
	String ConcList2 	= ListOfObjectsInFolder(1,ConcFolder2)
	ConcList2 			= ExclusiveWaveList(ConcList2,"_d",";")
	
	String Species1 	= StrVAROrDefault("root:Geochem:gSpecies1","")
	String Species2 	= StrVAROrDefault("root:Geochem:gSpecies2","")
	String SaveFolder = StrVAROrDefault("root:Geochem:gSaveFolder","Correlations")
	String FolderList 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
	String NewFolderName = ""
	
	Prompt Species1, "Species 1", popup, ConcList
	Prompt Species2, "Species 2", popup, ConcList2
	Prompt SaveFolder, "Destination", popup, FolderList
	Prompt NewFolderName, "Destination"
	DoPrompt "Choose correlation pair", Species1, Species2, SaveFolder, NewFolderName
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gSpecies1 = Species1
	String /G root:Geochem:gSpecies2 = Species2
	
	if (cmpstr(SaveFolder,"new - enter below")==0)
		NewDataFolder/O $("root:"+NewFolderName)
		SaveFolder 	= NewFolderName
	endif
	String /G root:Geochem:gSaveFolder = SaveFolder
		
	SetDataFolder SaveFolder
	
	Print " * Calculating correlations between",Species1,"at",ConcFolder,"and",Species2,"at",ConcFolder2
	String CCNote = "CType=SpeciesCorrelation;Data1Folder="+ConcFolder+";Data2Folder="+ConcFolder2+";"
	
	String Ele1 	= StripSuffixBySeparator(Species1,"_")
	String Unit1 	= ReturnLastSuffix(Species1,"_")
	String Ele2 	= StripSuffixBySeparator(Species2,"_")
	String Unit2 	= ReturnLastSuffix(Species2,"_")
	
	WAVE CC1 	= $("root:"+ConcFolder+":"+Species1)
	WAVE CC2 	= $("root:"+ConcFolder2+":"+Species2)
	
	// Stupidly, there are 2 conventions
	WAVE DD1 	= $("root:"+ConcFolder+":"+Ele1+"_d")
	WAVE DD2 	= $("root:"+ConcFolder2+":"+Ele2+"_d")
	if (!WaveExists(DD1))
		WAVE DD1 	= $("root:"+ConcFolder+":"+Species1+"_d")
	endif
	if (!WaveExists(DD2))
		WAVE DD2 	= $("root:"+ConcFolder2+":"+Species2+"_d")
	endif
	if (!WaveExists(DD1) || !WaveExists(DD2))
		Print " 	- cannot locate date wave - check naming"
		return 0
	endif
	
	CCCalculation(Ele1,CC1,DD1,Ele2,CC2,DD2,CCNote)
	
	SetDataFolder root:
End

Function CCCalculation(Ele1,CC1,DD1,Ele2,CC2,DD2,CCNote)
	String Ele1, Ele2, CCNote
	Wave CC1,DD1,CC2,DD2
	
	Variable i, Day1, Val1, Day2, Val2, nDays=0
	
	Variable NN1=DimSize(DD1,0), NN2=DimSize(DD2,0)
	
	if (cmpstr(Ele1,Ele2) ==0)
		Ele1 = Ele1 +"a"
		Ele2 = Ele2 +"b"
	endif
	
	if (strlen(Ele1+Ele1)<20)
		String Corr1Name = Ele1+"_"+Ele2+"_"+Ele1
		String Corr2Name = Ele1+"_"+Ele2+"_"+Ele2
	else
		Corr1Name = Ele1+"_"+Ele2+"_1"
		Corr2Name = Ele1+"_"+Ele2+"_2"
	endif
	
	Make /D/O/N=0 $(Corr1Name) /WAVE=Ele1Wave
	Make /D/O/N=0 $(Corr2Name) /WAVE=Ele2Wave
	Note Ele1Wave, Ele1+"_"+Ele2+"_"+Ele1
	Note Ele2Wave, Ele1+"_"+Ele2+"_"+Ele2
	Make /D/O/N=0 $(Ele1+"_"+Ele2+"_d") /WAVE=DateWave
	Make /O/N=0 $(Ele1+"_"+Ele2+"_RGB") /WAVE=RGBWave
	
	// Loop through all valid Element1 dates
	for(i=0;i<NN1;i+=1)
		Day1 	= DD1[i]
		if (numtype(Day1) == 0)
			Val1 	= CC1[i]
			if (numtype(Val1) == 0)
				FindValue /V=(Day1) DD2
				if (V_value != -1)
					Val2 	= CC2[V_value]
					if (numtype(Val2) == 0)
						nDays += 1
						Redimension /N=(nDays) Ele1Wave, Ele2Wave, DateWave, RGBWave
						Ele1Wave[nDays-1] 	= Val1
						Ele2Wave[nDays-1] 	= Val2
						DateWave[nDays-1] 	= Day1
					endif
				endif
			endif
		endif
	endfor
	
	if (nDays==0)
		Print " 	- no common days - cannot calculate correlations" 
		KillWaves /Z Ele1Wave, Ele2Wave, DateWave, RGBWave
		return 0
	endif
	
	Note /K Ele1Wave,CCNote
	Note /K Ele2Wave,CCNote
	SetScale d, DateWave[0], DateWave[nDays-1],"dat", DateWave
End

Function TwoSpeciesMath()
	
	SetDataFolder root:

	String ConcFolder 		= StrVAROrDefault("root:Geochem:gConcFolder","ER_PH_ISCO")
	String ConcFolder2 	= StrVAROrDefault("root:Geochem:gConcFolder2","same")
	Prompt ConcFolder, "Data source", popup, ListOfObjectsInFolder(4,"root:")
	Prompt ConcFolder2, "Second source", popup, "same;"+ListOfObjectsInFolder(4,"root:")
	DoPrompt "Choose data folders for calculations", ConcFolder, ConcFolder2
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gConcFolder = ConcFolder
	if (cmpstr(ConcFolder2,"same") == 0)
		ConcFolder2 	= ConcFolder
	endif
	String /G root:Geochem:gConcFolder2 = ConcFolder2
	
	String ConcList 	= ListOfObjectsInFolder(1,ConcFolder)
	ConcList 				= ExclusiveWaveList(ConcList,"_d",";")
	String ConcList2 	= ListOfObjectsInFolder(1,ConcFolder2)
	ConcList2 			= ExclusiveWaveList(ConcList2,"_d",";")
	
	String Species1 	= StrVAROrDefault("root:Geochem:gSpecies1","")
	String Species2 	= StrVAROrDefault("root:Geochem:gSpecies2","")
	Variable MathValue = NumVAROrDefault("root:Geochem:gMathValue",1)
	String Operation 	= StrVAROrDefault("root:Geochem:gOperation","Add")
	String ResultName 	= StrVAROrDefault("root:Geochem:gResultName","XdivY")
	String SaveFolder 	= StrVAROrDefault("root:Geochem:gSaveFolder","Correlations")
	String FolderList 	= "new - enter below;"+ListOfObjectsInFolder(4,"root:")
	String NewFolderName = ""
	
	Prompt Species1, "Species 1", popup, ConcList
	Prompt Species2, "Species 2 of value", popup, "value;" + ConcList2
	Prompt MathValue, "Optional value"
	Prompt Operation, "Operation", popup, "Add;Divide;Subtract;Multiply;"
	Prompt ResultName, "Name the result or blank for default" 
	Prompt SaveFolder, "Destination", popup, FolderList
	Prompt NewFolderName, "Destination"
	DoPrompt "Choose data pair and operation", Species1, Species2, Operation, MathValue, ResultName, SaveFolder, NewFolderName
	if (V_flag)
		return 0
	endif
	String /G root:Geochem:gSpecies1 = Species1
	String /G root:Geochem:gSpecies2 = Species2
	Variable /G root:Geochem:gMathValue = MathValue
	String /G root:Geochem:gOperation = Operation
	String /G root:Geochem:gResultName = ResultName
	
	if (cmpstr(SaveFolder,"new - enter below")==0)
		NewDataFolder/O $("root:"+NewFolderName)
		SaveFolder 	= NewFolderName
	endif
	String /G root:Geochem:gSaveFolder = SaveFolder
		
	SetDataFolder $(SaveFolder)
	
	Print " * Calculating ",Species1,"in",ConcFolder,Operation,Species2,"in",ConcFolder2,"and saving in",SaveFolder
	String CCNote = "CType=SpeciesMathOperation;Operation="+Operation+";Data1Folder="+ConcFolder+";Data2Folder="+ConcFolder2+";"
	
	String Ele1 	= StripSuffixBySeparator(Species1,"_")
	String Unit1 	= ReturnLastSuffix(Species1,"_")
	String Ele2 	= StripSuffixBySeparator(Species2,"_")
	String Unit2 	= ReturnLastSuffix(Species2,"_")
	
	WAVE CC1 	= $("root:"+ConcFolder+":"+Species1)
	WAVE CC2 	= $("root:"+ConcFolder2+":"+Species2)
	
	// Stupidly, there are 2 conventions
	WAVE DD1 	= $("root:"+ConcFolder+":"+Ele1+"_d")
	WAVE DD2 	= $("root:"+ConcFolder2+":"+Ele2+"_d")
	if (!WaveExists(DD1))
		WAVE DD1 	= $("root:"+ConcFolder+":"+Species1+"_d")
	endif
	if (!WaveExists(DD2))
		WAVE DD2 	= $("root:"+ConcFolder2+":"+Species2+"_d")
	endif
	if ((!WaveExists(DD1)) || (!WaveExists(DD1)))
		Print " 	- cannot locate date wave - check naming"
		return 0
	endif
	
	MathCalculation(Ele1,Unit1,CC1,DD1,Ele2,Unit2,CC2,DD2,Operation,MathValue,ResultName,CCNote)
	
	SetDataFolder root:
End

Function MathCalculation(Ele1,Unit1,CC1,DD1,Ele2,Unit2,CC2,DD2,Operation,MathValue,ResultName,CCNote)
	String Ele1, Ele2, Unit1, Unit2, Operation, ResultName, CCNote
	Wave CC1,DD1,CC2,DD2
	Variable MathValue
	
	Variable i, Day1, Val1, Day2, Val2, nDays=0
	
	Variable NN1=DimSize(DD1,0), NN2=DimSize(DD2,0)
	
	if (strlen(ResultName) == 0)
		if (cmpstr(Ele1,Ele2) ==0)
			Ele1 = Ele1 +"a"
			Ele2 = Ele2 +"b"
		endif
//		ResultName = Ele1+"_"+Operation[0,2]+"_"+Ele2
		ResultName = Ele1+Operation[0,2]+Ele2+"_"+Unit1
	endif
	
	Make /D/O/N=0 $(ResultName) /WAVE=MathWave
	Make /D/O/N=0 $(ResultName+"_d") /WAVE=DateWave
	Make /O/N=0 $(ResultName+"_RGB") /WAVE=RGBWave
	
	// Loop through all valid Element1 dates
	for(i=0;i<NN1;i+=1)
		Day1 	= DD1[i]
		if (numtype(Day1) == 0)
			Val1 	= CC1[i]
			if (numtype(Val1) == 0)
				if (WaveExists(CC2))
					FindValue /V=(Day1) DD2
					if (V_value != -1)
						Val2 	= CC2[V_value]
					endif
				else
					Val2 = MathValue
				endif
					
					if (numtype(Val2) == 0)
						nDays += 1
						Redimension /N=(nDays) MathWave, DateWave, RGBWave
						DateWave[nDays-1] 	= Day1
						if (cmpstr(Operation,"Divide") == 0)
							MathWave[nDays-1] 	= Val1/Val2
						elseif (cmpstr(Operation,"Add") == 0)
							MathWave[nDays-1] 	= Val1+Val2
						elseif (cmpstr(Operation,"Multiply") == 0)
							MathWave[nDays-1] 	= Val1*Val2
						elseif (cmpstr(Operation,"Subtract") == 0)
							MathWave[nDays-1] 	= Val1-Val2
						endif
					endif
			endif
		endif
	endfor
	
	if (nDays==0)
		Print " 	- no common days - cannot perform operation" 
		KillWaves /Z MathWave, DateWave, RGBWave
		return 0
	endif
	
	Note /K MathWave,CCNote
	SetScale d, DateWave[0], DateWave[nDays-1],"dat", DateWave
End




//Function CQCalculation2()
//
//	NewDataFolder/O/S root:CQ
//	
//	WAVE Discharge 	= root:discharge
//	WAVE Dis_Date 		= root:discharge_d
//	SetScale d 0,0,"", Dis_Date
//	
//	String Ele, DataList = DataFolderDir(2,root:)
//	String DataList1 = InclusiveWaveList(DataList,"ppb",",")
//	String DataList2 = InclusiveWaveList(DataList,"uM",",")
//	
//	Variable i, NData = ItemsinList(DataList1,",")
//	
//	// metals in ppb
////	for (i=0;i<NData;i+=1)
////		Ele 	= StripSuffixBySeparator(StringFromList(i,DataList1,","),"_")
////		WAVE Element = $("root:"+Ele+"_ppb")
////		WAVE Ele_Date = $("root:"+Ele+"_d")
////		SetScale d 0,0,"", Ele_Date
////		CQCorrelation(Ele,Discharge,Dis_Date,Element,Ele_Date)
////	endfor
//	
//	NData = ItemsinList(DataList2,",")
//	
//	// species in uM
//	for (i=0;i<NData;i+=1)
//		Ele 	= StripSuffixBySeparator(StringFromList(i,DataList2,","),"_")
//		WAVE Element = $("root:"+Ele+"_uM")
//		WAVE Ele_Date = $("root:"+Ele+"_d")
//		SetScale d 0,0,"", Ele_Date
//		if (stringmatch(Ele,"N"))
//			CQCalculation(Ele,Discharge,Dis_Date,Element,Ele_Date)
//		endif
//	endfor
//	
//End



//	Function CQCalculation()
//	
//		NewDataFolder/O/S root:CQ
//		
//		WAVE Discharge 	= root:discharge
//		WAVE Dis_Date 		= root:discharge_d
//		SetScale d 0,0,"", Dis_Date
//		
//		// Loop through all elements
//		Variable i, j, nDates=0, EDate, EVal, DateStart=0
//		Variable NEDates, NDDates = DimSize(Dis_Date,0)
//		String ZStr, ZName, ZCName, ZQName
//		
//		for (i=0;i<100;i+=1)
//		
//		ZStr 		= ZNumberToAtom(i)
//		ZName 	= ZStr + "_ppb"
//		ZCName 	= ZStr + "_C"
//		ZQName 	= ZStr + "_Q"
//		
//		WAVE element 	= $("root:" + ZName)
//		WAVE ele_date 	= $("root:" + ZStr + "_d")
//	
//			if (WaveExists(element))
//				NEDates 	= DimSize(ele_date,0)
//				SetScale d 0,0,"", ele_date
//				
//				Make /O/D/N=0 $ZCName /WAVE=CC
//				Make /O/D/N=0 $ZQName /WAVE=QQ
//				
//				for(j=0;j<NEDates;j+=1)
//				
//					EDate = ele_date[j]
//					EVal = element[j]
//					
//					if ((numtype(EDate) == 2) || (numtype(EVal) == 2))
//					else
//						FindValue /V=(EDate) /S=(DateStart) Dis_Date
//						
//						if (V_value != -1)
//							nDates += 1
//							Redimension /N=(nDates) CC, QQ
//							CC[nDates-1] 	= element[V_value]
//							QQ[nDates-1] 	= Discharge[V_value]
//		//					DateStart = V_value
//						endif
//						
//					endif
//				endfor
//				
//			endif
//			
//	//		Variable nn=0
//	//		for (j=0;j<nDates;j+=1)
//	//			if (numtype(CC[nn]) == 2)
//	//				DeletePoints nn, 1, CC, QQ
//	//			else
//	//				nn+=1
//	//			endif
//	//		endfor
//		
//		endfor
//		
//		SetDataFolder root:
//	
//	End