#pragma rtGlobals=1		// Use modern global access method.

Function DisplayPolarizationPanel()

	NewDataFolder /O root:POLARIZATION
	
	// Bad way of preventing error, but if somebody tries to display the Polarization Panel before
	// the Stack Browser has been initialized, they'll receive an error that will crash all macros.
	// If I could verify the existance of the SPHINX folders, gStackName would presumably not be null
	// and this would be unnecessary. For now... Workaround.
	InitializeStackBrowser()
	//Also, we need the SVD data folder to exist in order to get ROI information... So...
	NewDataFolder /O root:SPHINX:SVD

	// Associate Polarization Panel with the desired Stack
	MakeStringIfNeeded("root:SPHINX:Browser:gStackName","")
	SVAR gStackName 	= root:SPHINX:Browser:gStackName
	gStackName 	= ChooseStack(" Choose a stack to analyze polarization",gStackName,0)
	
	WAVE SPHINXStack	= $("root:SPHINX:Stacks:"+gStackName)
	if (WaveExists(SPHINXStack) == 1)
		Variable NumX = Dimsize(SPHINXStack,0)
		Variable NumY = Dimsize(SPHINXStack,1)
		Variable NumE = Dimsize(SPHINXStack,2)
		
		MakeStringIfNeeded("root:POLARIZATION:StackName","")
		SVAR StackName 	= root:POLARIZATION:StackName
		StackName = gStackName
	
		PolarizationAnalysisPanel(SPHINXStack,"Polarization Analysis: "+gStackName,"Analysis",NumX,NumY,NumE)
	endif

End

Function PolarizationAnalysisPanel(SPHINXStack,Title,Folder,NumX,NumY,NumE)
	Wave SPHINXStack
	String Title,Folder
	Variable NumX, NumY, NumE
	
	// Locations of panel globals
	String PanelFolder 	= "root:POLARIZATION:"+Folder
	String PanelName 	= "Polarization"+Folder
	
	NewDataFolder /O/S $PanelFolder
	Make /O/D/N=(NumE) spectrum,fit_spectrum
	Make /O/D/N=3 root:POLARIZATION:NumData
	Wave NumData = $("root:POLARIZATION:NumData")
	NumData[0] = NumX
	NumData[1] = NumY
	NumData[2] = NumE

	
	DoWindow /K $PanelName
	NewPanel /K=1/W=(350,50,650,350) as Title
	Dowindow /C $PanelName
	CheckWindowPosition(PanelName,350,50,650,350)
	
	// Create the Plot of Extracted Spectra
	Display/W=(0,130,300,300)/HOST=# spectrum vs root:SPHINX:Browser:energy
	AppendToGraph fit_spectrum
	ModifyGraph rgb(fit_spectrum)=(0,0,65535)
	RenameWindow #, PixelPlot
	SetActiveSubwindow ##
	Display/W=(150,0,300,130)/HOST=#
	RenameWindow #, PixelData
	// Populate an initial single pixel plot
	AnalyzePixel()
	
	// Group Box and Buttons
	GroupBox PolAnalyzeGroup,pos={15,3},size={120,67}, title="Pol Analysis",fColor=(39321,1,1)
	Button AnalyzePixel,pos={23,22}, size={100,20},fSize=13,proc=PolarizationPanelButtons,title="Analyze Pixel"
	Button AnalyzeStack,pos={23,47}, size={100,20},fSize=13,proc=PolarizationPanelButtons,title="Analyze ROI"
End

Function AnalyzePixel()

	ExtractPixelPolarizations()
	FitCosSq()

	//Generate Text Box Results
	NVAR PixelAX	= $("root:SPHINX:Browser:gCursorAX")
	NVAR PixelAY	= $("root:SPHINX:Browser:gCursorAY")
	NVAR chisq	= $("root:POLARIZATION:Analysis:chisq")
	Wave FitCoef	= $("root:POLARIZATION:Analysis:W_coef")
	NVAR MaxPolAngle = $("root:POLARIZATION:Analysis:MaxPolAngle")
	SetActiveSubwindow PolarizationAnalysis#PixelData
	String ResultsText = "X:\t"+num2str(PixelAX)+"\rY:\t"+num2str(PixelAY)+"\r\rf(x)=a + b cos(x + c)\S2\M\rChiSq:\t"+num2str(chisq)+"\ra:\t"+num2str(FitCoef[0])+"\rb:\t"+num2str(FitCoef[1])+"\rc:\t"+num2str(FitCoef[2])+"\rPeak Angle:\t"+num2str(MaxPolAngle)
	TextBox /F=0 /C /N=ResultsBox /A=LT ResultsText
	SetActiveSubwindow ##
	
End

Function AnalyzeStack()
	
	Wave NumData = $("root:POLARIZATION:NumData")
	//Variable MinX = 0
	//Variable MinY = 0
	Variable MaxX = NumData[0]
	Variable MaxY = NumData[1]
	//Variable MaxX = 128
	//Variable MaxY = 128
	//Get ROI Positions from SVD Folder
	NVAR X1 	= root:SPHINX:SVD:gSVDLeft
	NVAR X2 	= root:SPHINX:SVD:gSVDRight
	NVAR Y1 	= root:SPHINX:SVD:gSVDBottom
	NVAR Y2 	= root:SPHINX:SVD:gSVDTop
	if (!NVAR_Exists(X1))
		Print " *** Please select an ROI"
		return 0
	endif
	
	Variable i,j,n,m,nm,nMax,mMax,V_chisq,V_max,MaxPolAngle
	NVAR Bin = root:SPHINX:Browser:gCursorBin
	String BrowserFolder = "root:SPHINX:Browser"
	String PanelFolder = "root:POLARIZATION:Analysis"
	String StackFolder = "root:SPHINX:Stacks"
	SVAR StackName = root:POLARIZATION:StackName
	WAVE spectrum 	= $(PanelFolder + ":spectrum")
	WAVE SPHINXStack = $(StackFolder+":"+StackName)

	Make /O/N=(NumData[0],NumData[1])/D $(StackFolder+":POL_Analysis_"+StackName)
	Make /O/N=(NumData[0],NumData[1])/D $(StackFolder+":POL_ChiSq_"+StackName)
	Make /O/N=(NumData[0],NumData[1])/D $(StackFolder+":POL_Intensity_"+StackName)
	WAVE /D POLAnalysis = $("root:SPHINX:Stacks:POL_Analysis_"+StackName)
	WAVE /D POLChiSq = $("root:SPHINX:Stacks:POL_ChiSq_"+StackName)
	WAVE /D POLIntensity = $("root:SPHINX:Stacks:POL_Intensity_"+StackName)
	
	Variable Duration, timeRef = startMSTimer
	OpenProcBar("Analyzing Polarization Angle Intensity Data for "+StackName)
	
	for (i=X1;i<X2;i+=Bin)
		for (j=Y1;j<Y2;j+=Bin)
			nm 		= 0
			//May get data from a bit past ROI, but binning stats are more accurate for Intensity data!
			mMax 	= (i+Bin > MaxX) ? MaxX : (i+Bin)
			nMax 	= (j+Bin > MaxY) ? MaxY : (j+Bin)
			spectrum = 0
				
			//EXTRACT SPECTRUM from PIXEL i,j (with Binning, if specified)
				for (m=i;m < mMax;m+=1)
					for (n=j;n < nMax;n+=1)
							spectrum[] 	+= SPHINXStack[m][n][p]
							nm += 1
					endfor
				endfor
		
			//If Binning worked, continue
				if (nm > 0)
				// Normalize Extracted Spectra, per settings in Stack Browser
					NVAR gInvertSpectraFlag	= $(BrowserFolder+":gInvertSpectraFlag")
					if (gInvertSpectraFlag == 0)
						spectrum = -1*spectrum
					endif
					spectrum /= Bin^2
				
				//PERFORM FIT FUNCTION FOR spectrum
					Make/D/N=3/O W_coef
					W_coef[0] = {120,20,1}
					FuncFit/NTHR=0/N/Q=1 cos_squared W_coef  spectrum /X=:::SPHINX:Browser:energy /D
					
					WaveStats /Q fit_spectrum
		
					MaxPolAngle = 0
					if ((V_maxloc > 100) && (V_maxloc < 190))
						MaxPolAngle = V_maxloc
					elseif ((V_minloc > 0) && (V_minloc <= 145))
						MaxPolAngle = V_minloc + 90
					elseif ((V_minloc > 145) && (V_minloc < 190))
						MaxPolAngle = V_minloc - 90
					elseif ((V_minloc == 100) && (V_maxloc == 190))
						MaxPolAngle = 190
					elseif ((V_minloc == 190) && (V_maxloc == 100))
						MaxPolAngle = 100	
					else
						MaxPolAngle = 999
					endif
				
			//If Binning didn't work
								
				else
					MaxPolAngle	= NaN
					V_Chisq		= NaN
					V_max		= NaN
				endif
				
			//ASSIGN RESULTS TO NEW STACK
				for (m=i;m < mMax;m+=1)
					for (n=j;n < nMax;n+=1)
						POLAnalysis[m][n] = MaxPolAngle
						POLChiSq[m][n] = V_Chisq
						POLIntensity[m][n] = V_max
					endfor
				endfor
		endfor
		UpdateProcessBar((i-X1)/(X2-X1))
	endfor
	
	CloseProcessBar()
	
	Duration = stopMSTimer(timeRef)/1000000
	Print " 		.............. took  ", Duration,"  seconds for ",(X2-X1)*(Y2-Y1),"pixels. "
	SetActiveSubwindow PolarizationAnalysis#PixelData
	String ResultsText = "Stack Analysis Completed\r\rView Analysis and X\S2\M\rimages in SPHINX"
	TextBox /F=0 /C /N=ResultsBox /A=LT ResultsText
	SetActiveSubwindow ##
	
End

Function FitCosSq()

	Make/D/N=3/O W_coef
	W_coef[0] = {120,20,1}
	FuncFit/NTHR=0/Q=1 cos_squared W_coef  spectrum /X=:::SPHINX:Browser:energy /D
	
	MakeVariableIfNeeded("root:POLARIZATION:Analysis:chisq",0)
	NVAR chisq = $("root:POLARIZATION:Analysis:chisq")
	chisq = V_chisq
	
	WaveStats /Q fit_spectrum
	//V_minloc	X location of minimum Y value.
	//V_min	Minimum Y value.
	//V_maxloc	X location of maximum Y value.
	//V_max	Maximum Y value.
	MakeVariableIfNeeded("root:POLARIZATION:Analysis:MaxPolAngle",0)
	NVAR MaxPolAngle = $("root:POLARIZATION:Analysis:MaxPolAngle")
	if ((V_maxloc > 100) && (V_maxloc < 190))
		MaxPolAngle = V_maxloc
	elseif ((V_minloc > 0) && (V_minloc <= 145))
		MaxPolAngle = V_minloc + 90
	elseif ((V_minloc > 145) && (V_minloc < 190))
		MaxPolAngle = V_minloc - 90
	elseif ((V_minloc == 100) && (V_maxloc == 190))
		MaxPolAngle = 190
	elseif ((V_minloc == 190) && (V_maxloc == 100))
		MaxPolAngle = 100	
	else
		MaxPolAngle = 999
	endif
	
End

Function ExtractPixelPolarizations()
	Wave NumData = $("root:POLARIZATION:NumData")
	Variable NumX = NumData[0]
	Variable NumY = NumData[1]
	String BrowserFolder = "root:SPHINX:Browser"
	String PanelFolder = "root:POLARIZATION:Analysis"
	String StackFolder = "root:SPHINX:Stacks"
	SVAR StackName = root:POLARIZATION:StackName
	WAVE SPHINXStack = $(StackFolder+":"+StackName)
	
	NVAR PixelAX	= $("root:SPHINX:Browser:gCursorAX")
	
	Variable CsrX = NumVarOrDefault(BrowserFolder + ":gCursorAX",NumX/2)
	Variable CsrY = NumVarOrDefault(BrowserFolder + ":gCursorAY",NumY/2)
	
	if ((numtype(CsrX) == 2) || (numtype(CsrY) == 2))
		return 0
	endif
			
	NVAR gCursorBin	= $(BrowserFolder+":gCursorBin")
	WAVE spectrum 	= $(PanelFolder + ":spectrum")
	
	// hBox bins, if specified in the StackBrowser.
	Variable i, j, hBox = trunc(gCursorBin/2)
	
	spectrum 	= 0 	// Perhaps ImageTransform setBeam would be faster? 
	for (i=(CsrX-hBox);i<=(CsrX+hBox);i+=1)
		for (j=(CsrY-hBox);j<=(CsrY+hBox);j+=1)
			spectrum[] 	+= SPHINXStack[i][j][p]
		endfor
	endfor
	
	// Normalize Extracted Spectra, per settings in Stack Browser
	NVAR gInvertSpectraFlag	= $(BrowserFolder+":gInvertSpectraFlag")
	if (gInvertSpectraFlag == 0)
		spectrum = -1*spectrum
	endif
	
	spectrum /= gCursorBin^2

End

// *************************************************************
// ****	Button controls on the Polarization Analysis panel only
// *************************************************************
Function PolarizationPanelButtons(B_Struct)
	STRUCT WMButtonAction &B_Struct
	
	String ctrlName 	= B_Struct.ctrlName
	String WindowName = B_Struct.win
	
	Variable eventCode 	= B_Struct.eventCode
	if (eventCode != 2)
		return 0	// Mouse up after pressing
	endif
	
	if (cmpstr(ctrlName,"AnalyzePixel") == 0)
		AnalyzePixel()
	elseif (cmpstr(ctrlName,"AnalyzeStack") == 0)
		try
			AnalyzeStack()
		catch
			CloseProcessBar()
			StopAllTimers()
			if (V_AbortCode == -1)
				Print " *** Polarization analysis routine aborted by user."
			else
				Print " *** Polarization analysis routine aborted due to error."
			endif
		endtry
			
			
	endif
End

// *************************************************************
// ****		Cos Squared Fit Function
// *************************************************************
Function cos_squared(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = a + b * (cos(x + c)) ^ 2
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 3
	//CurveFitDialog/ w[0] = a
	//CurveFitDialog/ w[1] = b
	//CurveFitDialog/ w[2] = c

	return w[0] + w[1] * (cos((pi*x/180) + w[2])) ^ 2
End