#pragma rtGlobals=1		// Use modern global access method.

Function RGBMapExportPanel()

	String Title = "Export 3 Component Maps as RGB Image"
	String PanelName = "RGBMapExport"
	String PanelFolder = "root:SPHINX:SVD"

	SetDataFolder root:SPHINX:SVD:

		DoWindow /K $PanelName
		NewPanel /K=1/W=(6,44,360,135) as Title
		DoWindow /C $PanelName
		CheckWindowPosition(PanelName,666,44,1060,135)

		Button CreateRGBMap,pos={10,15}, size={120,25},fSize=13,proc=RGBMapButtons,title="Create RGB Map"

		MakeVariableIfNeeded("root:SPHINX:SVD:gRGB_MaskFlag",1)
		CheckBox RGBMaskCheck,pos={270,10},size={212,577},title="Use Mask",fSize=11,variable=$("root:SPHINX:SVD:gRGB_MaskFlag")
		MakeVariableIfNeeded("root:SPHINX:SVD:gRGB_BEFlag",1)
		CheckBox RGBBECheck,pos={140,10},size={212,577},title="Enhance brightness",fSize=11,variable=$("root:SPHINX:SVD:gRGB_BEFlag")
		MakeVariableIfNeeded("root:SPHINX:SVD:gRGB_RGBSaveFlag",1)
		CheckBox RGBSaveRGBCheck,pos={140,26},size={212,577},title="Save RGB",fSize=11,variable=$("root:SPHINX:SVD:gRGB_RGBSaveFlag"),proc=RGBMapChecks
		MakeVariableIfNeeded("root:SPHINX:SVD:gRGB_PSaveFlag",1)
		CheckBox RGBSavePMapCheck,pos={230,26},size={212,577},title="Save pMaps",fSize=11,variable=$("root:SPHINX:SVD:gRGB_PSaveFlag"),proc=RGBMapChecks

		MakeVariableIfNeeded(PanelFolder+":Comp0",0)
		SetVariable Comp0,title="Components:   R",pos={5,65},limits={0,65535,1},size={145,25},fsize=13,value=$("root:SPHINX:SVD:Comp0")
		MakeVariableIfNeeded(PanelFolder+":Comp1",1)
		SetVariable Comp1,title="G",pos={155,65},limits={0,65535,1},size={50,25},fsize=13,value=$("root:SPHINX:SVD:Comp1")
		MakeVariableIfNeeded(PanelFolder+":Comp2",2)
		SetVariable Comp2,title="B",pos={215,65},limits={0,65535,1},size={50,25},fsize=13,value=$("root:SPHINX:SVD:Comp2")
	
	SetDataFolder root:
End

Function RGBMapChecks(CB_Struct) : CheckBoxControl 
	STRUCT WMCheckboxAction &CB_Struct 
	
	Variable eventCode		= CB_Struct.eventCode
	if (eventCode != 2)
		return 0
	endif
	
	String ctrlName			= CB_Struct.ctrlName
	Variable checked		= CB_Struct.checked
	
	NVAR gRGB_RGBSaveFlag 	= root:SPHINX:SVD:gRGB_RGBSaveFlag
	NVAR gRGB_PSaveFlag 		= root:SPHINX:SVD:gRGB_PSaveFlag
	
	// Don't allow saving the p-maps without the RGB map
	if (StrSearch(ctrlName,"RGBSavePMapCheck",0) > -1)
		if (checked)
			gRGB_RGBSaveFlag = 1
		endif
	elseif (StrSearch(ctrlName,"RGBSaveRGBCheck",0) > -1)
		if (gRGB_PSaveFlag == 1)
			gRGB_RGBSaveFlag = 1
		endif
	endif
End

Function RGBMapButtons(B_Struct)
	STRUCT WMButtonAction &B_Struct

	String ctrlName = B_Struct.ctrlName
	String WindowName = B_Struct.win

	Variable eventCode = B_Struct.eventCode
	if (eventCode != 2)
		return 0	// Mouse up after pressing
	endif
	
	Variable Success = CreateRGBMap()
	
	return Success
End

Function CreateRGBMap()

	SetDataFolder root:SPHINX:SVD:
	
	SVAR gStackName 	= root:SPHINX:Browser:gStackName
	
	NVAR gRGB_MaskFlag 		= root:SPHINX:SVD:gRGB_MaskFlag
	NVAR gRGB_BEFlag 			= root:SPHINX:SVD:gRGB_BEFlag
	NVAR gRGB_RGBSaveFlag 	= root:SPHINX:SVD:gRGB_RGBSaveFlag
	NVAR gRGB_PSaveFlag 		= root:SPHINX:SVD:gRGB_PSaveFlag
	
	WAVE RefSelect 		= root:SPHINX:SVD:ReferenceSelection
	WAVE /T RefDesc 		= root:SPHINX:SVD:ReferenceDescription
	Variable i, NRefs = DimSize(RefSelect,0)
	
	WAVE SPHINXStack	= $("root:SPHINX:Stacks:"+gStackName)
	if (WaveExists(SPHINXStack) != 1)
		print "No stack exists"
		return 0
	else
		Variable NumX = DimSize(SPHINXStack,0)
		Variable NumY = DimSize(SPHINXStack,1)
	endif
	
	String RChannel, GChannel, BChannel

	NVAR Comp0 = root:SPHINX:SVD:Comp0
	WAVE pmap0 = $("root:SPHINX:Stacks:" + gStackName + "_pMap_" + num2str(Comp0))
	if (!WaveExists(pmap0))
		print "Missing proportional map", gStackName + "_pMap_" + num2str(Comp0)
		return 0
	else
		RChannel = " 		- Red = "+gStackName + "_pMap_" + num2str(Comp0) + " = " + RefDesc[Comp0][1]
	endif
	NVAR Comp1 = root:SPHINX:SVD:Comp1
	WAVE pmap1 = $("root:SPHINX:Stacks:" + gStackName + "_pMap_" + num2str(Comp1))
	if (!WaveExists(pmap1))
		print "Missing proportional map", gStackName + "_pMap_" + num2str(Comp1)
		return 0
	else
		GChannel = " 		- Green = "+gStackName + "_pMap_" + num2str(Comp1) + " = " + RefDesc[Comp1][1]
	endif
	NVAR Comp2 = root:SPHINX:SVD:Comp2
	WAVE pmap2 = $("root:SPHINX:Stacks:" + gStackName + "_pMap_" + num2str(Comp2))
	if (!WaveExists(pmap2))
		print "Missing proportional map", gStackName + "_pMap_" + num2str(Comp2)
		return 0
	else
		BChannel = " 		- Blue = "+gStackName + "_pMap_" + num2str(Comp2) + " = " + RefDesc[Comp2][1]
	endif

	WAVE SVDMask 	= $("root:SPHINX:Stacks:"+gStackName + "_Map_mask")
	if (!WaveExists(SVDMask))
		Make /O/N=(NumX, NumY) $("root:SPHINX:Stacks:"+gStackName + "_Map_mask") /WAVE=SVDMask
		return 0
	endif
	
	if (gRGB_MaskFlag)
		String MaskDescription = ""
		if (SelectImageMask(SVDMask,gStackName,NumX,NumY,MaskDescription) == 0)
			return 0
		endif
	else
		SVDMask = 0
	endif
	
	String RGBMapName = gStackName + "_Cmp_"+Num2Str(Comp0)+"_"+Num2Str(Comp1)+"_"+Num2Str(Comp2)
	String MaskName, RGBNoMaskNAme = "RGBNoMask"
	if (gRGB_BEFlag)
		RGBMapName = RGBMapName+"_BE"
		RGBNoMaskNAme = RGBNoMaskNAme+"_BE"
	endif
	if (gRGB_MaskFlag)
		RGBMapName = RGBMapName+"_Mask_"
	endif
	String /G root:SPHINX:Stacks:gRGBMapName = RGBMapName
	
	WAVE /D RGB_Map = $("root:SPHINX:Stacks:"+RGBMapName)
	if (!waveExists(RGB_Map))
		Make /O/N=(NumX, NumY, 3)/D $("root:SPHINX:Stacks:"+RGBMapName) /WAVE=RGB_Map
		RGB_Map = NaN
	endif
	Make /FREE/N=(NumX, NumY, 3)/D RGB_NoMask
	
	if (gRGB_PSaveFlag)
		Make /FREE/D/N=(NumX,NumY) PEx0
		Make /FREE/D/N=(NumX,NumY) PEx1
		Make /FREE/D/N=(NumX,NumY) PEx2
	endif
	
	// !*!*!*! This must be one-less than the highest gray level
	Variable MaxRGB = 65535 // 65536
	
	String RefList="		Components considered for brightness enhancement:"
	
	if (gRGB_BEFlag)
		Make /FREE/D/N=(NumX, NumY) TmpScaleImage
		
		// This only considers the 3 RGB maps, but we want to scale relative to all references used in the mapping
		// MultiThread TmpScaleImage[][] 	= max(max(pmap0[p][q],pmap1[p][q]),pmap2[p][q])
	
		for (i=0;i<NRefs;i+=1)
			if (isChecked(RefSelect[i][0]))
					// Set each pixel of the temporary image to the maximum among all waves
					WAVE Image 	= $("root:SPHINX:Stacks:"+gStackName + "_pMap_" + num2str(i))
					MultiThread TmpScaleImage[][] 	= max(Image[p][q],TmpScaleImage[p][q])
					RefList = RefList + RefDesc[i][1]+";"
			endif
		endfor
		
		MultiThread RGB_Map[][][0] 	= pmap0[p][q] * (MaxRGB/TmpScaleImage[p][q])
		MultiThread RGB_Map[][][1] 	= pmap1[p][q] * (MaxRGB/TmpScaleImage[p][q])
		MultiThread RGB_Map[][][2]	= pmap2[p][q] * (MaxRGB/TmpScaleImage[p][q])
		RGB_NoMask = RGB_Map
		
		if (gRGB_PSaveFlag) // to export individual p-maps with brightness enhancement
			PEx0[][] = pmap0[p][q] * (MaxRGB/TmpScaleImage[p][q])
			PEx1[][] = pmap1[p][q] * (MaxRGB/TmpScaleImage[p][q])
			PEx2[][] = pmap2[p][q] * (MaxRGB/TmpScaleImage[p][q])
		endif
	else
		MultiThread RGB_Map[][][0] 	= pmap0[p][q] * MaxRGB
		MultiThread RGB_Map[][][1] 	= pmap1[p][q] * MaxRGB
		MultiThread RGB_Map[][][2] 	= pmap2[p][q] * MaxRGB
		
		if (gRGB_PSaveFlag) // to export individual p-maps without brightness enhancement
			PEx0[][] = pmap0[p][q] * MaxRGB
			PEx1[][] = pmap1[p][q] * MaxRGB
			PEx2[][] = pmap2[p][q] * MaxRGB
		endif
	endif
	
	if (gRGB_MaskFlag)
		// What is the best value to set masked points to? Zero or 2^16 = 65536
		MultiThread RGB_Map[][][0] 	= (SVDMask[p][q] == 0) ? RGB_Map[p][q][0] : 0
		MultiThread RGB_Map[][][1] 	= (SVDMask[p][q] == 0) ? RGB_Map[p][q][1] : 0
		MultiThread RGB_Map[][][2]	= (SVDMask[p][q] == 0) ? RGB_Map[p][q][2] : 0
	endif
	
	Print " *** Created a RGB image called",RGBMapName;print RChannel;print GChannel;print BChannel;print "		"+MaskDescription
	if (gRGB_BEFlag)
		print RefList
	endif
	
	RGBDisplayPanel(RGB_Map,RGBMapName,RGBMapName,NumX,NumY)
	
	Variable FolderSaveflag = 0 
	// Enforce saving the RGB map when the pMaps are saved as it makes folder handling easiest. 
	gRGB_RGBSaveFlag = (gRGB_PSaveFlag == 1) ? gRGB_PSaveFlag : gRGB_RGBSaveFlag
	
	if (gRGB_RGBSaveFlag && gRGB_PSaveFlag)
		NewPath /Q/O/M="Location for a new folder containing RGB-Map and P-Maps" RGBPath
		if (V_flag !=0)
			return 1
		else
			FolderSaveflag = 1
			PathInfo RGBPath
			String RGBFoldername = RGBMapName
			Prompt RGBFoldername, "Enter name of folder for saving"
			DoPrompt "RGB Export", RGBFoldername
			if (V_flag)
				return 0
			endif
			String RGBPathAndFolder = S_path + RGBFoldername
			NewPath /O/Q/C RGBPath, RGBPathAndFolder
		endif
	endif
	
	if (gRGB_RGBSaveFlag)

		// Save the RGB image with the complete User-Input name
		Print " 		Saving the RGB image as",RGBPathAndFolder
		ImageSave /O/DS=8/T="tiff"/P=RGBPath RGB_Map as (RGBFoldername+".tif")
//		ExportRGBMap(RGB_Map,RGBPathAndFolder, FolderSaveflag)
			
		if (gRGB_MaskFlag)
			// If the RGBMap is masked, save both the mask and the unmasked RGB Map with simpler names
			MaskName = "Mask_"+ReplaceString(RGBMapName,RGBFoldername,"")+".tif"
			Print " 		Saving the Mask image as",MaskName
			ImageSave /O/DS=8/T="tiff"/P=RGBPath SVDMask as MaskName
			
			RGBNoMaskNAme = RGBNoMaskNAme+".tif"
			Print " 		Saving the unmasked RGB file as",RGBNoMaskNAme
			ImageSave /DS=8/T="tiff"/P=RGBPath RGB_NoMask as RGBNoMaskNAme
		endif
	endif
	 
	 if (gRGB_PSaveFlag)
	 	String Extension = ".tif"
	 	if (gRGB_BEFlag)
	 		Extension = "_BE.tif"
	 	endif
		ImageSave /U/D=16/T="tiff"/P=RGBPath PEx0 as (NameOfWave(pmap0)+Extension)
		ImageSave /U/D=16/T="tiff"/P=RGBPath PEx1 as (NameOfWave(pmap1)+Extension)
		ImageSave /U/D=16/T="tiff"/P=RGBPath PEx2 as (NameOfWave(pmap2)+Extension)
	endif
		
	return 1
End

Function /S ExportRGBMap(RGBImage,RGBName,RGBPathflag)
	Wave RGBImage
	String RGBName
	Variable RGBPathflag
	
	if (WaveExists(RGBImage))
		if (RGBPathflag)
			ImageSave /O/DS=8/T="tiff"/P=RGBPath RGBImage as (RGBName+".tif")
		else
			ImageSave /DS=8/T="tiff" RGBImage as (RGBName+".tif")
		endif
	endif
End

Function DisplayRGBImage()

	SVAR gRGBMapName 	= root:SPHINX:Stacks:gRGBMapName
	
	String RGBName, RGBFolder 	= "root:SPHINX:Stacks"
	
	String RGBList = ReturnImageList(RGBFolder,1)
	Prompt RGBName, "List of RGB maps", popup, RGBList
	DoPrompt "Select RGB maps to view", RGBName
	if (V_flag)
		return 0
	endif
	gRGBMapName 	= RGBName
	
	WAVE RGB_Map = $("root:SPHINX:Stacks:"+gRGBMapName)
	Variable NumX = DimSize(RGB_Map,0)
	Variable NumY = DimSize(RGB_Map,1)
	
	RGBDisplayPanel(RGB_Map,gRGBMapName,gRGBMapName,NumX,NumY)
		
End




// 		Exporting the RGB as CMY looks awful! 	
//		rgb_be_name = gStackName + "_RGB_Map_BE.tiff"
//		SavePICT /C=1/E=-7 /WIN=$("RGBMapBE#StackImage")  as rgb_be_name
//		rgb_be_name = gStackName + "_CMY_Map_BE.tiff"
//		SavePICT /C=2/E=-7 /WIN=$("RGBMapBE#StackImage")  as rgb_be_name