#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// ***************************************************************************
// **************** 			Raman Spectromicroscopy
// ***************************************************************************
//
//

// *************************************************************
// ****		Load WiTec Raman stack
// *************************************************************


Function LoadWiTecRaman()

	InitializeStackBrowser()
	
	Variable NDataPts, NAxisPts, NPixels, Side, i, j, n=0
	String WiTecData, WiTecAxis, StackName, StackAxis

	String OldDf = GetDataFolder(1)
	SetDataFolder root:SPHINX:Stacks
		
		// First load the data file with N spectra
		Loadwave /A=$"data"/Q/N/O/D/G
	
		if (V_flag==1)
			StackName 	= StripSpacesAndSuffix(S_Filename,".")
			StackName 	= CleanUpDataName(StackName)
			WiTecData = StringFromList(0, S_waveNames)
			WAVE WTData 	= $WiTecData
		else
			return 0
		endif
		
		// Next load the axis file with wavenumber values
		Loadwave /A=$"axis"/Q/N/O/D/G
		if (V_flag==1)
			WiTecAxis = StringFromList(0, S_waveNames)
			WAVE WTAxis 	= $WiTecAxis
		else
			return 0
		endif
		
		NDataPts 		= DimSize($WiTecData,0)
		NAxisPts 		= DimSize($WiTecAxis,0)
		NPixels 		= NDataPts/NAxisPts
		Side 			= sqrt(NPixels)
		
		Print "NDataPts=",NDataPts,"   NAxisPts =",NAxisPts,"   NPixels=",NPixels,"   Side=",Side
		
		Make /O/U/N=(Side,Side,NAxisPts) RamanMatrix=0
		
		for (i=0;i<Side;i+=1)
			for (j=0;j<Side;j+=1)
				RamanMatrix[j][i][] 	= WTData[n*NAxisPts + r]
				n += 1
			endfor
		endfor
		
		Prompt StackName, "Name of imported stack"
		do
			DoPrompt "Please avoid odd characters and 'stack'!", StackName
			if (V_flag)
				return 0
			endif
			StackName 	= CleanUpDataName(StackName)
		while(StrSearch(StackName,"stack",0,2) > -1)
		StackAxis 	= StackName+"_axis"

		if (WaveExists($StackName) == 1)
			DoWindow /K StackBrowser
			RemoveDataFromAllWindows(StackName)
			KillWaves $StackName
		endif
		
		Rename RamanMatrix, $StackName
		Rename WTAxis, $StackAxis
//		KillWaves /Z data0, wave0
		
		DisplayStackBrowser(StackName)
		
		Print " *** Loaded a WiTec Raman stack: "+StackName+" from "+S_Filename
//		Print " 		...	 (",NX,"x",NY,") images and ",NAxis," optical wavelength values."
		
	SetDataFolder $(OldDf)
	
End
	